package fr.inria.gforge.dspace.overlay.benchmarks.analyzer.gui;

import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;

import fr.inria.gforge.dspace.overlay.benchmarks.analyzer.gui.dialogs.AbstractChartDialog;
import fr.inria.gforge.dspace.overlay.benchmarks.analyzer.gui.dialogs.AddStatementsDialog;
import fr.inria.gforge.dspace.overlay.benchmarks.analyzer.gui.dialogs.OperationsTimeComparaisonDialog;
import fr.inria.gforge.dspace.overlay.benchmarks.analyzer.gui.dialogs.StatementsDistributionDialog;
import fr.inria.gforge.dspace.overlay.benchmarks.analyzer.parser.BenchmarkBean;
import fr.inria.gforge.dspace.overlay.benchmarks.analyzer.parser.BenchmarkBean.Type;

/**
 * @author Laurent Pellegrino
 * 
 * @version $Id: BenchmarksFrame.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class BenchmarksFrame extends JFrame {

	private static final long serialVersionUID = 1L;

	private static final int NB_PEERS_COL = 0;
	
	private static final int ADD_STATEMENTS_COL = 2;
	
	private List<BenchmarkBean> beans;
	
	private List<BenchmarkBean> localBeans;
	
	private List<BenchmarkBean> remoteBeans;

	private Map<String, AbstractChartDialog> dialogs = new HashMap<String, AbstractChartDialog>();

	public BenchmarksFrame(List<BenchmarkBean> beans) {
		this.beans = beans;
	}

	public static List<BenchmarkBean> filterAndSortBenchmarksBeans(
			List<BenchmarkBean> beans, BenchmarkBean.Type type) {
		List<BenchmarkBean> filteredBeans = new ArrayList<BenchmarkBean>();
		for (BenchmarkBean bean : beans) {
			if (bean.getType() == type) {
				filteredBeans.add(bean);
			}
		}
		Collections.sort(filteredBeans);
		return filteredBeans;
	}

	public void createAndShowGUI() {
		final JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() > 1) {
					getOperationsTimeComparaisonDialog(
							tabbedPane.getSelectedIndex() == 0 
								? Type.LOCAL : Type.REMOTE).setVisible(true);
				}
			}
		});
		
		
		this.localBeans = filterAndSortBenchmarksBeans(
				this.beans, Type.LOCAL);
		if (this.localBeans.size() > 0) {
			tabbedPane.addTab("Local",
					this.createBenchmarksPanel(Type.LOCAL, this.localBeans));
			tabbedPane.setMnemonicAt(0, KeyEvent.VK_L);
		}

		this.remoteBeans = filterAndSortBenchmarksBeans(this.beans,
				Type.REMOTE);
		if (this.remoteBeans.size() > 0) {
			tabbedPane.addTab("Remote",
					this.createBenchmarksPanel(Type.REMOTE, this.remoteBeans));
			tabbedPane.setMnemonicAt(1, KeyEvent.VK_R);
		}

		super.getContentPane().add(tabbedPane);
		super.setTitle("Benchmarks Results");
		super.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		super.setSize(700, 300);
		super.setMinimumSize(super.getSize());
		super.setLocationRelativeTo(null);
		super.setVisible(true);
	}

	private JComponent createBenchmarksPanel(final Type type, final List<BenchmarkBean> beans) {
		final JTable table = new JTable(new BenchmarksTableModel(beans));
		table.setDragEnabled(false);
		table.setRowHeight(25);
		table.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() > 1) {
					int row = table.rowAtPoint(e.getPoint());
					int col = table.columnAtPoint(e.getPoint());

					switch (col) {
					case NB_PEERS_COL:
						getStatementsDistributionDialog(type, row).setVisible(true);
						break;
					case ADD_STATEMENTS_COL:
						getAddStatementsDialog(type, row).setVisible(true);
						break;
					}
				}
			}
		});
		
		return new JScrollPane(table);
	}
	
	private AbstractChartDialog getDialog(Class<?> clazz, Type type, int col, int row) {
		String id = type.toString() + "" + col + "" + row;

		if (this.dialogs.get(id) == null) {
			Constructor<?> constructor = null;
			try {
				constructor = clazz
						.getDeclaredConstructor(BenchmarkBean.class);
			} catch (SecurityException e1) {
				e1.printStackTrace();
			} catch (NoSuchMethodException e1) {
				e1.printStackTrace();
			}

			BenchmarkBean bean;
			if (type == Type.LOCAL) {
				bean = this.localBeans.get(row);
			} else {
				bean = this.remoteBeans.get(row);
			}
			
			try {
				AbstractChartDialog dialog = (AbstractChartDialog) constructor.newInstance(bean);
				dialog.create();
				this.dialogs.put(id, dialog);
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			}
		}

		return this.dialogs.get(id);
	}

	private OperationsTimeComparaisonDialog getOperationsTimeComparaisonDialog(Type type) {
		String id = type.toString() + "operationsTimeComparaisonDialog"; 
		
		if (this.dialogs.get(id) == null) {
			OperationsTimeComparaisonDialog dialog;
			
			if (type == Type.LOCAL) {
				dialog = new OperationsTimeComparaisonDialog(this.localBeans);
			} else {
				dialog = new OperationsTimeComparaisonDialog(this.remoteBeans);
			}
			
			dialog.create();
			this.dialogs.put(id, dialog);
		}
		
		return (OperationsTimeComparaisonDialog) this.dialogs.get(id);
	}
	
	private AddStatementsDialog getAddStatementsDialog(Type type, int row) {
		return (AddStatementsDialog) this.getDialog(
					AddStatementsDialog.class, type, ADD_STATEMENTS_COL, row);
	}
	
	private StatementsDistributionDialog getStatementsDistributionDialog(Type type, int row) {
		return (StatementsDistributionDialog) this.getDialog(
				StatementsDistributionDialog.class, type, NB_PEERS_COL, row);
	}

}