package fr.inria.gforge.dspace.overlay.benchmarks.analyzer.gui;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;

/**
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: FilesToAnalyzeChooser.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class FilesToAnalyzeChooser extends JFrame
									implements ActionListener {

	private static final long serialVersionUID = 1L;

	private JButton addButton, removeButton, analyzeButton;

	private JList list;

	private DefaultListModel listModel = new DefaultListModel();

	private JProgressBar progressBar;
	
	private File lastDirectoryChosenInFileChooser;
	
	private JPanel defaultPanel;
	
	public FilesToAnalyzeChooser(String[] defaultFiles) {
		this.list = new JList(this.listModel);
		this.list.setDragEnabled(false);
		JScrollPane listScrollPane = new JScrollPane(list);

		this.addButton = new JButton("Add",
							createImageIcon("/images/database_add.png"));
		this.addButton.addActionListener(this);

		this.removeButton = new JButton("Remove",
								createImageIcon("/images/database_delete.png"));
		this.removeButton.addActionListener(this);

		this.analyzeButton = new JButton("Analyze",
								createImageIcon("/images/database_go.png"));
		this.analyzeButton.addActionListener(this);
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.add(this.addButton);
		buttonPanel.add(this.removeButton);
		buttonPanel.add(this.analyzeButton);

		defaultPanel = new JPanel(new BorderLayout());
		this.defaultPanel.add(buttonPanel, BorderLayout.PAGE_START);
		this.defaultPanel.add(listScrollPane, BorderLayout.CENTER);
		
		File file;
		for (String pathToFile : defaultFiles) {
			file = new File(pathToFile);
			if (file.exists()) {
				if (file.isFile()) {
					this.listModel.addElement(file);
				} else {
					for (File f: file.listFiles()) {
						if (f.isFile()) {
							this.listModel.addElement(f);
						}
					}
				}
			}
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == this.addButton) {
			JFileChooser chooser = new JFileChooser(this.lastDirectoryChosenInFileChooser);
			chooser.setMultiSelectionEnabled(true);

			switch (chooser.showOpenDialog(this)) {
				case JFileChooser.APPROVE_OPTION:
					for (File file : chooser.getSelectedFiles()) {
						this.listModel.addElement(file);
					}
					
					if (chooser.getSelectedFiles().length > 0) {
						this.lastDirectoryChosenInFileChooser = 
							chooser.getSelectedFiles()[0].getParentFile();
					}
					break;
			}
		} else if (e.getSource() == this.removeButton) {
			while (this.list.getSelectedIndex() != -1) {
				this.listModel.remove(this.list.getSelectedIndex());
			}
		} else if (e.getSource() == this.analyzeButton) {
			if (this.listModel.size() > 0) {
				File[] selectedFiles = new File[this.listModel.size()];
				for (int i = 0; i < selectedFiles.length; i++) {
					selectedFiles[i] = (File) this.listModel.get(i);
				}

				this.progressBar = new JProgressBar(0, 100);
				this.defaultPanel.add(progressBar, BorderLayout.SOUTH);
				this.defaultPanel.revalidate();
				super.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

				this.addButton.setEnabled(false);
				this.removeButton.setEnabled(false);
				this.analyzeButton.setEnabled(false);
				
				SelectedFilesParserTask task = 
					new SelectedFilesParserTask(this, selectedFiles);
				task.execute();
			} else {
				JOptionPane.showMessageDialog(
						this, 
						"No file to analyze.",
						"Message",
						JOptionPane.INFORMATION_MESSAGE);

			}
		}
	}

	protected static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = FilesToAnalyzeChooser.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		} else {
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}

	public void createAndShowGUI() {
		super.setTitle("SOA4ALL Space Benchmarks Analyzer");
		super.add(this.defaultPanel);
		super.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		super.setSize(700, 300);
		super.setMinimumSize(super.getSize());
		super.setLocationRelativeTo(null);
		super.setVisible(true);
	}

	public JProgressBar getProgressBar() {
		return this.progressBar;
	}

	public JButton getAddButton() {
		return this.addButton;
	}

	public JButton getRemoveButton() {
		return this.removeButton;
	}

	public JButton getAnalyzeButton() {
		return this.analyzeButton;
	}

	public DefaultListModel getListModel() {
		return this.listModel;
	}

}
