package fr.inria.gforge.dspace.overlay.benchmarks.analyzer.gui;

import java.awt.Cursor;
import java.awt.Toolkit;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicInteger;

import javax.swing.JOptionPane;
import javax.swing.SwingWorker;

import fr.inria.gforge.dspace.overlay.benchmarks.analyzer.parser.BenchmarkBean;
import fr.inria.gforge.dspace.overlay.benchmarks.analyzer.parser.BenchmarkParser;

/**
 * 
 * @author Laurent Pellegrino
 *
 * @version $Id: SelectedFilesParserTask.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class SelectedFilesParserTask extends SwingWorker<List<BenchmarkBean>, Void> {

	private File[] selectedFiles;
	
	private FilesToAnalyzeChooser chooser;
	
	private List<File> invalidFiles = Collections.synchronizedList(new ArrayList<File>());
	
	public SelectedFilesParserTask(final FilesToAnalyzeChooser chooser, File[] selectedFiles) {
		this.chooser = chooser;
		this.selectedFiles = selectedFiles;
		
		addPropertyChangeListener(new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt) {
                if("progress".equals(evt.getPropertyName())) {
                    chooser.getProgressBar().setValue((Integer) evt.getNewValue());
                }
            }
        });
	}
	
	@Override
	protected List<BenchmarkBean> doInBackground() throws Exception {
		ExecutorService threadsPool = Executors.newFixedThreadPool(Runtime.getRuntime().availableProcessors() * 4);
		final List<BenchmarkBean> beans = new ArrayList<BenchmarkBean>();
		
		final AtomicInteger nbFilesHandled = new AtomicInteger();
		final int value = 100 / this.selectedFiles.length;
		
		final CountDownLatch doneSignal = new CountDownLatch(this.selectedFiles.length);
		for (final File file : this.selectedFiles){
			threadsPool.execute(new Runnable() {
				@Override
				public void run() {
					try {
						beans.add(BenchmarkParser.parse(file));
						setProgress(value * nbFilesHandled.incrementAndGet());
					} catch (Exception e) {
						invalidFiles.add(file);
					}
					doneSignal.countDown();
				}
			});
		}
		
		doneSignal.await();
		threadsPool.shutdown();
		
		return beans;
	}
	
    @Override
    public void done() {
        Toolkit.getDefaultToolkit().beep();
        this.chooser.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        
        List<BenchmarkBean> beans = null;
        try {
        	beans = super.get();
        } catch (InterruptedException e) {
			e.printStackTrace();
		} catch (ExecutionException e) {
			e.printStackTrace();
		}
		
		if (this.invalidFiles.size() > 0) {
			StringBuffer buf = new StringBuffer();
			buf.append("The following files have invalid syntax and will be ignored:\n");
			for (File file : invalidFiles) {
				buf.append("  - ");
				buf.append(file.toString());
				buf.append("\n");
			}
			
			JOptionPane.showMessageDialog(
					chooser, 
					buf.toString(),
					"Information",
					JOptionPane.INFORMATION_MESSAGE);
			this.invalidFiles.clear();
		}
		
		if (beans.isEmpty()) {
			this.chooser.getAddButton().setEnabled(true);
			this.chooser.getRemoveButton().setEnabled(true);
			this.chooser.getAnalyzeButton().setEnabled(true);
			this.chooser.getListModel().clear();
		} else {
			this.chooser.getProgressBar().setValue(100);
			this.chooser.setVisible(false);
			new BenchmarksFrame(beans).createAndShowGUI();
		}
    }

}
