package fr.inria.gforge.dspace.overlay.benchmarks.analyzer.gui.charts;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.util.ShapeUtilities;

import fr.inria.gforge.dspace.overlay.benchmarks.analyzer.parser.BenchmarkBean;
import fr.inria.gforge.dspace.overlay.benchmarks.measurement.AddStatementMeasurementResults;

/**
 * @author Laurent Pellegrino
 * 
 * @version $Id: AddStatementsDistributionTimeChart.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class AddStatementsDistributionTimeChart extends AbstractChart {

	public AddStatementsDistributionTimeChart(BenchmarkBean bean) {
		super(bean);
	}

	public ChartPanel create() {
		XYSeries serie = new XYSeries("Time to add one statement in ms");

		for (AddStatementMeasurementResults result : 
				this.bean.getAddRandomStatementsMeasurementResults().
				getMeasurementsResults()) {
			serie.add(
					result.getStartTime() - 
						this.bean.getAddRandomStatementsMeasurementResults().
							getMeasurementsResults().get(0).getStartTime(), 
					result.getExecutionTime(), 
					false);
		}

		XYSeriesCollection dataset = new XYSeriesCollection(serie);
		final JFreeChart chart = ChartFactory.createXYLineChart(
				"Distribution by time (average is "
				+ this.bean.getAddStatementsTimeAverage() + " ms)",
				"Operation start time", "Time to add one statement in ms",
				dataset, PlotOrientation.VERTICAL, true, false, false);
		chart.setBackgroundPaint(new Color(246, 246, 246));

		final XYPlot plot = chart.getXYPlot();
		plot.setBackgroundPaint(Color.WHITE);
		plot.setDomainGridlinePaint(Color.LIGHT_GRAY);
		plot.setRangeGridlinePaint(Color.LIGHT_GRAY);
		plot.getRenderer().setSeriesStroke(0,
				new BasicStroke(1.0f, BasicStroke.CAP_ROUND,
						BasicStroke.CAP_ROUND));

		XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
		renderer.setSeriesLinesVisible(0, false);
		renderer.setSeriesShape(0, ShapeUtilities.createRegularCross(1.0f, 1.0f));
		plot.setRenderer(renderer);

		ChartPanel chartPanel = new ChartPanel(chart);
		chartPanel.setPreferredSize(new Dimension(700, 350));
		return chartPanel;
	}

}
