package fr.inria.gforge.dspace.overlay.benchmarks.analyzer.gui.charts;

import java.awt.Color;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.util.ShapeUtilities;

import fr.inria.gforge.dspace.overlay.benchmarks.analyzer.parser.BenchmarkBean;
import fr.inria.gforge.dspace.overlay.benchmarks.measurement.AddStatementMeasurementResults;

/**
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: AddStatementsNbHopsChart.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class AddStatementsNbHopsChart extends AbstractChart {

	private static final long serialVersionUID = 1L;

	public AddStatementsNbHopsChart(BenchmarkBean bean) {
		super(bean);
	}

	public ChartPanel create() {
		XYSeries singleTripSerie = new XYSeries("Nb of hops for single trip");
		XYSeries returnTripSerie = new XYSeries("Nb of hops for return trip");
		long first = super.bean.getAddRandomStatementsMeasurementResults().
		getMeasurementsResults().get(0).getStartTime();
		for (AddStatementMeasurementResults result : 
			this.bean.getAddRandomStatementsMeasurementResults().
			getMeasurementsResults()) {
			singleTripSerie.add(result.getStartTime() - first,
					result.getNbHopsSingleTrip(), false);
			returnTripSerie.add(result.getStartTime() - first,
					result.getNbHopsReturnTrip(), false);
		}

		XYSeriesCollection dataset = new XYSeriesCollection();
		dataset.addSeries(singleTripSerie);
		dataset.addSeries(returnTripSerie);
		
		final JFreeChart chart = ChartFactory.createXYLineChart(
				"Nb of hops by call (average singleTrip=" +
				super.bean.getAddStatementsSingleTripAverage() +
				", returnTrip=" + super.bean.getAddStatementsReturnTripAverage()
				+ ")", "Operation start time", "Nb hops", dataset,
				PlotOrientation.VERTICAL, true, false, false);
		chart.setBackgroundPaint(new Color(246, 246, 246));
		
		final XYPlot plot = chart.getXYPlot();
		plot.setBackgroundPaint(Color.WHITE);
		plot.setDomainGridlinePaint(Color.LIGHT_GRAY);
		plot.setRangeGridlinePaint(Color.LIGHT_GRAY);
		
		final XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
		renderer.setSeriesLinesVisible(0, false);
		renderer.setSeriesLinesVisible(1, false);
		renderer.setSeriesShape(0,
				ShapeUtilities.createDiamond(1.0f));
		renderer.setSeriesShape(1,
				ShapeUtilities.createRegularCross(4.0f, 0.1f));
		plot.setRenderer(renderer);
		
		final ChartPanel chartPanel = new ChartPanel(chart);
		chartPanel.setPreferredSize(new java.awt.Dimension(700, 350));
		return chartPanel;
	}
	
}
