package fr.inria.gforge.dspace.overlay.benchmarks.analyzer.gui.charts;

import java.awt.Color;
import java.util.List;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.util.ShapeUtilities;

import fr.inria.gforge.dspace.overlay.benchmarks.analyzer.parser.BenchmarkBean;

/**
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: OperationsTimeComparaisonChart.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class OperationsTimeComparaisonChart extends AbstractChart {

	private static final long serialVersionUID = 1L;

	private List<BenchmarkBean> beans;
	
	public OperationsTimeComparaisonChart(List<BenchmarkBean> beans) {
		super(beans.get(0));
		this.beans = beans;
	}

	public ChartPanel create() {
		XYSeries addStatementsSerie = new XYSeries("Time to perform add statements");
		XYSeries queryAllSerie = new XYSeries("Time to perform query all");
		XYSeries theoricalSerie = new XYSeries("Theorical (3/4 * nbPeers^(1/3))");
		
		int scale = 1;
		
		for (int i=1; i<getNumberOfNumbers(
				(int) Math.max(this.beans.get(this.beans.size()-1).getTasksTimeMeasurementResults().getMeasurementsResults().get(1).getExecutionTime(),
						 this.beans.get(this.beans.size()-1).getTasksTimeMeasurementResults().getMeasurementsResults().get(2).getExecutionTime()))
						;i++) {
			scale *= 10;
		}
		
		for (BenchmarkBean bean : this.beans) {
			addStatementsSerie.add(
					bean.getNbPeers(), 
					bean.getTasksTimeMeasurementResults().getMeasurementsResults().get(1).getExecutionTime(),
					false);
			queryAllSerie.add(
					bean.getNbPeers(), 
					bean.getTasksTimeMeasurementResults().getMeasurementsResults().get(2).getExecutionTime(),
					false);
			theoricalSerie.add(
					bean.getNbPeers(),
					3/4.0 * Math.pow(bean.getNbPeers(), (1 / 3.0)) * scale,
					false);
					
		}

		XYSeriesCollection dataset = new XYSeriesCollection();
		dataset.addSeries(addStatementsSerie);
		dataset.addSeries(queryAllSerie);
		dataset.addSeries(theoricalSerie);
		
		final JFreeChart chart = ChartFactory.createXYLineChart(
				"Operations time comparaison by peers",
				"Nb peers", "Operation time in ms",
				dataset,
				PlotOrientation.VERTICAL, true, false, false);
		chart.setBackgroundPaint(new Color(246, 246, 246));
		
		final XYPlot plot = chart.getXYPlot();
		plot.setBackgroundPaint(Color.WHITE);
		plot.setDomainGridlinePaint(Color.LIGHT_GRAY);
		plot.setRangeGridlinePaint(Color.LIGHT_GRAY);
		
		final XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
		renderer.setSeriesLinesVisible(0, true);
		renderer.setSeriesLinesVisible(1, true);
		renderer.setSeriesShape(0,
				ShapeUtilities.createDiamond(2.0f));
		renderer.setSeriesShape(1,
				ShapeUtilities.createDiamond(2.0f));
		renderer.setSeriesShape(2,
				ShapeUtilities.createDiamond(2.0f));
		plot.setRenderer(renderer);
		
		final ChartPanel chartPanel = new ChartPanel(chart);
		chartPanel.setPreferredSize(new java.awt.Dimension(700, 350));
		return chartPanel;
	}
	
	private static int getNumberOfNumbers(int n) {
		int nb = 1;
		
		while ((n = (n/10)) > 0) {
			nb++;
		}
		
		return nb;
	}
	
}
