package fr.inria.gforge.dspace.overlay.benchmarks.analyzer.gui.dialogs;

import java.awt.event.KeyEvent;

import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;

import fr.inria.gforge.dspace.overlay.benchmarks.analyzer.gui.charts.AddStatementsDistributionTimeChart;
import fr.inria.gforge.dspace.overlay.benchmarks.analyzer.gui.charts.AddStatementsNbHopsChart;
import fr.inria.gforge.dspace.overlay.benchmarks.analyzer.parser.BenchmarkBean;
import fr.inria.gforge.dspace.overlay.benchmarks.measurement.AddStatementMeasurementResults;

/**
 * @author Laurent Pellegrino
 * 
 * @version $Id
 */
public class AddStatementsDialog extends AbstractChartDialog {

	private static final long serialVersionUID = 1L;

	public AddStatementsDialog(BenchmarkBean bean) {
		super(bean);
	}

	public void create() {
		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.addTab("Distribution by time", 
				new AddStatementsDistributionTimeChart(super.bean).create());
		tabbedPane.setMnemonicAt(0, KeyEvent.VK_D);
		tabbedPane.addTab("Hops by call", 
				new AddStatementsNbHopsChart(super.bean).create());
		tabbedPane.setMnemonicAt(1, KeyEvent.VK_H);
		tabbedPane.addTab("Values", new JScrollPane(this.createValuesTable()));
		tabbedPane.setMnemonicAt(1, KeyEvent.VK_V);
		
		StringBuffer title = new StringBuffer();
		title.append("AddStatement operation with ");
		title.append(super.bean.getNbPeers());
		title.append(" peer(s) and ");
		title.append(super.bean.getAddRandomStatementsMeasurementResults().
						getMeasurementsResults().size());
		title.append(" statement(s)");
		
		super.setTitle(title.toString());
		super.add(tabbedPane);
		super.pack();
		super.setMinimumSize(super.getPreferredSize());
		super.setLocationByPlatform(true);
	}
	
	private JTable createValuesTable() {
		String[] columnsNames = {
				"startTime",
				"endTime",
				"duration",
				"nbHopsSingleTrip",
				"nbHopsReturnTrip"
		};
		
		Integer[][] data = new Integer[super.bean.getAddRandomStatementsMeasurementResults().
		       						getMeasurementsResults().size()][5];
		
		int i=0;
		for (AddStatementMeasurementResults result: 
				super.bean.getAddRandomStatementsMeasurementResults().
					getMeasurementsResults()) {
			data[i][0] = (int) (result.getStartTime() - 
					super.bean.getAddRandomStatementsMeasurementResults().
						getMeasurementsResults().get(0).getStartTime());
			data[i][1] = (int) data[i][0] + result.getExecutionTime();
			data[i][2] = (int) result.getExecutionTime();
			data[i][3] = (int) result.getNbHopsSingleTrip();
			data[i][4] = (int) result.getNbHopsReturnTrip();
			i++;
		}
		
		final JTable table = new JTable(data, columnsNames);
		table.setDragEnabled(false);
		table.setRowHeight(25);
		table.setEnabled(false);
		return table;
	}

}
