package fr.inria.gforge.dspace.overlay.benchmarks.analyzer.parser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import fr.inria.gforge.dspace.overlay.benchmarks.analyzer.parser.BenchmarkBean.Type;
import fr.inria.gforge.dspace.overlay.benchmarks.measurement.AddStatementsMeasurementResults;
import fr.inria.gforge.dspace.overlay.benchmarks.measurement.PeersMeasurementResults;
import fr.inria.gforge.dspace.overlay.benchmarks.measurement.QueryMeasurementResults;
import fr.inria.gforge.dspace.overlay.benchmarks.measurement.TasksTimeMeasurementResults;

/**
 * @author Laurent Pellegrino
 * 
 * @version $Id: BenchmarkParser.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class BenchmarkParser {

	public static BenchmarkBean parse(File file)
			throws InvalidSyntaxFileException {
		BufferedReader reader = null;
		try {
			reader = new BufferedReader(new FileReader(file));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}

		Type type;
		int nbPeers = 0;
		TasksTimeMeasurementResults tasksTimeMeasurementResults;
		AddStatementsMeasurementResults addRandomStatementsMeasurementResults;
		QueryMeasurementResults queryAllMeasurementResults;
		PeersMeasurementResults peersMeasurementResults;

		try {
			type = parseType(reader);
			nbPeers = parseNbPeers(reader);
			
			reader.readLine();
			reader.readLine();
			
			tasksTimeMeasurementResults = new TasksTimeMeasurementResults();
			tasksTimeMeasurementResults.readResults(reader);
			
			reader.readLine();
			
			addRandomStatementsMeasurementResults = new AddStatementsMeasurementResults();
			addRandomStatementsMeasurementResults.readResults(reader);
			
			reader.readLine();
			
			queryAllMeasurementResults = new QueryMeasurementResults();
			queryAllMeasurementResults.readResults(reader);
			
			reader.readLine();
			
			peersMeasurementResults = new PeersMeasurementResults();
			peersMeasurementResults.readResults(reader);
		} catch (Exception e) {
			throw new InvalidSyntaxFileException(file);
		} finally {
			try {
				reader.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		return new BenchmarkBean(type, 
								 nbPeers, 
								 tasksTimeMeasurementResults, 
								 addRandomStatementsMeasurementResults,
								 queryAllMeasurementResults,
								 peersMeasurementResults);
	}

	private static Type parseType(BufferedReader bufferedReader)
			throws IOException {
		return bufferedReader.readLine().split(" ")[1].equals("local") ? 
				Type.LOCAL : Type.REMOTE;
	}

	private static int parseNbPeers(BufferedReader bufferedReader)
			throws NumberFormatException, IOException {
		return Integer.parseInt(bufferedReader.readLine().split(" ")[1]);
	}
	
	public static void main(String[] args) {
		BenchmarkBean res = null;
		try {
			res = BenchmarkParser.parse(new File(args[0]));
		} catch (InvalidSyntaxFileException e) {
			e.printStackTrace();
		}
		
		System.out.println(res.toString());
	}
	
}
