package fr.inria.gforge.dspace.overlay.benchmarks;

import java.io.File;

import org.objectweb.proactive.api.PALifeCycle;

import fr.inria.gforge.dspace.overlay.benchmarks.space.SpaceNetworkBenchmarksDistributed;
import fr.inria.gforge.dspace.overlay.benchmarks.space.SpaceNetworkBenchmarksSingleMachine;

/**
 * Launches overlay benchmarks. The benchmarks can be run on the local machine
 * or by using a GCMA descriptor file to deployment the active objects on
 * different machines.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: BenchmarkLauncher.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class BenchmarkLauncher {

    public static final String DEFAULT_GCMA_DESCRIPTOR = 
    		System.getProperty("dspace.bundle.home")
            	+ File.separator + "deployment" + File.separator + "GCMA.xml";

    public static void main(String[] args) {
        new BenchmarkLauncher().parseArguments(args);
    }

    public void parseArguments(String[] args) {
        if (args.length == 0) {
            help();
        }

        if (args[0].equalsIgnoreCase("-sl")) {
            this.parseArgumentsForSL(this.subArray(args, 1));
        } else if (args[0].equalsIgnoreCase("-sd")) {
            this.parseArgumentsForSD(this.subArray(args, 1));
        } else if (args[0].equalsIgnoreCase("-gl")) {
            this.parseArgumentsForGL(this.subArray(args, 1));
        } else if (args[0].equalsIgnoreCase("-gd")) {
            this.parseArgumentsForGD(this.subArray(args, 1));
        } else if (args[0].equalsIgnoreCase("-help")) {
            help();
        } else {
            help();
        }
    }

    private String[] subArray(String array[], int start) {
        assert start > 0 && start < array.length;

        String[] newArray = new String[array.length - start];
        for (int i = start; i < array.length; i++) {
            newArray[i - start] = array[i];
        }

        return newArray;
    }

    private void parseArgumentsForSL(String[] args) {
        if (args.length == 2) {
            try {
                int nbPeers = Integer.parseInt(args[0]);
                int nbAddStatementOperations = Integer.parseInt(args[1]);
                Benchmark bench = new SpaceNetworkBenchmarksSingleMachine(
                							nbPeers, nbAddStatementOperations);
                bench.run(true);
                PALifeCycle.exitSuccess();
            } catch (NumberFormatException e) {
                help();
            }
        } else {
            help();
        }
    }

    private void parseArgumentsForSD(String[] args) {
        Benchmark bench = null;
        if (args.length == 2) {
            try {
                bench = new SpaceNetworkBenchmarksDistributed(
                			new File(DEFAULT_GCMA_DESCRIPTOR), 
                            Integer.parseInt(args[0]), 
                            Integer.parseInt(args[1]));
            } catch(NumberFormatException e) {
                help();
            }
        } else {
            help();
        }
        bench.run(true);
        PALifeCycle.exitSuccess();
    }

    private void parseArgumentsForGL(String[] args) {
        System.err.println("Not yet implemented.");
    }

    private void parseArgumentsForGD(String[] args) {
        System.err.println("Not yet implemented.");
    }

    public static void help() {
        StringBuffer buf = new StringBuffer();
        buf.append("Usage: java ");
        buf.append(BenchmarkLauncher.class.getCanonicalName());
        buf.append(" -SL <nbPeers> <nbStatementsToAdd>\n");
        buf.append("       java ");
        buf.append(BenchmarkLauncher.class.getCanonicalName());
        buf.append(" -SD <nbPeers> <pathToRdfFileToLoad|nbStatementsToAdd>\n");
        buf.append("       java ");
        buf.append(BenchmarkLauncher.class.getCanonicalName());
        buf.append(" -GL <nbChordPeers> <nbSpaces> [nbStatementsToAddPerSpace]\n");
        buf.append("       java ");
        buf.append(BenchmarkLauncher.class.getCanonicalName());
        buf.append(" -GD [pathToGCMA]\n");
        buf.append("       java ");
        buf.append(BenchmarkLauncher.class.getCanonicalName());
        buf.append(" -help\n");

        System.err.println(buf.toString());
        System.exit(1);
    }

}
