package fr.inria.gforge.dspace.overlay.benchmarks.measurement;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;

/**
 * @author Laurent Pellegrino
 * 
 * @version $Id: AddStatementMeasurementResults.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class AddStatementMeasurementResults 
						extends AbstractMeasurementResults 
						implements Comparable<AddStatementMeasurementResults> {

    private long startTime;

    private long endTime;

    private int nbHopsSingleTrip;

    private int nbHopsReturnTrip;

    private int executionTime;
    
    public AddStatementMeasurementResults() {
    	super();
    }
    
    public AddStatementMeasurementResults(String associatedTaskSimpleName) {
    	super(associatedTaskSimpleName);
    }
    
    public AddStatementMeasurementResults(
    		long startTime, 
    		long endTime, 
    		int nbHopsSingleTrip,
            int nbHopsReturnTrip) {
        this.startTime = startTime;
        this.endTime = endTime;
        this.executionTime = (int)(endTime - startTime);
        this.nbHopsSingleTrip = nbHopsSingleTrip;
        this.nbHopsReturnTrip = nbHopsReturnTrip;
    }

    public AddStatementMeasurementResults(
    		long startTime, 
    		int executionTime, 
    		int nbHopsSingleTrip,
            int nbHopsReturnTrip) {
        this.startTime = startTime;
        this.endTime = startTime + executionTime;
        this.executionTime = executionTime;
        this.nbHopsSingleTrip = nbHopsSingleTrip;
        this.nbHopsReturnTrip = nbHopsReturnTrip;
    }
    
    public int getExecutionTime() {
        return this.executionTime;
    }

    public long getStartTime() {
        return this.startTime;
    }

    public long getEndTime() {
        return this.endTime;
    }

    public int getNbHopsSingleTrip() {
        return this.nbHopsSingleTrip;
    }

    public int getNbHopsReturnTrip() {
        return this.nbHopsReturnTrip;
    }

    public int compareTo(AddStatementMeasurementResults result) {
        if (this.startTime < result.startTime) {
            return -1;
        } else if (this.startTime > result.startTime) {
            return 1;
        } else {
            return 0;
        }
    }
    
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("startTime=");
        buf.append(this.startTime);
        buf.append(", endTime=");
        buf.append(this.endTime);
        buf.append(", duration=");
        buf.append(this.executionTime);
        buf.append(", nbHopsForRoundTrip=");
        buf.append(this.nbHopsSingleTrip);
        buf.append(", nbHopsForReturnTrip=");
        buf.append(this.nbHopsReturnTrip);

        return buf.toString();
    }

	protected void write(BufferedWriter writer) throws IOException {
		writer.write(Long.toString(this.startTime));
		writer.write(' ');
		writer.write(Long.toString(this.endTime));
		writer.write(' ');
		writer.write(Integer.toString(this.getNbHopsSingleTrip()));
		writer.write(' ');
		writer.write(Integer.toString(this.getNbHopsReturnTrip()));
		writer.newLine();
	}

	protected void read(BufferedReader reader) throws IOException {
		String[] line = reader.readLine().split(" ");
		this.startTime = Long.parseLong(line[0]);
		this.endTime = Long.parseLong(line[1]);
		this.executionTime = (int) (this.endTime - this.startTime);
		this.nbHopsSingleTrip = Integer.parseInt(line[2]);
		this.nbHopsReturnTrip = Integer.parseInt(line[3]);
	}

}
