package fr.inria.gforge.dspace.overlay.benchmarks.space;

import java.io.BufferedWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.objectweb.proactive.core.util.ProActiveRandom;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.ontoware.rdf2go.model.Statement;

import fr.inria.gforge.dspace.overlay.benchmarks.Benchmark;
import fr.inria.gforge.dspace.overlay.benchmarks.tasks.AbstractTask;
import fr.inria.gforge.dspace.overlay.benchmarks.tasks.space.AddStatementsTask;
import fr.inria.gforge.dspace.overlay.benchmarks.tasks.space.GetNbStatementsByPeerTask;
import fr.inria.gforge.dspace.overlay.benchmarks.tasks.space.QueryAllTask;
import fr.inria.gforge.dspace.overlay.benchmarks.tasks.space.QueryWithFixedSubjectPredicateTask;
import fr.inria.gforge.dspace.overlay.benchmarks.tasks.space.QueryWithFixedSubjectTask;
import fr.inria.gforge.dspace.overlay.initializers.SpaceNetworkInitializer;
import fr.inria.gforge.dspace.overlay.util.RDF2GoBuilder;

/**
 * 
 * @author Laurent Pellegrino
 *
 * @version $Id: AbstractSpaceNetworkBenchmarks.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public abstract class AbstractSpaceNetworkBenchmarks extends Benchmark {

    protected SpaceNetworkInitializer initializer;

    protected int nbPeers = 10;

    protected int nbStatementsToAdd = 1000;

    protected List<Statement> statementsToAdd;

    protected List<Peer> peersReferences;

    public AbstractSpaceNetworkBenchmarks(int nbPeers, int nbStatementToAdd) {
        this.nbPeers = nbPeers;
        this.nbStatementsToAdd = nbStatementToAdd;
        this.initializer = new SpaceNetworkInitializer(
                				RDF2GoBuilder.createURI("http://www.inria.fr"));
    }
    public List<AbstractTask> getTasksToExecute() {
    	List<AbstractTask> tasks = new ArrayList<AbstractTask>();
//    	try {
//			tasks.add(new AddBSBMStatementsTask(this, true));
//		} catch (RDFParseException e) {
//			e.printStackTrace();
//		} catch (RDFHandlerException e) {
//			e.printStackTrace();
//		} catch (IOException e) {
//			e.printStackTrace();
//		} catch (URISyntaxException e) {
//			e.printStackTrace();
//		}
//		tasks.add(new QueryAllTask(this));
//    	tasks.add(new BSBMCustomQuery1Task(this));
//    	tasks.add(new BSBMCustomQuery2Task(this));
//    	tasks.add(new BSBMCustomQuery3Task(this));
//    	tasks.add(new BSBMCustomQuery4Task(this));
//    	tasks.add(new BSBMQueriesInParallel(this));
    	tasks.add(new AddStatementsTask(this, true, this.nbStatementsToAdd));
    	tasks.add(new QueryAllTask(this));
    	tasks.add(new QueryWithFixedSubjectPredicateTask(this));
    	tasks.add(new QueryWithFixedSubjectTask(this));
    	tasks.add(new GetNbStatementsByPeerTask(this));
    	return tasks;
    }

    public String getOutputFilename() {
        Calendar calendar = Calendar.getInstance();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd-HH-mm-ss");
        StringBuffer buf = new StringBuffer();
        buf.append(this.getClass().getSimpleName());
        buf.append("-");
        buf.append(this.nbPeers);
        buf.append("-");
        buf.append(sdf.format(calendar.getTime()));
        
        return buf.toString();
    }
    
    protected void writeHeader(BufferedWriter writer) throws IOException {
    	writer.write("type ");
    	writer.write(this.initializer.isLocalSetup() ? "local" : "remote");
    	writer.newLine();
    	writer.write("nbPeers ");
        writer.write(Integer.toString(this.nbPeers));
        writer.newLine();
        writer.newLine();
    }

    public Peer getRandomPeer() {
        return this.peersReferences.get(ProActiveRandom.nextInt(this.peersReferences.size()));
    }

    public int getNbPeers() {
        return this.nbPeers;
    }

    public int getNbStatementsToAdd() {
        return this.nbStatementsToAdd;
    }

    public List<Peer> getPeersReferences() {
		return peersReferences;
	}
	public void setPeersReferences(List<Peer> peersReferences) {
		this.peersReferences = peersReferences;
	}
	public SpaceNetworkInitializer getSpaceNetworkInitializer() {
        return this.initializer;
    }

}
