package fr.inria.gforge.dspace.overlay.benchmarks.tasks.space;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CountDownLatch;

import org.objectweb.proactive.api.PAFuture;
import org.objectweb.proactive.extensions.structuredp2p.messages.response.can.ForwardResponseMessage;
import org.ontoware.rdf2go.model.Statement;

import fr.inria.gforge.dspace.overlay.benchmarks.measurement.AddStatementMeasurementResults;
import fr.inria.gforge.dspace.overlay.benchmarks.measurement.AddStatementsMeasurementResults;
import fr.inria.gforge.dspace.overlay.benchmarks.space.AbstractSpaceNetworkBenchmarks;
import fr.inria.gforge.dspace.overlay.benchmarks.tasks.AbstractTask;
import fr.inria.gforge.dspace.overlay.messages.query.can.AddStatementQueryMessage;
import fr.inria.gforge.dspace.overlay.util.SemanticHelper;

/**
 * @author Laurent Pellegrino
 * 
 * @version $Id: AddStatementsTask.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class AddStatementsTask extends AbstractTask {

	private boolean inParallel = false;

	private List<Statement> statements;

	private AddStatementsMeasurementResults measurementResults =
			new AddStatementsMeasurementResults(this.getClass().getSimpleName()); 
	
	public AddStatementsTask(
			AbstractSpaceNetworkBenchmarks benchmark, 
			boolean inParallel,
			List<Statement> statements) {
		this(benchmark, inParallel);
		this.statements = statements;
	}

	public AddStatementsTask(
			AbstractSpaceNetworkBenchmarks benchmark, 
			boolean inParallel,
			int nbRandomStatements) {
		this(benchmark, inParallel);
		this.statements = new ArrayList<Statement>();
		for (int i = 0; i < nbRandomStatements; i++) {
			this.statements.add(
				SemanticHelper.generateRandomStatement(
						benchmark.getSpaceNetworkInitializer().getSpaceURI()));
		}
	}

	private AddStatementsTask(AbstractSpaceNetworkBenchmarks benchmark, 
			boolean inParallel) {
		super(benchmark);
		this.inParallel = inParallel;
	}
	
	public void runInParallel() {
		final CountDownLatch doneSignal = 
			new CountDownLatch(this.statements.size());

		for (int i = 0; i < this.statements.size(); i++) {
			final int index = i;
			super.benchmark.getThreadsPool().execute(new Runnable() {
				public void run() {
					executeAndMeasureResults(index);
					doneSignal.countDown();
				}
			});
		}

		try {
			doneSignal.await();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	private void runInSequential() {
		for (int i = 0; i < this.statements.size(); i++) {
			this.executeAndMeasureResults(i);
		}
	}

	private void executeAndMeasureResults(final int index) {
		long startTime = System.currentTimeMillis();
		ForwardResponseMessage response = performAddStatement(statements.get(index));
		long endTime = System.currentTimeMillis();

		synchronized (measurementResults) {
			 measurementResults.add(
					 new AddStatementMeasurementResults(
							 startTime, 
							 endTime, 
							 response.getOutboundHopCount(),
							 response.getReturnHopCount()));
		 }
	}
	
	private ForwardResponseMessage performAddStatement(Statement stmt) {
		return (ForwardResponseMessage) PAFuture.getFutureValue(
				super.benchmark.getRandomPeer().send(
						new AddStatementQueryMessage(
								super.benchmark.getSpaceNetworkInitializer().getSpaceURI(),
								stmt)));
	}

	public void run() {
		if (this.inParallel) {
			this.runInParallel();
		} else {
			this.runInSequential();
		}
		super.benchmark.add(this.measurementResults);
	}

}
