package fr.inria.gforge.dspace.overlay.benchmarks.tasks.space;

import org.objectweb.proactive.api.PAFuture;
import org.objectweb.proactive.extensions.structuredp2p.exceptions.DispatchException;
import org.ontoware.rdf2go.model.QueryResultTable;
import org.ontoware.rdf2go.model.QueryRow;

import fr.inria.gforge.dspace.overlay.api.messages.query.ConstructQuery;
import fr.inria.gforge.dspace.overlay.api.messages.query.SelectQuery;
import fr.inria.gforge.dspace.overlay.api.messages.response.ConstructResponse;
import fr.inria.gforge.dspace.overlay.api.messages.response.SelectResponse;
import fr.inria.gforge.dspace.overlay.benchmarks.measurement.QueryMeasurementResults;
import fr.inria.gforge.dspace.overlay.benchmarks.space.AbstractSpaceNetworkBenchmarks;
import fr.inria.gforge.dspace.overlay.benchmarks.tasks.AbstractTask;
import fr.inria.gforge.dspace.overlay.util.SemanticHelper;

/**
 * @author Laurent Pellegrino
 * 
 * @version $Id: QueryWithFixedSubjectPredicateTask.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class QueryWithFixedSubjectPredicateTask extends AbstractTask {

	public QueryWithFixedSubjectPredicateTask(AbstractSpaceNetworkBenchmarks benchmark) {
		super(benchmark);
	}
	
	public void run() {
		StringBuffer query = new StringBuffer();
		try {
			SelectResponse response = (SelectResponse) PAFuture.getFutureValue(
					super.benchmark.getRandomPeer().send(
							new SelectQuery(
								super.benchmark.getSpaceNetworkInitializer().getSpaceURI(),
								"SELECT ?s ?p WHERE { ?s ?p ?o } LIMIT 1")));
			
			QueryResultTable resultTable = response.getResult();
			QueryRow row = resultTable.iterator().next();
			String subject = "<" + row.getValue("s") + ">";
			String predicate = "<" + row.getValue("p") + ">";
			
			query.append("CONSTRUCT { ");
			query.append(subject);
			query.append(" ");
			query.append(predicate);
			query.append(" ?o } WHERE { ");
			query.append(subject);
			query.append(" ");
			query.append(predicate);
			query.append(" ?o }");
		} catch (DispatchException e) {
			e.printStackTrace();
		}
		
		try {
			ConstructResponse response = (ConstructResponse) PAFuture.getFutureValue(
					super.benchmark.getRandomPeer().send(
							new ConstructQuery(
								super.benchmark.getSpaceNetworkInitializer().getSpaceURI(),
								query.toString())));
			
			super.benchmark.add(
					new QueryMeasurementResults(
							this.getClass().getSimpleName(),
							response.getNbStepsForSingleTrip(),
							response.getNbStepsForReturnTrip(),
							response.getLatency(),
							SemanticHelper.size(response.getResult())));
		} catch (DispatchException e) {
			e.printStackTrace();
		}
	}

}
