package fr.inria.gforge.dspace.overlay.benchmarks.tasks.space.bsbm;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import fr.inria.gforge.dspace.overlay.benchmarks.space.AbstractSpaceNetworkBenchmarks;
import fr.inria.gforge.dspace.overlay.benchmarks.tasks.AbstractTask;

/**
 * @author Laurent Pellegrino
 * 
 * @version $Id: BSBMQueriesInParallel.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class BSBMQueriesInParallel extends AbstractTask {

	private ExecutorService threadsPool;
	
	private List<BSBMQueryTask> BSBMQueries;
	
	public BSBMQueriesInParallel(AbstractSpaceNetworkBenchmarks benchmark) {
		super(benchmark);
		
		this.BSBMQueries = new ArrayList<BSBMQueryTask>();
		BSBMQueries.add(new BSBMCustomQuery1Task(super.benchmark, false));
		BSBMQueries.add(new BSBMCustomQuery2Task(super.benchmark, false));
		BSBMQueries.add(new BSBMCustomQuery3Task(super.benchmark, false));
		BSBMQueries.add(new BSBMCustomQuery4Task(super.benchmark, false));

		this.threadsPool = Executors.newFixedThreadPool(BSBMQueries.size());
	}

	public void run() {
		final CountDownLatch doneSignal = new CountDownLatch(BSBMQueries.size());
		
		for (final BSBMQueryTask query : BSBMQueries) {
			threadsPool.execute(new Runnable() {
				public void run() {
					query.run();
					doneSignal.countDown();
				}
			});
		}
		
		try {
			doneSignal.await();
		} catch (InterruptedException e) {
			e.printStackTrace();
		} finally {
			threadsPool.shutdown();
		}
	}

}
