package fr.inria.gforge.dspace.overlay.connector;

import java.io.IOException;
import java.util.Iterator;
import java.util.Set;

import org.objectweb.proactive.ActiveObjectCreationException;
import org.objectweb.proactive.api.PAActiveObject;
import org.ontoware.aifbcommons.collection.ClosableIterable;
import org.ontoware.aifbcommons.collection.ClosableIterator;
import org.ontoware.rdf2go.model.QueryResultTable;
import org.ontoware.rdf2go.model.Statement;
import org.ontoware.rdf2go.model.TriplePattern;
import org.ontoware.rdf2go.model.node.NodeOrVariable;
import org.ontoware.rdf2go.model.node.Resource;
import org.ontoware.rdf2go.model.node.ResourceOrVariable;
import org.ontoware.rdf2go.model.node.URI;
import org.ontoware.rdf2go.model.node.UriOrVariable;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.api.SemanticSpaceOperations;
import fr.inria.gforge.dspace.overlay.overlay.can.SemanticSpaceCANOverlay;
import fr.inria.gforge.dspace.overlay.overlay.chord.SemanticSpaceChordOverlay;
import fr.inria.gforge.dspace.overlay.tracker.SemanticChordTracker;

/**
 * OperationsConnector is a light application that can be plugged to an active
 * object which implements {@link SemanticSpaceOperations} like a
 * {@link SemanticChordTracker}, {@link SemanticSpaceChordOverlay},
 * {@link SemanticSpaceCANOverlay}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: OperationsConnector.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class OperationsConnector implements SemanticSpaceOperations {

    // private Set<URI> joinedSpaces = new HashSet<URI>();

    private SemanticChordTracker remoteTracker;

    /**
     * Constructor.
     * 
     * @param entryPoint
     *            the active entry point which implements
     *            {@link SemanticSpaceOperations} to use.
     */
    public OperationsConnector(SemanticChordTracker entryPoint) {
        this.remoteTracker = entryPoint;
    }

    /**
     * Constructor.
     * 
     * @param entryPointURL
     *            the URL to use in order to lookup up a
     *            {@link SemanticChordTracker} active object which will be used
     *            as an entry point for operations.
     */
    public OperationsConnector(String entryPointURL) {
        try {
            this.remoteTracker = PAActiveObject.lookupActive(SemanticChordTracker.class,
                    entryPointURL);
        } catch (ActiveObjectCreationException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public SemanticChordTracker getRemoteTracker() {
        return this.remoteTracker;
    }

    public void addAll(URI space, Iterator<? extends Statement> iterator)
            throws SemanticSpaceException {
        this.remoteTracker.addAll(space, iterator);
    }

    public void addStatement(URI space, Statement statement) throws SemanticSpaceException {
        this.remoteTracker.addStatement(space, statement);
    }

    public void addStatement(URI space, Resource subject, URI predicate, String literal)
            throws SemanticSpaceException {
        this.remoteTracker.addStatement(space, subject, predicate, literal);
    }

    public void addStatement(URI space, Resource subject, URI predicate, String literal,
            String languageTag) throws SemanticSpaceException {
        this.remoteTracker.addStatement(space, subject, predicate, literal);
    }

    public void addStatement(URI space, Resource subject, URI predicate, String literal,
            URI datatypeURI) throws SemanticSpaceException {
        this.remoteTracker.addStatement(space, subject, predicate, literal, datatypeURI);
    }

    public void addStatement(URI space, String subjectURIString, URI predicate, String literal)
            throws SemanticSpaceException {
        this.addStatement(space, subjectURIString, predicate, literal);
    }

    public void addStatement(URI space, String subjectURIString, URI predicate, String literal,
            String languageTag) throws SemanticSpaceException {
        this.remoteTracker.addStatement(space, subjectURIString, predicate, literal);
    }

    public void addStatement(URI space, String subjectURIString, URI predicate, String literal,
            URI datatypeURI) throws SemanticSpaceException {
        this.remoteTracker.addStatement(space, subjectURIString, predicate, literal, datatypeURI);
    }

    public boolean contains(URI space, Statement s) throws SemanticSpaceException {
        return this.remoteTracker.contains(space, s);
    }

    public boolean contains(URI space, ResourceOrVariable subject, UriOrVariable predicate,
            NodeOrVariable object) throws SemanticSpaceException {
        return this.remoteTracker.contains(space, subject, predicate, object);
    }

    public boolean contains(URI space, ResourceOrVariable subject, UriOrVariable predicate,
            String plainLiteral) throws SemanticSpaceException {
        return this.remoteTracker.contains(space, subject, predicate, plainLiteral);
    }

    public URI createFederation(Set<URI> spaces) throws SemanticSpaceException {
        return this.remoteTracker.createFederation(spaces);
    }

    public void createSpaceRelation(URI sSpaces, URI relation, URI oSpace)
            throws SemanticSpaceException {
        this.remoteTracker.createSpaceRelation(sSpaces, relation, oSpace);
    }

    public void deleteFederation(URI fed) throws SemanticSpaceException {
        this.remoteTracker.deleteFederation(fed);
    }

    public ClosableIterator<Statement> findStatements(URI space, TriplePattern triplepattern)
            throws SemanticSpaceException {
        // if (!this.joinedSpaces.contains(space)) {
        // throw new SpaceNotJoinedException("Space '" + space
        // + "' not joined. Joins it before to perform operation on.");
        // }
        return this.remoteTracker.findStatements(space, triplepattern);
    }

    public void joinSpace(URI space) throws SemanticSpaceException {
        // if (this.joinedSpaces.contains(space)) {
        // throw new SpaceAlreadyJoinedException("Space '" + space +
        // "' is already joined.");
        // }
        //
        // if (((GetResponseMessage)
        // PAFuture.getFutureValue(this.chordTracker.getRandomPeer().send(
        // new GetMessage(new Key(space.toString()))))).getCANPeer() == null) {
        // throw new SpaceNotExistsException("Space does not exists.");
        // }

        // this.chordTracker.joinSpace(space);
        // this.joinedSpaces.add(space);
    }

    public void leaveSpace(URI space) throws SemanticSpaceException {
        // if (!this.joinedSpaces.contains(space)) {
        // throw new SpaceNotJoinedException("Space '" + space +
        // "' is not joined.");
        // }

        // this.chordTracker.leaveSpace(space);
        // this.joinedSpaces.remove(space);
    }

    public Set<URI> listJoinedSpaces() {
        // return this.chordTracker.listJoinedSpaces();
        return null;
    }

    public Set<URI> listSpaces() {
        return this.remoteTracker.listSpaces();
    }

    public void registerEndPoint(URI endpoint) throws SemanticSpaceException {
        // TODO Auto-generated method stub
    }

    public void removeAll(URI space) throws SemanticSpaceException {
        // if (!this.joinedSpaces.contains(space)) {
        // throw new SpaceNotJoinedException("Space '" + space
        // + "' not joined. Joins it before to perform operation on.");
        // }
        this.remoteTracker.removeAll(space);
    }

    public void removeAll(URI space, Iterator<? extends Statement> it)
            throws SemanticSpaceException {
        // if (!this.joinedSpaces.contains(space)) {
        // throw new SpaceNotJoinedException("Space '" + space
        // + "' not joined. Joins it before to perform operation on.");
        // }
        this.remoteTracker.removeAll(space, it);
    }

    public void removeStatement(URI space, Statement statement) throws SemanticSpaceException {
        // if (!this.joinedSpaces.contains(space)) {
        // throw new SpaceNotJoinedException("Space '" + space
        // + "' not joined. Joins it before to perform operation on.");
        // }
        this.remoteTracker.removeStatement(space, statement);
    }

    public void removeStatement(URI space, Resource subject, URI predicate, String literal)
            throws SemanticSpaceException {
        // if (!this.joinedSpaces.contains(space)) {
        // throw new SpaceNotJoinedException("Space '" + space
        // + "' not joined. Joins it before to perform operation on.");
        // }
        this.remoteTracker.removeStatement(space, subject, predicate, literal);
    }

    public void removeStatement(URI space, Resource subject, URI predicate, String literal,
            String languageTag) throws SemanticSpaceException {
        // if (!this.joinedSpaces.contains(space)) {
        // throw new SpaceNotJoinedException("Space '" + space
        // + "' not joined. Joins it before to perform operation on.");
        // }
        this.remoteTracker.removeStatement(space, subject, predicate, literal);
    }

    public void removeStatement(URI space, Resource subject, URI predicate, String literal,
            URI datatypeURI) throws SemanticSpaceException {
        // if (!this.joinedSpaces.contains(space)) {
        // throw new SpaceNotJoinedException("Space '" + space
        // + "' not joined. Joins it before to perform operation on.");
        // }
        this.remoteTracker.removeStatement(space, subject, predicate, literal, datatypeURI);
    }

    public void removeStatement(URI space, String subjectURIString, URI predicate, String literal)
            throws SemanticSpaceException {
        // if (!this.joinedSpaces.contains(space)) {
        // throw new SpaceNotJoinedException("Space '" + space
        // + "' not joined. Joins it before to perform operation on.");
        // }
        this.remoteTracker.removeStatement(space, subjectURIString, predicate, literal);
    }

    public void removeStatement(URI space, String subjectURIString, URI predicate, String literal,
            String languageTag) throws SemanticSpaceException {
        // if (!this.joinedSpaces.contains(space)) {
        // throw new SpaceNotJoinedException("Space '" + space
        // + "' not joined. Joins it before to perform operation on.");
        // }
        this.remoteTracker.removeStatement(space, subjectURIString, predicate, literal, languageTag);
    }

    public void removeStatement(URI space, String subjectURIString, URI predicate, String literal,
            URI datatypeURI) throws SemanticSpaceException {
        // if (!this.joinedSpaces.contains(space)) {
        // throw new SpaceNotJoinedException("Space '" + space
        // + "' not joined. Joins it before to perform operation on.");
        // }
        this.remoteTracker.removeStatement(space, subjectURIString, predicate, literal, datatypeURI);
    }

    public void removeStatements(URI space, ResourceOrVariable subject, UriOrVariable predicate,
            NodeOrVariable object) throws SemanticSpaceException {
        // if (!this.joinedSpaces.contains(space)) {
        // throw new SpaceNotJoinedException("Space '" + space
        // + "' not joined. Joins it before to perform operation on.");
        // }
        this.remoteTracker.removeStatements(space, subject, predicate, object);
    }

    public void removeStatements(URI space, TriplePattern triplePattern)
            throws SemanticSpaceException {
        // if (!this.joinedSpaces.contains(space)) {
        // throw new SpaceNotJoinedException("Space '" + space
        // + "' not joined. Joins it before to perform operation on.");
        // }
        this.remoteTracker.removeStatements(space, triplePattern);
    }

    public boolean sparqlAsk(URI space, String query) throws SemanticSpaceException {
        // if (!this.joinedSpaces.contains(space)) {
        // throw new SpaceNotJoinedException("Space '" + space
        // + "' not joined. Joins it before to perform operation on.");
        // }
        return this.remoteTracker.sparqlAsk(space, query);
    }

    public ClosableIterable<Statement> sparqlConstruct(URI space, String query)
            throws SemanticSpaceException {
        // if (!this.joinedSpaces.contains(space)) {
        // throw new SpaceNotJoinedException("Space '" + space
        // + "' not joined. Joins it before to perform operation on.");
        // }

        return this.remoteTracker.sparqlConstruct(space, query);
    }

    public QueryResultTable sparqlSelect(URI space, String queryString)
            throws SemanticSpaceException {
        // if (!this.joinedSpaces.contains(space)) {
        // throw new SpaceNotJoinedException("Space '" + space
        // + "' not joined. Joins it before to perform operation on.");
        // }
        return this.remoteTracker.sparqlSelect(space, queryString);
    }

    public ClosableIterable<Statement> sparqlDescribe(URI space, String query)
	    throws SemanticSpaceException {
	// TODO implements it
	return null;
    }

    /* ------------------------------------------ */

    // public void leaveJoinedSpaces() {
    // for (URI spaceURI : this.joinedSpaces) {
    // try {
    // this.leaveSpace(spaceURI);
    // } catch (SemanticSpaceException e) {
    // e.printStackTrace();
    // }
    // }
    // }

    // public Set<URI> getJoinedSpaces() {
    // return this.joinedSpaces;
    // }

}
