package fr.inria.gforge.dspace.overlay.datastore;

import java.util.Iterator;

import org.ontoware.aifbcommons.collection.ClosableIterable;
import org.ontoware.aifbcommons.collection.ClosableIterator;
import org.ontoware.rdf2go.exception.MalformedQueryException;
import org.ontoware.rdf2go.model.QueryResultTable;
import org.ontoware.rdf2go.model.Statement;
import org.ontoware.rdf2go.model.TriplePattern;
import org.ontoware.rdf2go.model.node.NodeOrVariable;
import org.ontoware.rdf2go.model.node.Resource;
import org.ontoware.rdf2go.model.node.ResourceOrVariable;
import org.ontoware.rdf2go.model.node.URI;
import org.ontoware.rdf2go.model.node.UriOrVariable;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;

/**
 * @author Laurent Pellegrino
 * 
 * @version $Id: SemanticDataStoreOperations.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public interface SemanticDataStoreOperations {


    // public void addAll(URI space, Iterator<? extends Statement> other)
    //       throws SemanticSpaceException;

    public void addStatement(URI space, Statement statement) throws SemanticSpaceException;

    public void addStatement(URI space, Resource subject, URI predicate, String literal)
            throws SemanticSpaceException;

    public void addStatement(URI space, Resource subject, URI predicate, String literal,
            String languageTag) throws SemanticSpaceException;

    public void addStatement(URI space, Resource subject, URI predicate, String literal,
            URI datatypeURI) throws SemanticSpaceException;

    public void addStatement(URI space, String subjectURIString, URI predicate, String literal)
            throws SemanticSpaceException;

    public void addStatement(URI space, String subjectURIString, URI predicate, String literal,
            String languageTag) throws SemanticSpaceException;

    public void addStatement(URI space, String subjectURIString, URI predicate, String literal,
            URI datatypeURI) throws SemanticSpaceException;

    // query

    public ClosableIterable<Statement> sparqlConstruct(URI space, String query)
            throws SemanticSpaceException, MalformedQueryException;

    public QueryResultTable sparqlSelect(URI space, String queryString)
            throws MalformedQueryException, SemanticSpaceException;

    public boolean sparqlAsk(URI space, String query) throws SemanticSpaceException;

 
    public ClosableIterator<Statement> findStatements(URI space, TriplePattern triplepattern)
            throws SemanticSpaceException;

    // remove

    public void removeAll(URI space) throws SemanticSpaceException;

    public void removeAll(URI space, Iterator<? extends Statement> statements)
            throws SemanticSpaceException;

    public void removeStatement(URI space, Statement statement) throws SemanticSpaceException;

    public void removeStatement(URI space, Resource subject, URI predicate, String literal)
            throws SemanticSpaceException;

    public void removeStatement(URI space, Resource subject, URI predicate, String literal,
            String languageTag) throws SemanticSpaceException;

    public void removeStatement(URI space, Resource subject, URI predicate, String literal,
            URI datatypeURI) throws SemanticSpaceException;

    public void removeStatement(URI space, String subjectURIString, URI predicate, String literal)
            throws SemanticSpaceException;

    public void removeStatement(URI space, String subjectURIString, URI predicate, String literal,
            String languageTag) throws SemanticSpaceException;

    public void removeStatement(URI space, String subjectURIString, URI predicate, String literal,
            URI datatypeURI) throws SemanticSpaceException;

    public void removeStatements(URI space, ResourceOrVariable subject, UriOrVariable predicate,
            NodeOrVariable object) throws SemanticSpaceException;

    public void removeStatements(URI space, TriplePattern triplePattern)
            throws SemanticSpaceException;

    // contain

    public boolean contains(URI space, Statement s) throws SemanticSpaceException;

    public boolean contains(URI space, ResourceOrVariable subject, UriOrVariable predicate,
            NodeOrVariable object) throws SemanticSpaceException;

    public boolean contains(URI space, ResourceOrVariable subject, UriOrVariable predicate,
            String plainLiteral) throws SemanticSpaceException;
}
