package fr.inria.gforge.dspace.overlay.kernel.operations.datastore;

import java.io.Serializable;

import org.ontoware.rdf2go.model.Statement;
import org.ontoware.rdf2go.model.node.Resource;
import org.ontoware.rdf2go.model.node.URI;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.datastore.SemanticDataStore;

/**
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: AddStatementOperation.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class AddStatementOperation extends AbstractDatastoreOperation {

    private static final long serialVersionUID = 1L;
    
    private InnerAddStatementOperation operation;

    public AddStatementOperation(final URI space, final Statement statement) {
        super(space);
        this.operation = new InnerAddStatementOperation() {
            private static final long serialVersionUID = 1L;

            public DatastoreResponseOperation<Boolean> handle(SemanticDataStore dataStore) {
                try {
                    dataStore.addStatement(space, statement);
                } catch (SemanticSpaceException e) {
                    e.printStackTrace();
                }
                return new DatastoreResponseOperation<Boolean>(true);
            };
        };
    }

    public AddStatementOperation(final URI space, final Resource subject, final URI predicate,
            final String literal) {
        super(space);
        this.operation = new InnerAddStatementOperation() {
            private static final long serialVersionUID = 1L;

            public DatastoreResponseOperation<Boolean> handle(SemanticDataStore dataStore) {
                try {
                    dataStore.addStatement(space, subject, predicate, literal);
                } catch (SemanticSpaceException e) {
                    e.printStackTrace();
                }
                return new DatastoreResponseOperation<Boolean>(true);
            };
        };
    }

    public AddStatementOperation(final URI space, final Resource subject, final URI predicate,
            final String literal, final String languageTag) {
        super(space);
        this.operation = new InnerAddStatementOperation() {
            private static final long serialVersionUID = 1L;

            public DatastoreResponseOperation<Boolean> handle(SemanticDataStore dataStore) {
                try {
                    dataStore.addStatement(space, subject, predicate
                            , literal, languageTag);
                } catch (SemanticSpaceException e) {
                    e.printStackTrace();
                }
                return new DatastoreResponseOperation<Boolean>(true);
            };
        };
    }

    public AddStatementOperation(final URI space, final Resource subject, final URI predicate,
            final String literal, final URI datatypeURI) {
        super(space);
        this.operation = new InnerAddStatementOperation() {
            private static final long serialVersionUID = 1L;

            public DatastoreResponseOperation<Boolean> handle(SemanticDataStore dataStore) {
                try {
                    dataStore.addStatement(space, subject, predicate
                            , literal, datatypeURI);
                } catch (SemanticSpaceException e) {
                    e.printStackTrace();
                }
                return new DatastoreResponseOperation<Boolean>(true);
            };
        };
    }

    public AddStatementOperation(final URI space, final String subjectURIString,
            final URI predicate, final String literal) {
        super(space);
        this.operation = new InnerAddStatementOperation() {
            private static final long serialVersionUID = 1L;

            public DatastoreResponseOperation<Boolean> handle(SemanticDataStore dataStore) {
                try {
                    dataStore.addStatement(space, subjectURIString,
                            predicate, literal);
                } catch (SemanticSpaceException e) {
                    e.printStackTrace();
                }
                return new DatastoreResponseOperation<Boolean>(true);
            };
        };
    }

    public AddStatementOperation(final URI space, final String subjectURIString,
            final URI predicate, final String literal, final String languageTag) {
        super(space);
        this.operation = new InnerAddStatementOperation() {
            private static final long serialVersionUID = 1L;

            public DatastoreResponseOperation<Boolean> handle(SemanticDataStore dataStore) {
                try {
                    dataStore.addStatement(space, subjectURIString,
                            predicate, literal, languageTag);
                } catch (SemanticSpaceException e) {
                    e.printStackTrace();
                }
                return new DatastoreResponseOperation<Boolean>(true);
            };
        };
    }

    public AddStatementOperation(final URI space, final String subjectURIString,
            final URI predicate, final String literal, final URI datatypeURI) {
        super(space);
        this.operation = new InnerAddStatementOperation() {
            private static final long serialVersionUID = 1L;

            public DatastoreResponseOperation<Boolean> handle(SemanticDataStore dataStore) {
                try {
                    dataStore.addStatement(space, subjectURIString,
                            predicate, literal, datatypeURI);
                } catch (SemanticSpaceException e) {
                    e.printStackTrace();
                }
                return new DatastoreResponseOperation<Boolean>(true);
            };
        };
    }

    public DatastoreResponseOperation<Boolean> handle(SemanticDataStore dataStore)
            throws SemanticSpaceException {
        return this.operation.handle(dataStore);
    }

    public abstract class InnerAddStatementOperation implements Serializable {

        private static final long serialVersionUID = 1L;

        public InnerAddStatementOperation() {

        }

        public abstract DatastoreResponseOperation<Boolean> handle(SemanticDataStore dataStore);

    }
}
