package fr.inria.gforge.dspace.overlay.kernel.operations.datastore;

import java.io.Serializable;

import org.ontoware.rdf2go.model.Statement;
import org.ontoware.rdf2go.model.node.Resource;
import org.ontoware.rdf2go.model.node.URI;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.datastore.SemanticDataStore;

/**
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: RemoveStatementOperation.java 3622 2010-03-12 09:15:46Z
 *          laurentp $
 */
public class RemoveStatementOperation extends AbstractDatastoreOperation {

	private static final long serialVersionUID = 1L;
	
	private InnerRemoveStatementOperation operation;

    public RemoveStatementOperation(final URI space, final Statement statement) {
        super(space);
        this.operation = new InnerRemoveStatementOperation() {
			private static final long serialVersionUID = 1L;

			public DatastoreResponseOperation<Boolean> handle(SemanticDataStore dataStore) {
                try {
                    dataStore.removeStatement(space, statement);
                } catch (SemanticSpaceException e) {
                    e.printStackTrace();
                }
                return new DatastoreResponseOperation<Boolean>(true);
            };
        };
    }

    public RemoveStatementOperation(final URI space, final Resource subject,
            final URI predicate, final String literal) {
        super(space);
        this.operation = new InnerRemoveStatementOperation() {
			private static final long serialVersionUID = 1L;

			public DatastoreResponseOperation<Boolean> handle(SemanticDataStore dataStore) {
                try {
                    dataStore.removeStatement(space, subject, predicate
                            , literal);
                } catch (SemanticSpaceException e) {
                    e.printStackTrace();
                }
                return new DatastoreResponseOperation<Boolean>(true);
            };
        };
    }

    public RemoveStatementOperation(final URI space, final Resource subject,
            final URI predicate, final String literal, final String languageTag) {
        super(space);
        this.operation = new InnerRemoveStatementOperation() {
			private static final long serialVersionUID = 1L;

			public DatastoreResponseOperation<Boolean> handle(SemanticDataStore dataStore) {
                try {
                    dataStore.removeStatement(space, subject, predicate
                            , literal, languageTag);
                } catch (SemanticSpaceException e) {
                    e.printStackTrace();
                }
                return new DatastoreResponseOperation<Boolean>(true);
            };
        };
    }

    public RemoveStatementOperation(final URI space, final Resource subject,
            final URI predicate, final String literal, final URI datatypeURI) {
        super(space);
        this.operation = new InnerRemoveStatementOperation() {
			private static final long serialVersionUID = 1L;

			public DatastoreResponseOperation<Boolean> handle(SemanticDataStore dataStore) {
                try {
                    dataStore.removeStatement(space, subject, predicate
                            , literal, datatypeURI);
                } catch (SemanticSpaceException e) {
                    e.printStackTrace();
                }
                return new DatastoreResponseOperation<Boolean>(true);
            };
        };
    }

    public RemoveStatementOperation(final URI space, final String subjectURIString,
            final URI predicate, final String literal) {
        super(space);
        this.operation = new InnerRemoveStatementOperation() {
			private static final long serialVersionUID = 1L;

			public DatastoreResponseOperation<Boolean> handle(SemanticDataStore dataStore) {
                try {
                    dataStore.removeStatement(space, subjectURIString, predicate
                            , literal);
                } catch (SemanticSpaceException e) {
                    e.printStackTrace();
                }
                return new DatastoreResponseOperation<Boolean>(true);
            };
        };
    }

    public RemoveStatementOperation(final URI space, final String subjectURIString,
            final URI predicate, final String literal, final String languageTag) {
        super(space);
        this.operation = new InnerRemoveStatementOperation() {
			private static final long serialVersionUID = 1L;

			public DatastoreResponseOperation<Boolean> handle(SemanticDataStore dataStore) {
                try {
                    dataStore.removeStatement(space, subjectURIString, predicate
                            , literal, languageTag);
                } catch (SemanticSpaceException e) {
                    e.printStackTrace();
                }
                return new DatastoreResponseOperation<Boolean>(true);
            };
        };
    }

    public RemoveStatementOperation(final URI space, final String subjectURIString,
            final URI predicate, final String literal, final URI datatypeURI) {
        super(space);
        this.operation = new InnerRemoveStatementOperation() {
			private static final long serialVersionUID = 1L;

			public DatastoreResponseOperation<Boolean> handle(SemanticDataStore dataStore) {
                try {
                    dataStore.removeStatement(space, subjectURIString, predicate
                            , literal, datatypeURI);
                } catch (SemanticSpaceException e) {
                    e.printStackTrace();
                }
                return new DatastoreResponseOperation<Boolean>(true);
            };
        };
    }

    public DatastoreResponseOperation<Boolean> handle(SemanticDataStore dataStore)
            throws SemanticSpaceException {
        return this.operation.handle(dataStore);
    }

    public abstract class InnerRemoveStatementOperation implements Serializable {

		private static final long serialVersionUID = 1L;

		public InnerRemoveStatementOperation() {

        }

        public abstract DatastoreResponseOperation<Boolean> handle(SemanticDataStore dataStore);

    }
}
