package fr.inria.gforge.dspace.overlay.messages.query.can;

import java.io.IOException;

import org.objectweb.proactive.core.util.converter.ByteToObjectConverter;
import org.objectweb.proactive.core.util.converter.ObjectToByteConverter;
import org.objectweb.proactive.extensions.structuredp2p.messages.query.can.ForwardQueryMessage;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.coordinates.CoordinateValue;
import org.objectweb.proactive.extensions.structuredp2p.router.Router;
import org.objectweb.proactive.extensions.structuredp2p.router.can.UnicastQueryRouter;
import org.objectweb.proactive.extensions.structuredp2p.validator.can.UnicastConstraintsValidator;
import org.ontoware.rdf2go.model.Statement;
import org.ontoware.rdf2go.model.node.Resource;
import org.ontoware.rdf2go.model.node.URI;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import fr.inria.gforge.dspace.overlay.kernel.operations.datastore.AbstractDatastoreOperation;
import fr.inria.gforge.dspace.overlay.kernel.operations.datastore.AddStatementOperation;
import fr.inria.gforge.dspace.overlay.overlay.can.SemanticSpaceCANOverlay;
import fr.inria.gforge.dspace.overlay.util.SemanticHelper;

/**
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: AddStatementQueryMessage.java 3746 2010-03-29 09:00:44Z
 *          laurentp $
 */
public class AddStatementQueryMessage extends ForwardQueryMessage {

    private static final long serialVersionUID = 1L;

    private final static transient Logger logger = 
	LoggerFactory.getLogger(AddStatementQueryMessage.class);

    private byte[] operation;

    public AddStatementQueryMessage(final URI space, final Statement statement) {
        super(SemanticHelper.createCoordinateArrayFrom(statement));
        this.operation = 
            toByteArray(
                    new AddStatementOperation(space, statement));
    }

    public AddStatementQueryMessage(final URI space, final Resource subject, final URI predicate,
            final String literal) {
        super(SemanticHelper.createCoordinateArrayFrom(subject, predicate, literal));
        this.operation = 
            toByteArray(
                    new AddStatementOperation(
                            space, subject, predicate, literal));
    }

    public AddStatementQueryMessage(final URI space, final Resource subject, final URI predicate,
            final String literal, final String languageTag) {
        super(SemanticHelper.createCoordinateArrayFrom(subject, predicate, literal));
        this.operation = 
            toByteArray(
                    new AddStatementOperation(
                            space, subject, predicate, literal));
    }

    public AddStatementQueryMessage(final URI space, final Resource subject, final URI predicate,
            final String literal, final URI datatypeURI) {
        super(SemanticHelper.createCoordinateArrayFrom(subject, predicate, literal));
        this.operation = 
            toByteArray(
                    new AddStatementOperation(
                            space, subject, predicate, literal, datatypeURI));
    }

    public AddStatementQueryMessage(final URI space, final String subjectURIString,
            final URI predicate, final String literal) {
        super(SemanticHelper.createCoordinateArrayFrom(subjectURIString, predicate, literal));
        this.operation = 
            toByteArray(
                    new AddStatementOperation(
                            space, subjectURIString, predicate, literal));
    }

    public AddStatementQueryMessage(final URI space, final String subjectURIString,
            final URI predicate, final String literal, final String languageTag) {
        super(SemanticHelper.createCoordinateArrayFrom(subjectURIString, predicate, literal));
        this.operation = 
            toByteArray(
                    new AddStatementOperation(
                            space, subjectURIString, predicate, literal, languageTag));
    }

    public AddStatementQueryMessage(final URI space, final String subjectURIString,
            final URI predicate, final String literal, final URI datatypeURI) {
        super(SemanticHelper.createCoordinateArrayFrom(subjectURIString, predicate, literal));
        this.operation = 
            toByteArray(
                new AddStatementOperation(
                        space, subjectURIString, predicate, literal, datatypeURI));
    }

    public byte[] getOperation() {
        return this.operation;
    }

    public Router<ForwardQueryMessage, CoordinateValue[]> getRouter() {
        return new UnicastQueryRouter<ForwardQueryMessage>(new UnicastConstraintsValidator()) {
            protected void onDestinationReached(StructuredOverlay overlay, ForwardQueryMessage msg) {
                try {
                    ((SemanticSpaceCANOverlay) overlay).getLocalSemanticSpaceOverlayKernel().send(
                            (AbstractDatastoreOperation) ByteToObjectConverter.ObjectStream
                                    .convert(((AddStatementQueryMessage) msg).getOperation()));
                } catch (IOException e) {
                    e.printStackTrace();
                } catch (ClassNotFoundException e) {
                    e.printStackTrace();
                }

                if (logger.isDebugEnabled()) {
                    logger.debug("Destination reached {} and statement added", overlay);
                }
            };
        };
    }

    private static byte[] toByteArray(AddStatementOperation op) {
        byte[] result = null;
        try {
             result = ObjectToByteConverter.ObjectStream.convert(op);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result;
    }
    
}
