package fr.inria.gforge.dspace.overlay.messages.query.can;

import org.objectweb.proactive.extensions.structuredp2p.messages.query.AbstractQueryMessage;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.coordinates.CoordinateValue;

import fr.inria.gforge.dspace.overlay.messages.response.can.AnycastResponseMessage;

/**
 * Message used to dispatch a query to all peers validating the specified
 * constraints which are the coordinates to reach.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: AnycastQueryMessage.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public abstract class AnycastQueryMessage extends AbstractQueryMessage<CoordinateValue[]> {

    private static final long serialVersionUID = 1L;
    
    private AnycastRoutingList anycastRoutingList = new AnycastRoutingList();

    /**
     * Constructs a new message with the specified coordinates to reach.
     * 
     * @param coordinatesToReach
     *            the coordinates to reach.
     */
    public AnycastQueryMessage(CoordinateValue[] coordinatesToReach) {
        super(coordinatesToReach);
    }

    public abstract AnycastResponseMessage<?> createResponseMessage();

    /**
     * Returns the {@link AnycastRoutingList} containing the
     * {@link AnycastRoutingEntry} to use in order to route the response.
     * 
     * @return the {@link AnycastRoutingList} containing the
     *         {@link AnycastRoutingEntry} to use in order to route the
     *         response.
     */
    public AnycastRoutingList getAnycastRoutingList() {
        return this.anycastRoutingList;
    }

    public String toString() {
        StringBuffer buf = new StringBuffer("AnycastQueryMessage ID=");
        buf.append(this.getUUID());

        buf.append("\nStack: \n");
        for (AnycastRoutingEntry entry : this.anycastRoutingList) {
            buf.append("  - ");
            buf.append(entry.getPeerStub());

            if (entry.validatesKeyConstraints()) {
                buf.append(" (validates constraints)");
            } else {
                buf.append(" (doesn't validate constraints)");
            }

            buf.append("\n");
        }

        return buf.toString();
    }

}
