package fr.inria.gforge.dspace.overlay.messages.query.can;

import java.util.LinkedList;
import java.util.UUID;

import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;

import fr.inria.gforge.dspace.overlay.messages.response.can.AnycastResponseMessage;

/**
 * List used by {@link AnycastQueryMessage} and
 * {@link AnycastResponseMessage} to store {@link AnycastRoutingEntry}. It
 * contains an {@link AnycastRoutingEntry} for each {@link Peer} met while
 * routing an {@link AnycastQueryMessage}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: AnycastRoutingList.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class AnycastRoutingList extends LinkedList<AnycastRoutingEntry> {

    private static final long serialVersionUID = 1L;

    /**
     * Search the specified {@link UUID} which is associated to a {@link Peer}
     * identifier in the current list composed of {@link AnycastRoutingEntry}.
     * 
     * @param peerID
     *            the {@link UUID} to look for.
     * @return the {@link AnycastRoutingEntry} found or <code>null</code> if
     *         not found.
     */
    public AnycastRoutingEntry getRoutingResponseEntryBy(UUID peerID) {
        for (AnycastRoutingEntry entry : this) {
            if (entry.getPeerID().equals(peerID)) {
                return entry;
            }
        }

        return null;
    }
    
}
