package fr.inria.gforge.dspace.overlay.messages.query.can;

import org.objectweb.proactive.extensions.structuredp2p.messages.query.can.ForwardQueryMessage;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.coordinates.CoordinateValue;
import org.objectweb.proactive.extensions.structuredp2p.router.Router;
import org.objectweb.proactive.extensions.structuredp2p.router.can.UnicastQueryRouter;
import org.objectweb.proactive.extensions.structuredp2p.validator.can.UnicastConstraintsValidator;
import org.ontoware.rdf2go.model.Statement;
import org.ontoware.rdf2go.model.node.Resource;
import org.ontoware.rdf2go.model.node.URI;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import fr.inria.gforge.dspace.overlay.kernel.operations.datastore.RemoveStatementOperation;
import fr.inria.gforge.dspace.overlay.overlay.can.SemanticSpaceCANOverlay;
import fr.inria.gforge.dspace.overlay.util.SemanticHelper;

/**
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: RemoveStatementQueryMessage.java 3746 2010-03-29 09:00:44Z
 *          laurentp $
 */
public class RemoveStatementQueryMessage extends ForwardQueryMessage {

    private static final long serialVersionUID = 1L;

    private final static transient Logger logger = LoggerFactory
            .getLogger(RemoveStatementQueryMessage.class);

    private RemoveStatementOperation operation;

    public RemoveStatementQueryMessage(final URI space, final Statement statement) {
        super(SemanticHelper.createCoordinateArrayFrom(statement));
        this.operation = new RemoveStatementOperation(space, statement);
    }

    public RemoveStatementQueryMessage(final URI space, final Resource subject,
            final URI predicate, final String literal) {
        super(SemanticHelper.createCoordinateArrayFrom(subject, predicate, literal));
        this.operation = new RemoveStatementOperation(space, subject, predicate, literal);
    }

    public RemoveStatementQueryMessage(final URI space, final Resource subject,
            final URI predicate, final String literal, final String languageTag) {
        super(SemanticHelper.createCoordinateArrayFrom(subject, predicate, literal));
        this.operation = new RemoveStatementOperation(space, subject, predicate, literal,
                languageTag);
    }

    public RemoveStatementQueryMessage(final URI space, final Resource subject,
            final URI predicate, final String literal, final URI datatypeURI) {
        super(SemanticHelper.createCoordinateArrayFrom(subject, predicate, literal));
        this.operation = new RemoveStatementOperation(space, subject, predicate, literal,
                datatypeURI);
    }

    public RemoveStatementQueryMessage(final URI space, final String subjectURIString,
            final URI predicate, final String literal) {
        super(SemanticHelper.createCoordinateArrayFrom(subjectURIString, predicate, literal));
        this.operation = new RemoveStatementOperation(space, subjectURIString, predicate, literal);
    }

    public RemoveStatementQueryMessage(final URI space, final String subjectURIString,
            final URI predicate, final String literal, final String languageTag) {
        super(SemanticHelper.createCoordinateArrayFrom(subjectURIString, predicate, literal));
        this.operation = new RemoveStatementOperation(space, subjectURIString, predicate, literal,
                languageTag);
    }

    public RemoveStatementQueryMessage(final URI space, final String subjectURIString,
            final URI predicate, final String literal, final URI datatypeURI) {
        super(SemanticHelper.createCoordinateArrayFrom(subjectURIString, predicate, literal));
        this.operation = new RemoveStatementOperation(space, subjectURIString, predicate, literal,
                datatypeURI);
    }

    public RemoveStatementOperation getOperation() {
        return operation;
    }

    public Router<ForwardQueryMessage, CoordinateValue[]> getRouter() {
        return new UnicastQueryRouter<ForwardQueryMessage>(new UnicastConstraintsValidator()) {
            protected void onDestinationReached(StructuredOverlay overlay, ForwardQueryMessage msg) {
                ((SemanticSpaceCANOverlay) overlay).
                    getLocalSemanticSpaceOverlayKernel().send(
                        ((RemoveStatementQueryMessage)msg).getOperation());

                if (logger.isDebugEnabled()) {
                    logger.debug(
                            "Destination {} reached and remove statement operation performed",
                            overlay);
                }
            };
        };
    }
    
}
