package fr.inria.gforge.dspace.overlay.messages.query.can;

import org.objectweb.proactive.extensions.structuredp2p.overlay.can.AbstractCANOverlay;
import org.ontoware.rdf2go.model.Statement;
import org.ontoware.rdf2go.model.node.URI;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import fr.inria.gforge.dspace.overlay.kernel.operations.datastore.RemoveStatementOperation;
import fr.inria.gforge.dspace.overlay.messages.response.can.RemoveStatementsResponseMessage;
import fr.inria.gforge.dspace.overlay.overlay.can.SemanticSpaceCANOverlay;
import fr.inria.gforge.dspace.overlay.router.can.AnycastQueryRouter;
import fr.inria.gforge.dspace.overlay.util.SemanticHelper;
import fr.inria.gforge.dspace.overlay.validator.can.RemoveStatementsConstraintsValidator;

/**
 * Removes all statements that validates the specified constraints.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: RemoveStatementsQueryMessage.java 3819 2010-04-02 14:59:26Z
 *          laurentp $
 */
public class RemoveStatementsQueryMessage extends AnycastQueryMessage {

    private static final long serialVersionUID = 1L;

    private static final transient Logger logger = LoggerFactory
            .getLogger(RemoveStatementQueryMessage.class);

    private URI spaceURI;

    private Statement statement;

    public RemoveStatementsQueryMessage(final URI space, final Statement statement) {
        super(SemanticHelper.createCoordinateArrayFrom(statement));
        this.spaceURI = space;
        this.statement = statement;
    }

    public AnycastQueryRouter<AnycastQueryMessage> getRouter() {
        return new AnycastQueryRouter<AnycastQueryMessage>(new RemoveStatementsConstraintsValidator()) {
            public void onPeerWhichValidatesKeyConstraints(AbstractCANOverlay overlay, AnycastQueryMessage msg) {
                ((SemanticSpaceCANOverlay) overlay).getLocalSemanticSpaceOverlayKernel().send(
                        new RemoveStatementOperation(
                                ((RemoveStatementsQueryMessage)msg).getSpaceURI(),
                                ((RemoveStatementsQueryMessage)msg).getStatement()));
                
                if (logger.isDebugEnabled()) {
                    logger.debug("Remove statements performed on {}", overlay);
                }
            }
        };
    }

    public URI getSpaceURI() {
        return this.spaceURI;
    }

    public Statement getStatement() {
        return this.statement;
    }

    public RemoveStatementsResponseMessage createResponseMessage() {
        return new RemoveStatementsResponseMessage(this, this.getKeyToReach(), true);
    }

}
