package fr.inria.gforge.dspace.overlay.messages.query.can;

import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.coordinates.CoordinateValue;

import fr.inria.gforge.dspace.overlay.api.messages.query.SPARQLQuery;

/**
 * SemanticQueryMessage is a query used in order to retrieve data from peer by
 * Resource Description Framework criteria which is a method for conceptual
 * description or modeling of information. This kind of request will query some
 * {@link Peer} (each maintains an RDF datastore) in order to retrieve results.
 * 
 * This kind of query can only be used on a CAN structured peer-to-peer network.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: SemanticQueryMessage.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public abstract class SemanticQueryMessage extends AnycastQueryMessage {

    private static final long serialVersionUID = 1L;

    private SPARQLQuery sparqlQuery;

    /**
     * Constructs a new SemanticQueryMessage with the specified
     * <code>coordinatesToFind</code>.
     * 
     * @param coordinatesToFind
     *            the coordinates to reach.
     */
    public SemanticQueryMessage(CoordinateValue[] coordinatesToFind) {
        super(coordinatesToFind);
    }

    /**
     * Constructs a new SemanticQueryMessage with the specified
     * <code>query</code> and <code>coordinatesToFind</code>.
     * 
     * @param query
     *            the query to execute when coordinates are reached.
     * 
     * @param coordinatesToFind
     *            the coordinates to reach.
     */
    public SemanticQueryMessage(SPARQLQuery query, CoordinateValue[] coordinatesToFind) {
        super(coordinatesToFind);
        this.sparqlQuery = query;
    }

    /**
     * Returns the {@link SPARQLQuery} which is being handled.
     * 
     * @return the {@link SPARQLQuery} which is being handled.
     */
    public SPARQLQuery getSparqlQuery() {
        return this.sparqlQuery;
    }

    /**
     * Indicates if the key to reach has all its coordinates fixed with a not
     * <code>null</code> value or not.
     * 
     * @return <code>true</code> if the key to reach has all its coordinates
     *         fixed with a not <code>null</code>. <code>false</code> otherwise.
     */
    public boolean keyToReachContainsAllCoordinates() {
        for (CoordinateValue coordinate : super.getKeyToReach()) {
            if (coordinate == null) {
                return false;
            }
        }
        return true;
    }

}
