package fr.inria.gforge.dspace.overlay.messages.query.can;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.AbstractCANOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.coordinates.CoordinateValue;
import org.ontoware.rdf2go.model.node.URI;

import fr.inria.gforge.dspace.overlay.kernel.SemanticSpaceOverlayKernel;
import fr.inria.gforge.dspace.overlay.kernel.operations.datastore.DatastoreResponseOperation;
import fr.inria.gforge.dspace.overlay.kernel.operations.datastore.SPARQLSelectOperation;
import fr.inria.gforge.dspace.overlay.messages.response.can.StatementsCounterResponseMessage;
import fr.inria.gforge.dspace.overlay.overlay.can.SemanticSpaceCANOverlay;
import fr.inria.gforge.dspace.overlay.rdf2go.wrappers.QueryResultTableWrapper;
import fr.inria.gforge.dspace.overlay.router.can.AnycastQueryRouter;
import fr.inria.gforge.dspace.overlay.validator.can.RemoveStatementsConstraintsValidator;

/**
 * Count the number of statements by {@link Peer} by supposing that each
 * {@link Peer} as an unique {@link SemanticSpaceOverlayKernel}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: StatementsCounterQueryMessage.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class StatementsCounterQueryMessage extends AnycastQueryMessage {

    private static final long serialVersionUID = 1L;

    private URI spaceURI;

    private Map<UUID, Integer> nbStatementsByPeer = new HashMap<UUID, Integer>();

    public StatementsCounterQueryMessage(final URI space) {
        super(new CoordinateValue[] { null, null, null });
        this.spaceURI = space;
    }

    public Integer put(UUID key, Integer value) {
        return this.nbStatementsByPeer.put(key, value);
    }
    
    public AnycastQueryRouter<AnycastQueryMessage> getRouter() {
        return new AnycastQueryRouter<AnycastQueryMessage>( 
                new RemoveStatementsConstraintsValidator()) {
            @SuppressWarnings("unchecked")
            public void onPeerWhichValidatesKeyConstraints(AbstractCANOverlay overlay, AnycastQueryMessage msg) {
                ((StatementsCounterQueryMessage) msg).put(
                        overlay.getIdentifier(),
                        ((DatastoreResponseOperation<QueryResultTableWrapper>) 
                                ((SemanticSpaceCANOverlay) overlay)
                                    .getLocalSemanticSpaceOverlayKernel().send(
                                            new SPARQLSelectOperation(
                                                    spaceURI, "SELECT ?s WHERE { ?s ?p ?o }")))
                                                        .getValue().getRows().size());
            }
        };
    }

    public StatementsCounterResponseMessage createResponseMessage() {
        return new StatementsCounterResponseMessage(this, this.getKeyToReach(), this.nbStatementsByPeer);
    }

}
