package fr.inria.gforge.dspace.overlay.messages.query.can;

import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.AbstractCANOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.Zone;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.coordinates.CoordinateValue;
import org.objectweb.proactive.extensions.structuredp2p.validator.ConstraintsValidator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import fr.inria.gforge.dspace.overlay.api.messages.query.SPARQLQuery;
import fr.inria.gforge.dspace.overlay.messages.response.can.AnycastResponseMessage;
import fr.inria.gforge.dspace.overlay.overlay.can.SemanticSpaceCANOverlay;
import fr.inria.gforge.dspace.overlay.router.can.AnycastQueryRouter;
import fr.inria.gforge.dspace.overlay.validator.AnycastConstraintsValidator;

/**
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: RDFTriplePatternQueryMessage.java 3127 2010-01-06 15:59:42Z
 *          laurentp $
 */
public class TriplePatternQueryMessage extends SemanticQueryMessage {

    private static final long serialVersionUID = 1L;

    private final static transient Logger logger = LoggerFactory
            .getLogger(TriplePatternQueryMessage.class);

    public TriplePatternQueryMessage(SPARQLQuery query, CoordinateValue[] coordinatesToFind) {
        super(query, coordinatesToFind);

        if (logger.isDebugEnabled()) {
        	StringBuffer buf = new StringBuffer();
        	int i=0;
        	
        	buf.append("New TriplePatternQueryMessage created (");
        	for (CoordinateValue coord : super.getKeyToReach()) {
        		if (coord == null) { 
        			buf.append("null");
        		} else {
        			buf.append(coord.getValue());
        		}
        		
        		if (i < super.getKeyToReach().length - 1) {
        			buf.append(", ");
        		}
        		
        		i++;
        	}
        	buf.append(")");
        	logger.debug(buf.toString());
        }
    }

    public AnycastResponseMessage<?> createResponseMessage() {
        return super.getSparqlQuery().createResponseMessage(this);
    }

    public AnycastQueryRouter<TriplePatternQueryMessage> getRouter() {
        ConstraintsValidator<CoordinateValue[]> validator = new AnycastConstraintsValidator<CoordinateValue[]>() {
            public boolean validatesKeyConstraints(StructuredOverlay overlay, CoordinateValue[] key) {
                return this.validatesKeyConstraints(((SemanticSpaceCANOverlay) overlay).getZone(),
                        key);
            }

            public boolean validatesKeyConstraints(Zone zone, CoordinateValue[] key) {
                for (int i = 0; i < key.length; i++) {
                    // if coordinate is null we skip the test
                    if (key[i] != null) {
                        // the specified overlay does not contains the key
                        if (zone.contains(i, key[i]) != 0) {
                            return false;
                        }
                    }
                }
                return true;
            }
        };
        
        return new AnycastQueryRouter<TriplePatternQueryMessage>(validator) {
            public void onPeerWhichValidatesKeyConstraints(
                    AbstractCANOverlay overlay, AnycastQueryMessage msg) {
                
            };
        };
    }

}
