package fr.inria.gforge.dspace.overlay.messages.query.chord;

import org.objectweb.proactive.extensions.structuredp2p.messages.query.chord.FingerQueryMessage;
import org.objectweb.proactive.extensions.structuredp2p.messages.response.chord.FingerResponseMessage;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.chord.Entry;
import org.objectweb.proactive.extensions.structuredp2p.overlay.chord.Key;
import org.ontoware.rdf2go.model.node.URI;

import fr.inria.gforge.dspace.overlay.messages.response.chord.NbReferencesStoredResponseMessage;
import fr.inria.gforge.dspace.overlay.overlay.SemanticPeer;
import fr.inria.gforge.dspace.overlay.overlay.chord.SemanticResource;
import fr.inria.gforge.dspace.overlay.overlay.chord.SemanticSpaceChordOverlay;

/**
 * Message used to know the number of {@link SemanticPeer} stubs of type CAN
 * maintained by a {@link Peer} of type Chord for a specified <code>spaceURI</code>.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: NbReferencesStoredQueryMessage.java 4395 2010-04-28 18:22:30Z
 *          laurentp $
 */
public class NbReferencesStoredQueryMessage extends FingerQueryMessage {

    private static final long serialVersionUID = 1L;

    private int nbReferences;

    public NbReferencesStoredQueryMessage(URI spaceURI) {
        super(new Key(spaceURI.toString()));
    }

    public void onDestinationReached(StructuredOverlay overlay) {
        Entry<SemanticResource> entry = ((SemanticSpaceChordOverlay) overlay).getEntry(super
                .getKeyToReach());

        if (entry == null) {
            this.nbReferences = 0;
        } else {
            this.nbReferences = entry.getValue().getAll().size();
        }
    }

    public FingerResponseMessage createResponseMessage() {
        return new NbReferencesStoredResponseMessage(this, super.senderKey, this.nbReferences);
    }

}
