package fr.inria.gforge.dspace.overlay.messages.query.chord;

import org.objectweb.proactive.ActiveObjectCreationException;
import org.objectweb.proactive.api.PAActiveObject;
import org.objectweb.proactive.core.node.NodeException;
import org.objectweb.proactive.extensions.structuredp2p.messages.query.chord.FingerQueryMessage;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.chord.Key;
import org.ontoware.rdf2go.model.node.URI;

import fr.inria.gforge.dspace.overlay.api.SemanticFactory;
import fr.inria.gforge.dspace.overlay.kernel.SemanticSpaceOverlayKernel;
import fr.inria.gforge.dspace.overlay.overlay.SemanticPeer;
import fr.inria.gforge.dspace.overlay.overlay.can.SemanticSpaceCANOverlay;
import fr.inria.gforge.dspace.overlay.overlay.chord.SemanticSpaceChordOverlay;

/**
 * Perform {@link SemanticSpaceChordOverlay#put(URI, SemanticPeer)} when the
 * correct peer is reached with a {@link Peer} of type CAN. This message is used
 * by a {@link SemanticSpaceOverlayKernel} to add new {@link Peer} of type CAN
 * from a Chord network.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: NewPeerOnCANQueryMessage.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class NewPeerOnCANQueryMessage extends FingerQueryMessage {

    private static final long serialVersionUID = 1L;

    private URI spaceURI;

    private SemanticSpaceOverlayKernel remoteKernel;

    public NewPeerOnCANQueryMessage(URI spaceURI, SemanticSpaceOverlayKernel remoteKernel) {
        super(new Key(spaceURI.toString()));
        this.spaceURI = spaceURI;
        this.remoteKernel = remoteKernel;
    }

    public void onDestinationReached(StructuredOverlay overlay) {
        SemanticPeer peer = null;

        try {
            peer = SemanticFactory.newActiveSemanticPeer(
                            ((SemanticPeer) overlay.getLocalPeer()).getTrackers(),
                            new SemanticSpaceCANOverlay(spaceURI, this.remoteKernel),
                            PAActiveObject.getActiveObjectNode(this.remoteKernel));

            ((SemanticSpaceChordOverlay) overlay).put(spaceURI, peer);
        } catch (ActiveObjectCreationException e) {
            e.printStackTrace();
        } catch (NodeException e) {
            e.printStackTrace();
        }
    }

}
