
package fr.inria.gforge.dspace.overlay.messages.query.chord;

import org.objectweb.proactive.extensions.structuredp2p.messages.query.chord.FingerQueryMessage;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.chord.Key;
import org.ontoware.rdf2go.model.node.URI;

import fr.inria.gforge.dspace.overlay.kernel.SemanticSpaceOverlayKernel;
import fr.inria.gforge.dspace.overlay.overlay.SemanticPeer;
import fr.inria.gforge.dspace.overlay.overlay.chord.SemanticSpaceChordOverlay;

/**
 * Perform {@link SemanticSpaceChordOverlay#put(URI, SemanticPeer)} when the
 * correct peer is reached with a {@link Peer} of type CAN. This message is used
 * by a {@link SemanticSpaceOverlayKernel} to add new {@link Peer}s of type CAN
 * from a Chord network.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: NewPeersOnCANQueryMessage.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class NewPeersOnCANQueryMessage extends FingerQueryMessage {

    private static final long serialVersionUID = 1L;

    private URI spaceURI;

    private SemanticPeer[] remotePeers;

    public NewPeersOnCANQueryMessage(URI spaceURI, SemanticPeer[] remotePeers) {
        super(new Key(spaceURI.toString()));
        this.spaceURI = spaceURI;
        this.remotePeers = remotePeers;
    }

    public void onDestinationReached(StructuredOverlay overlay) {
        for (SemanticPeer peer : this.remotePeers) {
            ((SemanticSpaceChordOverlay) overlay).put(spaceURI, peer);
        }
    }

}
