package fr.inria.gforge.dspace.overlay.messages.response.can;

import java.io.IOException;

import org.objectweb.proactive.core.util.converter.ByteToObjectConverter;
import org.objectweb.proactive.extensions.structuredp2p.api.messages.Response;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.coordinates.CoordinateValue;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.api.messages.response.AskResponse;
import fr.inria.gforge.dspace.overlay.messages.query.can.SemanticQueryMessage;
import fr.inria.gforge.dspace.overlay.overlay.can.SemanticSpaceCANOverlay;
import fr.inria.gforge.dspace.overlay.router.can.AnycastResponseRouter;

/**
 * @author Laurent Pellegrino
 * 
 * @version $Id: AskResponseMessage.java 3576 2010-03-04 15:20:20Z laurentp
 *          $
 */
public class AskResponseMessage extends SemanticResponseMessage<Boolean> {

    private static final long serialVersionUID = 1L;

    private static final transient Logger logger = LoggerFactory
            .getLogger(AskResponseMessage.class);
    
    public AskResponseMessage(SemanticQueryMessage query,
            CoordinateValue[] coordinatesToReach,
            Boolean result) {
        super(query, coordinatesToReach);
        this.storeData(result);
        if (logger.isDebugEnabled()) {
            logger.debug("AskResponseMessage created for query {} with value set to {}", 
                    query.getUUID(), result);
        }
    }

    public AskResponseMessage(SemanticQueryMessage query, CoordinateValue[] coordinatesToReach) {
        super(query, coordinatesToReach);
        if (logger.isDebugEnabled()) {
            logger.debug("AskResponseMessage created for query {} with default value (false)",
                    query.getUUID());
        }
    }

    public AskResponseMessage(SemanticResponseMessage<?> response,
	    CoordinateValue[] keyToReach, Boolean result) {
	super (response, keyToReach);
	this.storeData(result);
    }

    public Boolean merge(Boolean data1, Boolean data2) {
        return data1 || data2;
    }

    public Boolean queryDataStore(StructuredOverlay overlay) {
        if (logger.isDebugEnabled()) {
            logger.debug("QueryDataStore on {} for ask query.", overlay);
        }
        try {
            return ((SemanticSpaceCANOverlay) overlay).getLocalSemanticSpaceOverlayKernel()
                    .sparqlAsk(super.getSparqlQuery().getSpaceURI(),
                            super.getSparqlQuery().toString());
        } catch (SemanticSpaceException e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * Redefined in order to merge data instead of storing intermediate results.
     * Indeed, an ask query response message returns one and only one result
     * which is a boolean. So the intermediate information can be merged in
     * order to avoid to have to carry some useless information.
     * 
     * @see AnycastResponseMessage#storeData(Object)
     */
    public void storeData(Boolean data) {
        if (super.dataRetrieved.size() == 0) {
            super.storeData(data);
        } else {
            try {
                super.storeData(
                        this.merge(
                                (Boolean) ByteToObjectConverter.ObjectStream.convert(
                                        super.dataRetrieved.remove(0).getBytes()), data));
            } catch (IOException e) {
                e.printStackTrace();
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            }
        }
    }

    public AnycastResponseRouter<? extends AnycastResponseMessage<Boolean>, Boolean> getRouter() {
        return new AnycastResponseRouter<SemanticResponseMessage<Boolean>, Boolean>(null);
    }

    public Response createResponse() {
        return new AskResponse(this);
    }

}
