package fr.inria.gforge.dspace.overlay.messages.response.can;

import org.objectweb.proactive.extensions.structuredp2p.api.messages.Response;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.coordinates.CoordinateValue;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import fr.inria.gforge.dspace.overlay.messages.query.can.SemanticQueryMessage;
import fr.inria.gforge.dspace.overlay.rdf2go.wrappers.ClosableIterableWrapper;
import fr.inria.gforge.dspace.overlay.router.can.AnycastResponseRouter;

/**
 * Construct and Describe SPARQL queries share the same result type.
 * 
 * @see ConstructResponseMessage
 * @see DescribeResponseMessage
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: ConstructResponseMessage.java 3652 2010-03-17 20:55:50Z
 *          laurentp $
 */
public abstract class ClosableIterableResponseMessage extends
        SemanticResponseMessage<ClosableIterableWrapper> {

    private static final long serialVersionUID = 1L;

    private static final transient Logger logger = 
        LoggerFactory.getLogger(ClosableIterableResponseMessage.class);

    public ClosableIterableResponseMessage(SemanticResponseMessage<?> response,
            CoordinateValue[] coordinatesToReach, ClosableIterableWrapper data) {
        super(response, coordinatesToReach);
        this.storeData(data);
        if (logger.isDebugEnabled()) {
            logger.debug("ClosableIterableResponseMessage created for query {} with data retrieved.",
                    response.getUUID());
        }
    }

    public ClosableIterableResponseMessage(SemanticQueryMessage query,
            CoordinateValue[] coordinatesToReach) {
        super(query, coordinatesToReach);
        if (logger.isDebugEnabled()) {
            logger.debug("ClosableiterableResponseMessage created for query {} with no data retrieved.", 
                    query.getUUID());
        }
    }

    /**
     * Stores the result only if it contains some data.
     * 
     * @see AnycastResponseMessage#storeData(Object)
     */
    public void storeData(ClosableIterableWrapper data) {
        if (data.getData().size() > 0) {
            super.storeData(data);
        }
    }
    
    /**
     * {@inheritDoc}
     */
    public ClosableIterableWrapper merge(
            ClosableIterableWrapper data1, ClosableIterableWrapper data2) {
        if (data1 == null) {
            throw new NullPointerException();
        }
        
        if (data2 != null) {
            data1.addAll(data2.getData());
        }
        
        if (logger.isDebugEnabled()) {
            StringBuffer buf = new StringBuffer();
            buf.append("New data have been merged, response contains ");
            buf.append(data1.getData().size());
            buf.append(" data.");
            logger.debug(buf.toString());
        }
        
        return data1;
    }

    public abstract ClosableIterableWrapper queryDataStore(StructuredOverlay overlay);

    public abstract Response createResponse();

    public AnycastResponseRouter<? extends 
            AnycastResponseMessage<ClosableIterableWrapper>, 
            ClosableIterableWrapper> getRouter() {
        return new AnycastResponseRouter
            <SemanticResponseMessage<ClosableIterableWrapper>, 
                ClosableIterableWrapper>(null);
    }

}
