package fr.inria.gforge.dspace.overlay.messages.response.can;

import org.objectweb.proactive.extensions.structuredp2p.api.messages.Response;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.coordinates.CoordinateValue;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.api.messages.query.ConstructQuery;
import fr.inria.gforge.dspace.overlay.api.messages.response.DescribeResponse;
import fr.inria.gforge.dspace.overlay.messages.query.can.SemanticQueryMessage;
import fr.inria.gforge.dspace.overlay.overlay.can.SemanticSpaceCANOverlay;
import fr.inria.gforge.dspace.overlay.rdf2go.wrappers.ClosableIterableWrapper;

/**
 * Response message for SPARQL query of type Construct.
 * 
 * @see AnycastResponseMessage
 * @see ConstructQuery
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: ConstructResponseMessage.java 3652 2010-03-17 20:55:50Z
 *          laurentp $
 */
public class DescribeResponseMessage extends
                    ClosableIterableResponseMessage {

    private static final long serialVersionUID = 1L;

    private static final transient Logger logger = 
        LoggerFactory.getLogger(DescribeResponseMessage.class);
    
    public DescribeResponseMessage(SemanticResponseMessage<?> response,
            CoordinateValue[] coordinatesToReach, ClosableIterableWrapper data) {
        super(response, coordinatesToReach, data);
    }

    public DescribeResponseMessage(SemanticQueryMessage query,
            CoordinateValue[] coordinatesToReach) {
        super(query, coordinatesToReach);
    }

    public Response createResponse() {
        return new DescribeResponse(this);
    }
    
    public ClosableIterableWrapper queryDataStore(StructuredOverlay overlay) {
        if (logger.isDebugEnabled()) {
            logger.debug("QueryDataStore on {} for describe query.", overlay);
        }

        try {
            return new ClosableIterableWrapper(((SemanticSpaceCANOverlay) overlay).
                    getLocalSemanticSpaceOverlayKernel().sparqlDescribe(
                           super.getSparqlQuery().getSpaceURI(), 
                           super.getSparqlQuery().toString()));
        } catch (SemanticSpaceException e) {
            e.printStackTrace();
        }

        return null;
    }

}
