package fr.inria.gforge.dspace.overlay.messages.response.can;

import org.objectweb.proactive.extensions.structuredp2p.api.messages.Response;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.coordinates.CoordinateValue;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.api.messages.response.SelectResponse;
import fr.inria.gforge.dspace.overlay.messages.query.can.AnycastQueryMessage;
import fr.inria.gforge.dspace.overlay.messages.query.can.SemanticQueryMessage;
import fr.inria.gforge.dspace.overlay.overlay.can.SemanticSpaceCANOverlay;
import fr.inria.gforge.dspace.overlay.rdf2go.wrappers.QueryResultTableWrapper;
import fr.inria.gforge.dspace.overlay.router.can.AnycastResponseRouter;

/**
 * @author Laurent Pellegrino
 * 
 * @version $Id: SelectResponseMessage.java 3652 2010-03-17 20:55:50Z
 *          laurentp $
 */
public class SelectResponseMessage extends
        SemanticResponseMessage<QueryResultTableWrapper> {

    private static final long serialVersionUID = 1L;
    
    private static final transient Logger logger = LoggerFactory
            .getLogger(SelectResponseMessage.class);

    public SelectResponseMessage(SemanticResponseMessage<?> response, 
            CoordinateValue[] coordinatesToReach,
            QueryResultTableWrapper queryResultTable) {
        super(response, coordinatesToReach);
        this.storeData(queryResultTable);
    }
    
    public SelectResponseMessage(AnycastQueryMessage query,
            CoordinateValue[] coordinatesToReach,
            QueryResultTableWrapper queryResultTable) {
        super((SemanticQueryMessage) query, coordinatesToReach);
        this.storeData(queryResultTable);
        if (logger.isDebugEnabled()) {
            logger.debug("SelectResponseMessage created with some results merged for query {}.", query.getUUID());
        }
    }

    public SelectResponseMessage(SemanticQueryMessage query,
            CoordinateValue[] coordinatesToReach) {
        super(query, coordinatesToReach);
        if (logger.isDebugEnabled()) {
            logger.debug("SelectResponseMessage created for query {}.", query.getUUID());
        }
    }

    /**
     * Stores the result only if it contains some data.
     * 
     * @see AnycastResponseMessage#storeData(Object)
     */
    public void storeData(QueryResultTableWrapper data) {
        if (data.getRows().size() > 0) {
            super.storeData(data);
        }
    }
    
    public QueryResultTableWrapper merge(QueryResultTableWrapper data1,
            QueryResultTableWrapper data2) {
        if (data1 == null) {
            throw new NullPointerException();
        }
        
        if (data2 != null) {
            data1.getRows().addAll(data2.getRows());
        }
        
        return data1;
    }
    
    public QueryResultTableWrapper queryDataStore(StructuredOverlay overlay) {
        QueryResultTableWrapper res = null;
        try {
            res = new QueryResultTableWrapper(((SemanticSpaceCANOverlay) overlay)
                              .getLocalSemanticSpaceOverlayKernel().sparqlSelect(
                                      super.getSparqlQuery().getSpaceURI(), 
                                      super.getSparqlQuery().toString()));
        } catch (SemanticSpaceException e) {
            e.printStackTrace();
        }

        if (logger.isDebugEnabled()) {
            logger.debug("QueryDataStore on {} for select query ({}).", overlay, res.getRows().size());
        }
        
        return res;
    }

    public AnycastResponseRouter<? extends 
                AnycastResponseMessage<QueryResultTableWrapper>, 
                    QueryResultTableWrapper> getRouter() {
        return new AnycastResponseRouter<
            SemanticResponseMessage<QueryResultTableWrapper>, 
            QueryResultTableWrapper>(null);
    }

    public Response createResponse() {
        return new SelectResponse(this);
    }

}
