package fr.inria.gforge.dspace.overlay.operations.chord;

import org.objectweb.proactive.extensions.structuredp2p.operations.BooleanResponseOperation;
import org.objectweb.proactive.extensions.structuredp2p.operations.Operation;
import org.objectweb.proactive.extensions.structuredp2p.operations.ResponseOperation;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.ontoware.rdf2go.model.node.URI;

import fr.inria.gforge.dspace.overlay.overlay.SemanticPeer;
import fr.inria.gforge.dspace.overlay.overlay.chord.SemanticSpaceChordOverlay;

/**
 * Add a new entry on the appropriate {@link Peer} (i.e. respecting Chord
 * property which says that entryToPut identified by key K will be managed by
 * the closest successor of K) by performing a lookup from the peer which
 * receives this message.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: PutOperation.java 5038 2010-08-19 09:09:50Z plaurent $
 */
@SuppressWarnings("serial")
public class PutOperation implements Operation {

    private URI spaceURI;

    private SemanticPeer canPeer;

    /**
     * Constructs a new put message by using the specified spaceURI and peer
     * reference.
     * 
     * @param spaceURI
     *            the spaceURI associated to the peer reference to store.
     * 
     * @param canPeer
     *            the peer reference to add.
     */
    public PutOperation(URI spaceURI, SemanticPeer canPeer) {
        this.spaceURI = spaceURI;
        this.canPeer = canPeer;
    }

    /**
     * {@inheritDoc}
     */
    public ResponseOperation handle(StructuredOverlay overlay) {
        return new BooleanResponseOperation(((SemanticSpaceChordOverlay) overlay).put(this.spaceURI,
                this.canPeer));
    }

}
