package fr.inria.gforge.dspace.overlay.rdf2go.wrappers;

import java.util.ArrayList;
import java.util.List;

import org.ontoware.aifbcommons.collection.ClosableIterator;
import org.ontoware.rdf2go.model.QueryResultTable;
import org.ontoware.rdf2go.model.QueryRow;

import fr.inria.gforge.dspace.overlay.util.SemanticHelper;

/**
 * A serializable object for wrapping not serializable {@link QueryResultTable}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: QueryResultTableWrapper.java 3470 2010-02-24 10:09:53Z laurentp
 *          $
 */
public class QueryResultTableWrapper implements RDF2GoWrapper<QueryResultTable> {

    private static final long serialVersionUID = 1L;

    private List<String> variables;

    private List<QueryRowWrapper> rows = new ArrayList<QueryRowWrapper>();

    public QueryResultTableWrapper(QueryResultTable qrt) {
        this.variables = qrt.getVariables();
        ClosableIterator<QueryRow> it = qrt.iterator();
        while (it.hasNext()) {
            this.rows.add(new QueryRowWrapper(it.next(), this.variables));
        }
    }

    public List<QueryRowWrapper> getRows() {
        return this.rows;
    }

    public QueryResultTable toRDF2Go() {
        List<QueryRow> queryRows = new ArrayList<QueryRow>();
        for (QueryRowWrapper qrw : this.rows) {
            queryRows.add(qrw.toRDF2Go());
        }

        return SemanticHelper.generateQueryResultTable(this.variables, queryRows);
    }

}
