package fr.inria.gforge.dspace.overlay.reasoner;

import org.openrdf.query.MalformedQueryException;
import org.openrdf.query.algebra.helpers.QueryModelVisitorBase;
import org.openrdf.query.parser.ParsedQuery;
import org.openrdf.query.parser.sparql.SPARQLParser;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceQueryException;
import fr.inria.gforge.dspace.overlay.api.messages.query.SPARQLQuery;
import fr.inria.gforge.dspace.overlay.messages.query.can.SemanticQueryMessage;
import fr.inria.gforge.dspace.overlay.reasoner.visitor.QueryIdentifierVisitor;

/**
 * Try to identify Sparql query type by using Sesame Visitor pattern.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: SPARQLQueryIdentifier.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class SPARQLQueryIdentifier extends QueryModelVisitorBase<RuntimeException> {

    public static SPARQLQueryIdentifier instance;

	/**
	 * Creates a new {@link SemanticQueryMessage} from a {@link SPARQLQuery}.
	 * 
	 * @param query
	 *            the {@link SPARQLQuery} from the public API to use.
	 * @return a new {@link SemanticQueryMessage} from the private API.
	 * 
	 * @throws QueryParseException
	 *             if query is malformed or if it has an unknown type.
	 */
	public static SemanticQueryMessage createQueryMessage(SPARQLQuery query)
			throws SemanticSpaceQueryException {
		return SPARQLQueryIdentifier.getInstance().newQueryMessageFrom(query);
	}

	private SemanticQueryMessage newQueryMessageFrom(SPARQLQuery query)
			throws SemanticSpaceQueryException {
		SPARQLParser parser = new SPARQLParser();
        ParsedQuery pq = null;

        try {
            pq = parser.parseQuery(query.toString(), null);
        } catch (MalformedQueryException e) {
           throw new SemanticSpaceQueryException(e);
        }

        
        QueryIdentifierVisitor visitor = new QueryIdentifierVisitor();
        pq.getTupleExpr().visit(visitor);
        return visitor.processIdentificationAndCreateQueryMessage(query);
	}
	
	private static SPARQLQueryIdentifier getInstance() {
		if (SPARQLQueryIdentifier.instance == null) {
			SPARQLQueryIdentifier.instance = new SPARQLQueryIdentifier();
		}
		return SPARQLQueryIdentifier.instance;
	}
    
}
