package fr.inria.gforge.dspace.overlay.reasoner;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import org.openrdf.query.MalformedQueryException;
import org.openrdf.query.QueryLanguage;
import org.openrdf.query.algebra.StatementPattern;
import org.openrdf.query.algebra.Var;
import org.openrdf.query.parser.ParsedQuery;
import org.openrdf.query.parser.QueryParserUtil;

import fr.inria.gforge.dspace.overlay.api.messages.query.ConstructQuery;
import fr.inria.gforge.dspace.overlay.api.messages.query.SPARQLQuery;
import fr.inria.gforge.dspace.overlay.reasoner.visitor.StatementPatternsVisitor;

/**
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: SPARQLQueryReasoner.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class SPARQLQueryReasoner implements Serializable {

	private static final long serialVersionUID = 1L;

	private static SPARQLQueryReasoner instance;
	
	public ParsedSPARQLQuery decompose(SPARQLQuery query)
			throws MalformedQueryException {
		return this.decomposeIfNecessary(
				query,
				QueryParserUtil.parseQuery(QueryLanguage.SPARQL,
						query.toString(), null));
	}

	private ParsedSPARQLQuery decomposeIfNecessary(SPARQLQuery query,
			ParsedQuery parsedQuery) {
		StatementPatternsVisitor visitor = new StatementPatternsVisitor();
		parsedQuery.getTupleExpr().visit(visitor);

		List<SPARQLQuery> subQueries;

		// The query has been decomposed into one or several construct
		// sub-queries
		if (visitor.requireFiltration()) {
			subQueries = new ArrayList<SPARQLQuery>(visitor
					.getStatementPatterns().size());
			for (StatementPattern pattern : visitor.getStatementPatterns()) {
				subQueries.add(new ConstructQuery(query.getSpaceURI(), this
						.createConstructQueryFrom(pattern)));
			}
		} else {
			subQueries = new ArrayList<SPARQLQuery>(1);
			subQueries.add(query);
		}

		return new ParsedSPARQLQuery(visitor.requireFiltration(), query,
				subQueries);
	}

	public String createConstructQueryFrom(StatementPattern pattern) {
		String subject = asString(pattern.getSubjectVar());
		String predicate = asString(pattern.getPredicateVar());
		String object = asString(pattern.getObjectVar());

		StringBuffer graphQuery = new StringBuffer(64);
		graphQuery.append("CONSTRUCT { ");
		graphQuery.append(subject);
		graphQuery.append(" ");
		graphQuery.append(predicate);
		graphQuery.append(" ");
		graphQuery.append(object);
		graphQuery.append(" } WHERE { ");
		graphQuery.append(subject);
		graphQuery.append(" ");
		graphQuery.append(predicate);
		graphQuery.append(" ");
		graphQuery.append(object);
		graphQuery.append(" }");

		return graphQuery.toString();
	}

	private static String asString(Var var) {
		if (var.getName().startsWith("-anon-")) {
			// Blank node
			return "_:" + var.getName().replaceAll("-", "");
		} else if (var.getValue() == null) {
			// Variable
			return "?" + var.getName();
		} else if (var.getValue().toString().startsWith("\"")) {
			// Literal
			return var.getValue().toString();
		} else {
			// IRI
			return "<" + var.getValue().stringValue() + ">";
		}
	}
	
	public static SPARQLQueryReasoner getInstance() {
		if (instance == null) {
			instance = new SPARQLQueryReasoner();
		}
		
		return instance;
	}

}
