package fr.inria.gforge.dspace.overlay.util;


/**
 * A set of utility operations that provide necessary information about the
 * architecture of the machine that the system is running on. The values
 * provided are automatically determined at JVM startup.
 * 
 * @author <a href="mailto:dev@avalon.apache.org">Avalon Development Team</a>
 * @author Laurent Pellegrino
 * 
 * @version $Id: SystemUtil.java 5041 2010-08-19 13:20:59Z plaurent $
 */
public final class SystemUtil {

	private static final int numProcessors;

	private static final String architecture;

	private static final String osName;

	private static final String osVersion;

	private static final boolean windows;

	private static final boolean linux;

	static {
		architecture = System.getProperty("os.arch");
		osName = System.getProperty("os.name");
		osVersion = System.getProperty("os.version");
		numProcessors = Runtime.getRuntime().availableProcessors();
		windows = SystemUtil.operatingSystem().startsWith("Windows");
		linux = SystemUtil.operatingSystem().startsWith("Linux");
	}

	/**
	 * Keep utility from being instantiated.
	 */
	private SystemUtil() {

	}

	public static final String architecture() {
		return architecture;
	}
	
	/**
	 * Return the number of processors available on this machine. This is useful
	 * in classes like Thread/Processor thread pool models.
	 */
	public static final int numProcessors() {
		return numProcessors;
	}

	/**
	 * Return the Operating System name.
	 */
	public static final String operatingSystem() {
		return osName;
	}

	/**
	 * Return the Operating System version.
	 */
	public static final String osVersion() {
		return osVersion;
	}

	/**
	 * Return <code>true</code> if running on Microsoft Windows.
	 */
	public static final boolean isWindows() {
		return windows;
	}

	/**
	 * Return <code>true</code> if running on Linux.
	 */
	public static final boolean isLinux() {
		return linux;
	}

}
