package fr.inria.gforge.dspace.overlay.validator.can;

import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.AbstractCANOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.Zone;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.coordinates.CoordinateValue;
import org.objectweb.proactive.extensions.structuredp2p.validator.ConstraintsValidator;

import fr.inria.gforge.dspace.overlay.messages.query.can.RemoveStatementsQueryMessage;
import fr.inria.gforge.dspace.overlay.messages.response.can.RemoveStatementsResponseMessage;
import fr.inria.gforge.dspace.overlay.validator.AnycastConstraintsValidator;

/**
 * {@link ConstraintsValidator} for {@link RemoveStatementsQueryMessage} and
 * {@link RemoveStatementsResponseMessage}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: RemoveStatementsConstraintsValidator.java 5041 2010-08-19 13:20:59Z plaurent $
 */
public class RemoveStatementsConstraintsValidator implements AnycastConstraintsValidator<CoordinateValue[]> {

    private static final long serialVersionUID = 1L;

    public boolean validatesKeyConstraints(StructuredOverlay overlay, CoordinateValue[] key) {
        return this.validatesKeyConstraints(((AbstractCANOverlay) overlay).getZone(), key);
    }

   public boolean validatesKeyConstraints(Zone zone, CoordinateValue[] key) {
        for (int i = 0; i < key.length; i++) {
            // if coordinate is null we skip the test
            if (key[i] != null) {
                // the specified overlay does not contains the key
                if (zone.contains(i, key[i]) != 0) {
                    return false;
                }
            }
        }
        return true;
    }
    
}
