package fr.inria.gforge.dspace.overlay.core.messages.chord;

import junit.framework.Assert;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.objectweb.proactive.api.PAFuture;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.initializer.GlobalTestsInitializer;
import fr.inria.gforge.dspace.overlay.messages.query.chord.NbReferencesStoredQueryMessage;
import fr.inria.gforge.dspace.overlay.messages.response.chord.NbReferencesStoredResponseMessage;
import fr.inria.gforge.dspace.overlay.util.RDF2GoBuilder;

/**
 * Provides tests for {@link NbReferencesStoredQueryMessage}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: TestNbReferencesStoredMessage.java 4395 2010-04-28 18:22:30Z
 *          laurentp $
 */
public class TestNbReferencesStoredMessage {

    private static final transient Logger logger = 
        LoggerFactory.getLogger(TestNbReferencesStoredMessage.class);

    private static GlobalTestsInitializer initializer = 
        new GlobalTestsInitializer(
                RDF2GoBuilder.createURI("http://www.inria.fr"));

    @BeforeClass
    public static void setUp() {
        initializer.setUpNetworkOnLocalMachine(3);
        try {
            initializer.createSpace(initializer.getDefaultSpaceURI());
            initializer.newPeersOnCANInParallel(initializer.getDefaultSpaceURI(), 5);
        } catch (SemanticSpaceException e) {
            e.printStackTrace();
        }
    }

    @Test
    public void testMessage() {
        int nbReferences = ((NbReferencesStoredResponseMessage) PAFuture
                .getFutureValue(initializer.getRandomTracker().getRandomPeer().send(
                        new NbReferencesStoredQueryMessage(initializer.getDefaultSpaceURI()))))
                .getNbReferences();

        if (logger.isDebugEnabled()) {
            logger.debug("{} reference(s) stored by a Chord node for space '{}'", 
                    nbReferences, initializer.getDefaultSpaceURI());
        }
        Assert.assertTrue(nbReferences > 0);
    }

    @AfterClass
    public static void tearDown() {
        initializer.tearDownNetwork();
    }

}
