package fr.inria.gforge.dspace.overlay.core.overlay.can;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import junit.framework.Assert;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.objectweb.proactive.ActiveObjectCreationException;
import org.objectweb.proactive.api.PAFuture;
import org.objectweb.proactive.core.node.NodeException;
import org.objectweb.proactive.extensions.structuredp2p.exceptions.StructuredP2PException;
import org.ontoware.rdf2go.model.Statement;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import fr.inria.gforge.dspace.overlay.api.SemanticFactory;
import fr.inria.gforge.dspace.overlay.initializers.SpaceNetworkInitializer;
import fr.inria.gforge.dspace.overlay.kernel.operations.datastore.DatastoreResponseOperation;
import fr.inria.gforge.dspace.overlay.kernel.operations.datastore.SPARQLConstructOperation;
import fr.inria.gforge.dspace.overlay.overlay.SemanticPeer;
import fr.inria.gforge.dspace.overlay.overlay.can.SemanticSpaceCANOverlay;
import fr.inria.gforge.dspace.overlay.rdf2go.wrappers.ClosableIterableWrapper;
import fr.inria.gforge.dspace.overlay.util.RDF2GoBuilder;
import fr.inria.gforge.dspace.overlay.util.SemanticHelper;

/**
 * Test data transfert for {@link SemanticSpaceCANOverlay}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: TestDataTransfert.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class TestDataTransfert {

    private final static Logger logger = LoggerFactory.getLogger(TestDataTransfert.class);

    private static SpaceNetworkInitializer initializer =
            new SpaceNetworkInitializer(
                    RDF2GoBuilder.createURI("http://www.inria.fr"));

    private static String[][] statementsToAdd = {
            { "http://A", "http://A", "http://A" },
            { "http://U", "http://U", "http://U" },
            { "http://Z", "http://Z", "http://Z" } };

    @BeforeClass
    public static void setUp() {
        initializer.setUpNetworkOnLocalMachine(1);
    }

    @Test
    public void testAddStatements() {
        StringBuffer query = new StringBuffer();
        for (String[] stmt : statementsToAdd) {
            initializer.addStatement(stmt[0], stmt[1], stmt[2]);
        }

        query.append("CONSTRUCT { ?s ?p ?o } WHERE { GRAPH<");
        query.append(initializer.getSpaceURI().toString());
        query.append("> { ?s ?p ?o }. }");

        Set<Statement> response = initializer.sparqlConstruct(query.toString());
        Assert.assertEquals(statementsToAdd.length, response.size());
        
        this.testDataTransfert();
    }

    @SuppressWarnings("unchecked")
    public void testDataTransfert() {
        SemanticPeer newPeer = null;
        try {
            newPeer = SemanticFactory.newActiveSemanticPeer(
                    Arrays.asList(initializer.getTrackers()),
                    new SemanticSpaceCANOverlay(
                            initializer.getSpaceURI(),
                            SemanticFactory.newActiveSemanticSpaceOverlayKernel(
                                    initializer.getTrackers(), true)));
        } catch (ActiveObjectCreationException e) {
            e.printStackTrace();
        } catch (NodeException e) {
            e.printStackTrace();
        }

        SemanticPeer oldPeer = (SemanticPeer) PAFuture.getFutureValue(
                                initializer.getRandomTracker().getRandomPeer());
        
        try {
            initializer.getRandomTracker().addOnNetwork(newPeer);
        } catch (StructuredP2PException e) {
            e.printStackTrace();
        }
        
        if (logger.isDebugEnabled()) {
            logger.debug("Initial peer manages "
                    + initializer.getRandomTracker().getRandomPeer());
            logger.debug("New peer manages " + newPeer);
        }

        Set<Statement> resultFromPeerA = SemanticHelper.asSet(
                    ((DatastoreResponseOperation<ClosableIterableWrapper>)PAFuture.getFutureValue((SemanticSpaceCANOverlay) PAFuture.getFutureValue(
                            oldPeer.getStructuredOverlay())).
                                    getRemoteSemanticSpaceOverlayKernel().send(new SPARQLConstructOperation(initializer.getSpaceURI(),
                                        "CONSTRUCT { ?s ?p ?o } WHERE { GRAPH<"
                                        + initializer.getSpaceURI() + "> { ?s ?p ?o } . } "))).getValue().toRDF2Go());

        Set<Statement> resultFromPeerB = SemanticHelper.asSet(
                    ((DatastoreResponseOperation<ClosableIterableWrapper>)PAFuture.getFutureValue((SemanticSpaceCANOverlay) PAFuture.getFutureValue(
                            newPeer.getStructuredOverlay())).
                                    getRemoteSemanticSpaceOverlayKernel().send(new SPARQLConstructOperation(initializer.getSpaceURI(),
                                        "CONSTRUCT { ?s ?p ?o } WHERE { GRAPH<"
                                        + initializer.getSpaceURI() + "> { ?s ?p ?o } . } "))).getValue().toRDF2Go());

        // Perform intersection between the previous results
        Set<Statement> newSet = new HashSet<Statement>(resultFromPeerA);
        newSet.retainAll(resultFromPeerB);

        if (logger.isDebugEnabled()) {
            logger.debug("Initial peer contains " + resultFromPeerA.size() + " data");
            logger.debug("New peer contains " + resultFromPeerB.size() + " data");
        }

        Assert.assertEquals(0, newSet.size());
        Assert.assertEquals(statementsToAdd.length, resultFromPeerA.size() + resultFromPeerB.size());
        Assert.assertTrue(resultFromPeerA.size() < statementsToAdd.length);
    }

    @AfterClass
    public static void tearDown() {
        initializer.tearDownNetwork();
    }

}
