package fr.inria.gforge.dspace.overlay.core.query;

import junit.framework.Assert;

import org.junit.Test;
import org.ontoware.rdf2go.model.node.URI;
import org.ontoware.rdf2go.model.node.impl.URIImpl;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceQueryException;
import fr.inria.gforge.dspace.overlay.api.messages.query.AskQuery;
import fr.inria.gforge.dspace.overlay.api.messages.query.ConstructQuery;
import fr.inria.gforge.dspace.overlay.api.messages.query.SelectQuery;
import fr.inria.gforge.dspace.overlay.messages.query.can.TriplePatternQueryMessage;
import fr.inria.gforge.dspace.overlay.reasoner.SPARQLQueryIdentifier;

/**
 * Test {@link SparqlQueryIdentifier}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: TestSparqlQueryIdentifier.java 3682 2010-03-22 14:36:40Z
 *          laurentp $
 */
public class TestSPARQLQueryIdentifier {

    private static final URI DEFAULT_SPACE_URI = new URIImpl("http://www.inria.fr");

//    @Test
//    public void testRangeQueryCreation1() throws SemanticSpaceQueryException {
//        RangeQueryMessage msg = 
//        	(RangeQueryMessage) SPARQLQueryIdentifier.createQueryMessage(
//        			new SelectQuery(
//                        DEFAULT_SPACE_URI,
//                        "SELECT ?s ?p ?o WHERE { ?s ?p ?o. FILTER ( str(?o) > \"http://aaa\" && str(?o) < \"http://bbb\" ). }"));
//
//        for (RangeQueryCondition condition : msg.getConditions()) {
//            Assert.assertEquals("o", condition.getVar());
//        }
//    }

//    @Test
//    public void testRangeQueryCreation2() throws SemanticSpaceQueryException {
//        RangeQueryMessage msg = (RangeQueryMessage) SPARQLQueryIdentifier
//                .createQueryMessage(new SelectQuery(
//                        DEFAULT_SPACE_URI,
//                        "CONSTRUCT { ?s ?p ?o } WHERE { ?s ?p ?o . FILTER ( str(?s) > \"0\" && str(?s) < \"U\" && str(?p) > \"U\" && str(?p) < \"{\" && str(?o) > \"0\" && str(?o) < \"U\"). }"));
//
//        Assert.assertTrue(msg instanceof RangeQueryMessage);
//    }

    @Test
    public void testTriplePatternQueryCreation1() throws SemanticSpaceQueryException {
        Assert
                .assertTrue(SPARQLQueryIdentifier.createQueryMessage(new SelectQuery(DEFAULT_SPACE_URI,
                        "SELECT ?s ?p ?o WHERE { <http://toto> <http://aa> <http://aa>. }")) instanceof TriplePatternQueryMessage);
    }

    @Test
    public void testTriplePatternQueryCreation2() throws SemanticSpaceQueryException {
        Assert
                .assertTrue(SPARQLQueryIdentifier.createQueryMessage(new SelectQuery(DEFAULT_SPACE_URI,
                        "SELECT ?s ?p ?o WHERE { ?s <http://aa> <http://aa>. }")) instanceof TriplePatternQueryMessage);
    }

    @Test
    public void testTriplePatternQueryCreation3() throws SemanticSpaceQueryException {
        Assert
                .assertTrue(SPARQLQueryIdentifier.createQueryMessage(new SelectQuery(DEFAULT_SPACE_URI,
                        "SELECT ?s ?p ?o WHERE { <http://toto> ?p <http://aa>. }")) instanceof TriplePatternQueryMessage);
    }

    @Test
    public void testTriplePatternQueryCreation4() throws SemanticSpaceQueryException {
        Assert
                .assertTrue(SPARQLQueryIdentifier.createQueryMessage(new SelectQuery(DEFAULT_SPACE_URI,
                        "SELECT ?s ?p ?o WHERE { <http://toto> <http://aa> ?o. }")) instanceof TriplePatternQueryMessage);
    }

    @Test
    public void testTriplePatternQueryCreation5() throws SemanticSpaceQueryException {
        Assert
                .assertTrue(SPARQLQueryIdentifier.createQueryMessage(new SelectQuery(DEFAULT_SPACE_URI,
                        "SELECT ?s ?p ?o WHERE { ?s ?p <http://aa>. }")) instanceof TriplePatternQueryMessage);
    }

    @Test
    public void testTriplePatternQueryCreation6() throws SemanticSpaceQueryException {
        Assert.assertTrue(SPARQLQueryIdentifier.createQueryMessage(new SelectQuery(DEFAULT_SPACE_URI,
                "SELECT ?s ?p ?o WHERE { ?s ?p ?o. }")) instanceof TriplePatternQueryMessage);
    }

    @Test
    public void testTriplePatternQueryCreation7() throws SemanticSpaceQueryException {
        Assert
                .assertTrue(SPARQLQueryIdentifier.createQueryMessage(new SelectQuery(DEFAULT_SPACE_URI,
                        "SELECT ?s ?p ?o WHERE { <http://toto> ?p ?o. }")) instanceof TriplePatternQueryMessage);
    }

    @Test
    public void testTriplePatternQueryCreation8() throws SemanticSpaceQueryException {
        Assert
                .assertTrue(SPARQLQueryIdentifier.createQueryMessage(new ConstructQuery(DEFAULT_SPACE_URI,
                        "CONSTRUCT { ?s ?p ?o } WHERE { <http://toto> <http://aa> <http://aa>. }")) instanceof TriplePatternQueryMessage);
    }

    @Test
    public void testTriplePatternQueryCreation9() throws SemanticSpaceQueryException {
        Assert
                .assertTrue(SPARQLQueryIdentifier.createQueryMessage(new ConstructQuery(DEFAULT_SPACE_URI,
                        "CONSTRUCT { ?s ?p ?o } WHERE { ?s ?p ?o. }")) instanceof TriplePatternQueryMessage);
    }

    @Test
    public void testTriplePatternQueryCreation10() throws SemanticSpaceQueryException {
        Assert.assertTrue(SPARQLQueryIdentifier.createQueryMessage(new AskQuery(DEFAULT_SPACE_URI,
                "ASK { ?s ?p ?o. }")) instanceof TriplePatternQueryMessage);
    }

    @Test
    public void testTriplePatternQueryCreation11() throws SemanticSpaceQueryException {
        Assert
                .assertTrue(SPARQLQueryIdentifier.createQueryMessage(new AskQuery(DEFAULT_SPACE_URI,
                        "ASK { GRAPH<http://hello-world> { ?s ?p ?o }. }")) instanceof TriplePatternQueryMessage);
    }

    @Test(expected = SemanticSpaceQueryException.class)
    public void testTriplePatternQueryCreationFailureWithAsk() throws SemanticSpaceQueryException {
        Assert.assertTrue(SPARQLQueryIdentifier.createQueryMessage(new AskQuery(DEFAULT_SPACE_URI,
                "ASK { ?s ?p . }")) instanceof TriplePatternQueryMessage);
    }

    @Test(expected = SemanticSpaceQueryException.class)
    public void testTriplePatternQueryCreationFailureWithConstruct() throws SemanticSpaceQueryException {
        Assert
                .assertTrue(SPARQLQueryIdentifier.createQueryMessage(new ConstructQuery(DEFAULT_SPACE_URI,
                        "CONSTRUCT { ?s ?p ?o } WHERE { <http://toto> <http://aa> http://c }")) instanceof TriplePatternQueryMessage);
    }

    @Test(expected = SemanticSpaceQueryException.class)
    public void testTriplePatternQueryCreationFailureWithSelect() throws SemanticSpaceQueryException {
        Assert.assertTrue(SPARQLQueryIdentifier.createQueryMessage(new SelectQuery(DEFAULT_SPACE_URI,
                "SELECT { ?s ?p ?o } WHERE { ?s ?p ?o. }")) instanceof TriplePatternQueryMessage);
    }

}
