package fr.inria.gforge.dspace.overlay.core.query;

import junit.framework.Assert;

import org.junit.Test;
import org.ontoware.rdf2go.model.node.URI;
import org.openrdf.query.MalformedQueryException;

import fr.inria.gforge.dspace.overlay.api.messages.query.AskQuery;
import fr.inria.gforge.dspace.overlay.api.messages.query.ConstructQuery;
import fr.inria.gforge.dspace.overlay.api.messages.query.SelectQuery;
import fr.inria.gforge.dspace.overlay.reasoner.ParsedSPARQLQuery;
import fr.inria.gforge.dspace.overlay.reasoner.SPARQLQueryReasoner;
import fr.inria.gforge.dspace.overlay.util.RDF2GoBuilder;

/**
 * Test {@link SPARQLQueryReasoner}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: TestSPARQLQueryReasoner.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class TestSPARQLQueryReasoner {

    private static final URI DEFAULT_SPACE_URI = RDF2GoBuilder.createURI("http://www.inria.fr");

    @Test
    public void testAskDecomposition() throws MalformedQueryException {
        ParsedSPARQLQuery parsedQuery = 
        	SPARQLQueryReasoner.getInstance().decompose(
        			new AskQuery(DEFAULT_SPACE_URI, "ASK { ?s ?p ?o. }"));

        Assert.assertEquals(1, parsedQuery.getSubQueries().size());
        Assert.assertEquals("ASK { ?s ?p ?o. }", parsedQuery.getSubQueries().get(0).toString());
    }

    @Test
    public void testConstructDecomposition() throws MalformedQueryException {
    	ParsedSPARQLQuery parsedQuery = 
        	SPARQLQueryReasoner.getInstance().decompose(
        			new ConstructQuery(
                            DEFAULT_SPACE_URI,
                            "CONSTRUCT { ?s ?p <http://test> }  WHERE { ?s ?p ?o. FILTER (str(?s) >= \"test\" && ?p < 5). }"));

        Assert.assertEquals(1, parsedQuery.getSubQueries().size());
        Assert.assertEquals("CONSTRUCT { ?s ?p ?o } WHERE { ?s ?p ?o }",
                			parsedQuery.getSubQueries().get(0).toString());
    }

    @Test
    public void testSelectDecomposition() throws MalformedQueryException {
    	ParsedSPARQLQuery parsedQuery = 
        	SPARQLQueryReasoner.getInstance().decompose(
        			new SelectQuery(
        					DEFAULT_SPACE_URI,
                    		"SELECT ?nick WHERE { ?x <http://mbox> <mailto:bob@work.example> . ?x <http://nick> ?nick }"));

        Assert.assertEquals(2, parsedQuery.getSubQueries().size());
    }

}
