package fr.inria.gforge.dspace.overlay.initializer;

import org.ontoware.rdf2go.model.node.URI;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.api.SemanticSpaceManagement;
import fr.inria.gforge.dspace.overlay.connector.ManagementConnector;
import fr.inria.gforge.dspace.overlay.initializers.GlobalNetworkInitializer;
import fr.inria.gforge.dspace.overlay.kernel.SemanticSpaceOverlayKernel;
import fr.inria.gforge.dspace.overlay.tracker.SemanticChordTracker;

/**
 * Provides some shortcuts for a {@link GlobalNetworkInitializer} and delegates
 * methods to a {@link ManagementConnector} in order to avoid code duplication
 * in tests initialization.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: GlobalTestsInitializer.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class GlobalTestsInitializer extends TestsInitializer implements SemanticSpaceManagement {

    private GlobalNetworkInitializer initializer = new GlobalNetworkInitializer();
    
    private ManagementConnector connector;
    
    public GlobalTestsInitializer(URI defaultSpaceURI) {
        super(defaultSpaceURI);
    }

    public void setUpNetworkOnLocalMachine(int nbChordPeers) {
        this.initializer.setUpNetworkOnLocalMachine();
        this.connector = this.initializer.getManagementConnector();
        this.connector.setAutoRemoveDatastoreAtShutdown(true);
        this.connector.newPeersOnChordInParallel(nbChordPeers);
    }

    public void newPeerOnChord() {
        this.connector.newPeerOnChord();
    }

    public void newPeersOnChordInParallel(int nb) {
        this.connector.newPeersOnChordInParallel(nb);
    }

    public void createSpace(URI spaceURI) throws SemanticSpaceException {
        this.connector.createSpace(spaceURI);
    }

    public void createSpace(URI spaceURI, SemanticSpaceOverlayKernel kernel)
            throws SemanticSpaceException {
        this.connector.createSpace(spaceURI, kernel);
    }

    public void newPeerOnCAN(URI spaceURI) throws SemanticSpaceException {
        this.connector.newPeerOnCAN(spaceURI);
    }

    public void newPeersOnCANInParallel(URI spaceURI, int nb) throws SemanticSpaceException {
        this.connector.newPeersOnCANInParallel(spaceURI, nb);
    }

    public void newPeerOnCAN(URI spaceURI, SemanticSpaceOverlayKernel kernel)
            throws SemanticSpaceException {
        this.connector.newPeerOnCAN(spaceURI, kernel);
    }

    public void removePeerOnChord() throws SemanticSpaceException {
        this.connector.removePeerOnChord();
    }

    public void removePeerOnCAN(URI spaceURI) throws SemanticSpaceException {
        this.connector.removePeerOnCAN(spaceURI);
    }

    public boolean spaceExists(URI spaceURI) throws SemanticSpaceException {
        return this.connector.spaceExists(spaceURI);
    }

    public SemanticChordTracker getRandomTracker() {
        return this.initializer.getRandomTracker();
    }

    public SemanticChordTracker[] getTrackers() {
        return this.initializer.getTrackers();
    }
    
    public void tearDownNetwork() {
        this.initializer.tearDownNetwork();
    }
    
}
