package fr.inria.gforge.dspace.overlay.samples.commands;

import java.util.ArrayList;
import java.util.List;

import fr.inria.gforge.dspace.overlay.samples.managers.Manager;

/**
 * A Command is an operation that a {@link Manager} can execute.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: Command.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public abstract class Command {

    private String argumentsLine = "";

    private List<Argument> commandArguments;

    private List<String> commandShortcuts;

    private String description;

    private String name;

    private Manager manager;

    public Command(Manager manager, String description, String[] commandShortcuts) {
        this.commandArguments = new ArrayList<Argument>();
        this.commandShortcuts = new ArrayList<String>();
        this.description = description;
        this.manager = manager;

        for (String shortcut : commandShortcuts) {
            this.commandShortcuts.add(shortcut);
        }
    }

    public Command(Manager manager, String description, String[] commandShortcuts,
            List<Argument> commandArguments) {
        this(manager, description, commandShortcuts);
        this.commandArguments = commandArguments;
    }

    public abstract void execute(String... args);

    public List<String> getCommandShortcuts() {
        return this.commandShortcuts;
    }

    public String getDescription() {
        return this.description;
    }

    public Manager getManager() {
        return this.manager;
    }

    public int getNbArguments() {
        return this.commandArguments.size();
    }

    public int getNbRequiredArguments() {
        int nb = 0;
        for (Argument arg : this.commandArguments) {
            if (!arg.isOptional()) {
                nb++;
            }
        }
        return nb;
    }

    public int getNbOptionalArguments() {
        return this.getNbArguments() - this.getNbRequiredArguments();
    }

    public List<Argument> getCommandArguments() {
        return this.commandArguments;
    }

    public String getArgumentsLine() {
        return this.argumentsLine;
    }

    public String getName() {
        return this.name;
    }

    public void setArgumentsLine(String argumentsLine) {
        this.argumentsLine = argumentsLine;
    }

    public void info(String msg) {
        System.out.print("*** ");
        System.out.println(msg);
    }

    public void error(String msg) {
        StringBuffer buf = new StringBuffer("-!- ");
        buf.append(msg);
        System.out.println(buf.toString());
    }

}
