package fr.inria.gforge.dspace.overlay.samples.commands.management;

import java.util.Arrays;

import org.objectweb.proactive.api.PAActiveObject;
import org.ontoware.rdf2go.model.node.URI;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.kernel.SemanticSpaceOverlayKernel;
import fr.inria.gforge.dspace.overlay.samples.commands.Argument;
import fr.inria.gforge.dspace.overlay.samples.managers.Manager;
import fr.inria.gforge.dspace.overlay.util.RDF2GoBuilder;

/**
 * Creates a new space by using the specified spaceURI.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: CreateSpaceCommand.java 5042 2010-08-19 14:00:48Z plaurent $
 */
public class CreateSpaceCommand extends AbstractManagementCommand {

    /**
     * Constructor.
     * 
     * @param manager
     *            the manager that will execute this command.
     */
    public CreateSpaceCommand(Manager manager) {
        super(manager, "Creates a new space", new String[] { "create-space", "cs" }, Arrays.asList(
                new Argument("spaceURI"), new Argument("kernelURI", true)));
    }

    private SemanticSpaceOverlayKernel lookupKernel(String kernelURI) {
        SemanticSpaceOverlayKernel kernel = null;
        try {
            kernel = (SemanticSpaceOverlayKernel) PAActiveObject.lookupActive(
                    SemanticSpaceOverlayKernel.class.getCanonicalName(), kernelURI);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return kernel;
    }

    /**
     * {@inheritDoc}
     */
    public void execute(String... args) {
        URI spaceURI = RDF2GoBuilder.createURI(args[0]);

        if (args.length > 0 && args.length < 3) {
            try {
            	if (super.getManagementConnector().spaceExists(spaceURI)) {
                    super.error("Space '" + args[0] + "' already exists.");
                } else {
                    if (args.length == 2) {
                        try {
                        	super.getManagementConnector().createSpace(
                                    spaceURI, this.lookupKernel(args[1]));
                            super.info(
                                    "Space '" + 
                                    args[0] + 
                                    "' has been created with specified kernel.");
                        } catch (Exception e) {
                            super.error(e.getMessage());
                        }
                    } else if (args.length == 1) {
                        try {
                        	super.getManagementConnector().createSpace(spaceURI);
                        } catch (SemanticSpaceException e) {
                            super.error(e.getMessage());
                        }
                        super.info("Space '" + args[0] + "' has been created.");
                    }
                }
            } catch (SemanticSpaceException e) {
                super.error(e.getMessage());
            }
        } else {
            super.error("Operation must have one argument which is the spaceURI which identify the space to create.");
        }
    }
    
}
