package fr.inria.gforge.dspace.overlay.samples.commands.management;

import java.util.Arrays;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.samples.commands.Argument;
import fr.inria.gforge.dspace.overlay.samples.managers.Manager;
import fr.inria.gforge.dspace.overlay.util.RDF2GoBuilder;

/**
 * Creates a new CAN peer and forces it to join the network.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: NewPeerOnCANCommand.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class NewPeerOnCANCommand extends AbstractManagementCommand {

    /**
     * Constructor.
     * 
     * @param manager
     *            the manager that will execute this command.
     */
    public NewPeerOnCANCommand(Manager manager) {
        super(manager, "Creates a new CAN peer and forces it to join the network", new String[] {
                "join-on-can", "join-can" }, Arrays.asList(new Argument("spaceURI")));
    }

    /**
     * {@inheritDoc}
     */
    public void execute(String... args) {
        if (args.length == 1) {
            try {
                super.getManagementConnector().newPeerOnCAN(
                        RDF2GoBuilder.createURI(args[0]));
                super.info("New CAN peer has been added on space '" + args[0] + "'.");
            } catch (SemanticSpaceException e) {
                super.error(e.getMessage());
            }
        } else {
            super.error("Operation must have one argument which is the spaceURI to use.");
        }
    }
    
}
