package fr.inria.gforge.dspace.overlay.samples.commands.management;

import java.util.Arrays;

import org.objectweb.proactive.api.PAFuture;
import org.ontoware.rdf2go.model.node.URI;

import fr.inria.gforge.dspace.overlay.messages.query.chord.NbReferencesStoredQueryMessage;
import fr.inria.gforge.dspace.overlay.messages.response.chord.NbReferencesStoredResponseMessage;
import fr.inria.gforge.dspace.overlay.samples.commands.Argument;
import fr.inria.gforge.dspace.overlay.samples.commands.Command;
import fr.inria.gforge.dspace.overlay.samples.managers.Manager;
import fr.inria.gforge.dspace.overlay.samples.managers.SemanticSpaceManagementManager;
import fr.inria.gforge.dspace.overlay.util.RDF2GoBuilder;

/**
 * Returns the number of CAN peer references stored by a Chord peer.
 * 
 * @author Laurent Pellegrino
 *
 * @version $Id$
 */
public class NumberOfReferencesCommand extends Command {

    public NumberOfReferencesCommand(Manager manager) {
        super(manager, "Returns the number of CAN peer references stored by a Chord peer",
                new String[] { "nb-references", "nb-ref" }, Arrays.asList(new Argument("spaceURI")));
    }
    

    public void execute(String... args) {
    	if (args.length > 0 ) {
    		URI spaceURI = RDF2GoBuilder.createURI(args[0]);

            int nbReferences = ((NbReferencesStoredResponseMessage) PAFuture.getFutureValue(
                    ((SemanticSpaceManagementManager) super.getManager()).getTracker()
                        .getRandomPeer().send(
                            new NbReferencesStoredQueryMessage(spaceURI)))).getNbReferences();
            super.info("Chord peer knows '" + nbReferences + "' peers of type CAN.");
        } else {
        	super.error("You must specify the spaceURI to create.");
        }
    }

}
