package fr.inria.gforge.dspace.overlay.samples.commands.management;

import java.util.Arrays;

import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.samples.commands.Argument;
import fr.inria.gforge.dspace.overlay.samples.managers.Manager;
import fr.inria.gforge.dspace.overlay.util.RDF2GoBuilder;

/**
 * Forces a {@link Peer} (randomly chosen) from a specified space to leave.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: RemovePeerOnCANCommand.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class RemovePeerOnCANCommand extends AbstractManagementCommand {

    /**
     * Constructor.
     * 
     * @param manager
     *            the manager that will execute this command.
     */
    public RemovePeerOnCANCommand(Manager manager) {
        super(manager, "Forces a peer from a specified space to leave", new String[] {
                "leave-on-can", "leave-can" }, Arrays.asList(new Argument("spaceURI")));
    }

    /**
     * {@inheritDoc}
     */
    public void execute(String... args) {
        if (args.length == 1) {
            try {
                super.getManagementConnector().removePeerOnCAN(
                        RDF2GoBuilder.createURI(args[0]));
                super.info("A peer of type CAN has left from space '" + args[0] + "'.");
            } catch (SemanticSpaceException e) {
                super.error(e.getMessage());
            }
        } else {
            super.error("Operation must have one argument which is spaceURI.");
        }
    }

}
