package fr.inria.gforge.dspace.overlay.samples.commands.management.shortcuts;

import java.util.Arrays;

import org.ontoware.rdf2go.model.node.impl.URIImpl;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.connector.ManagementConnector;
import fr.inria.gforge.dspace.overlay.samples.commands.Argument;
import fr.inria.gforge.dspace.overlay.samples.commands.management.AbstractManagementCommand;
import fr.inria.gforge.dspace.overlay.samples.managers.Manager;
import fr.inria.gforge.dspace.overlay.samples.managers.SemanticSpaceManagementManager;

/**
 * Automates Semantic Spaces creation.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: AutoInitializationCommand.java 3622 2010-03-12 09:15:46Z
 *          laurentp $
 */
public class AutoInitializationCommand extends AbstractManagementCommand {

    private boolean alreadyCalled = false;

    /**
     * Constructor.
     * 
     * @param manager
     *            the manager that will execute this command.
     */
    public AutoInitializationCommand(Manager manager) {
        super(manager, "Creates the specified number of Chord nodes and spaces", new String[] {
                "auto-initialization", "auto-init", "ai" }, Arrays.asList(new Argument(
                "nbChordNodes"), new Argument("spaceURI1", true), new Argument("spaceURI2", true),
                new Argument("...", true)));
    }

    /**
     * {@inheritDoc}
     */
    public void execute(String... args) {
        if (args.length > 0) {
            if (((SemanticSpaceManagementManager) 
                    super.getManager()).getTracker().getNbOfPeersStored() > 0) {
                this.alreadyCalled = true;
            }

            if (!alreadyCalled) {
                this.alreadyCalled = true;

                try {
                    int nbChordNodesToCreate = Integer.parseInt(args[0]);
                    if (nbChordNodesToCreate > 0) {
                    	ManagementConnector connector = 
                    		((SemanticSpaceManagementManager) super.getManager()).getManagementConnector();
                    	
                        for (int i = 0; i < nbChordNodesToCreate; i++) {
                            connector.newPeerOnChord();
                        }

                        for (int i = 1; i < args.length; i++) {
                            try {
                                connector.createSpace(new URIImpl(args[i]));
                            } catch (SemanticSpaceException e) {
                                super.error(e.getMessage());
                            }
                        }

                        super.info("AutoInitialization is done.");
                    } else {
                        super.error("The number of Chord nodes to create must be positive.");
                    }
                } catch (NumberFormatException e) {
                    super.error("The number of chord nodes to create must be a number.");
                }
            } else {
                super.error("AutoInitialization already done.");
            }
        } else {
            super.error("Operation must have at least one argument which is the number of Chord nodes to create.");
        }
    }
    
}
