package fr.inria.gforge.dspace.overlay.samples.commands.operations;

import java.io.IOException;
import java.util.Arrays;

import org.ontoware.rdf2go.model.Statement;
import org.ontoware.rdf2go.model.node.impl.URIImpl;
import org.openrdf.rio.RDFHandlerException;
import org.openrdf.rio.RDFParseException;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.api.SemanticSpaceOperations;
import fr.inria.gforge.dspace.overlay.samples.commands.Argument;
import fr.inria.gforge.dspace.overlay.samples.commands.Command;
import fr.inria.gforge.dspace.overlay.samples.managers.Manager;
import fr.inria.gforge.dspace.overlay.samples.managers.SemanticSpaceOperationsManager;
import fr.inria.gforge.dspace.overlay.samples.parsers.NTriplesStringParser;

/**
 * Performs a addStatement operation from an active object which implements
 * {@link SemanticSpaceOperations}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: AddStatementCommand.java 5041 2010-08-19 13:20:59Z plaurent $
 */
public class AddStatementCommand extends Command {

    private static final NTriplesStringParser parser = new NTriplesStringParser();

    /**
     * Constructor.
     * 
     * @param manager
     *            the manager that will execute this command.
     */
    public AddStatementCommand(Manager manager) {
        super(manager, "Adds the specified triple on a space using the N-Triples syntax",
                new String[] { "add-statement", "add" }, Arrays.asList(new Argument("spaceURI"),
                        new Argument("triple")));
    }

    /**
     * {@inheritDoc}
     */
    public void execute(final String... args) {
        if (args.length >= super.getNbRequiredArguments()) {
            String statementUsingNTriplesSyntax = super.getArgumentsLine().substring(
                    args[0].length(), super.getArgumentsLine().length());

            Statement parsedStatement = null;
            try {
                parsedStatement = parser.parse(args[0], statementUsingNTriplesSyntax);
            } catch (RDFParseException e) {
                super.error("The triple to add doesn't use N-Triples syntax.");
            } catch (RDFHandlerException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }

            try {
                ((SemanticSpaceOperationsManager) super.getManager()).getConnector().addStatement(
                        new URIImpl(args[0]), parsedStatement);
                super.info("Triple has been written.");
            }  catch (SemanticSpaceException e) {
                super.error(e.getMessage());
            } catch (Exception e) {
                super.error(e.getMessage());
            }
        } else {
            super.error("Operation must have "
                            + super.getNbRequiredArguments()
                            + " arguments which are spaceURI and the triple to add using N-Triples syntax.");
        }
    }

}
