package fr.inria.gforge.dspace.overlay.samples.commands.operations;

import java.util.Set;

import org.ontoware.rdf2go.model.node.URI;

import fr.inria.gforge.dspace.overlay.api.SemanticSpaceOperations;
import fr.inria.gforge.dspace.overlay.samples.commands.Command;
import fr.inria.gforge.dspace.overlay.samples.managers.Manager;
import fr.inria.gforge.dspace.overlay.samples.managers.SemanticSpaceOperationsManager;

/**
 * Performs a list spaces operation command from an active object which
 * implements {@link SemanticSpaceOperations}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: ListJoinedSpacesCommand.java 3127 2010-01-06 15:59:42Z laurentp
 *          $
 */
public class ListSpacesCommand extends Command {

    /**
     * Constructor.
     * 
     * @param manager
     *            the manager that will execute this command.
     */
    public ListSpacesCommand(Manager manager) {
        super(manager, "List all available spaces", new String[] { "list-spaces", "ls" });
    }

    /**
     * {@inheritDoc}
     */
    public void execute(String... args) {
        Set<URI> spacesAvailable = ((SemanticSpaceOperationsManager) super.getManager())
                .getConnector().listSpaces();
        StringBuffer buf = new StringBuffer();

        if (spacesAvailable.size() == 0) {
            buf.append("No space available.");
        } else {
            buf.append("Available spaces are :\n");

            for (URI space : spacesAvailable) {
                buf.append("  - ");
                buf.append(space);
                buf.append("\n");
            }
        }
        super.info(buf.toString());
    }

}
