package fr.inria.gforge.dspace.overlay.samples.commands.operations;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Arrays;

import org.ontoware.rdf2go.model.node.impl.URIImpl;
import org.openrdf.rio.RDFHandlerException;
import org.openrdf.rio.RDFParseException;
import org.openrdf.rio.ntriples.NTriplesParser;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.connector.OperationsConnector;
import fr.inria.gforge.dspace.overlay.samples.commands.Argument;
import fr.inria.gforge.dspace.overlay.samples.commands.Command;
import fr.inria.gforge.dspace.overlay.samples.managers.Manager;
import fr.inria.gforge.dspace.overlay.samples.managers.SemanticSpaceOperationsManager;
import fr.inria.gforge.dspace.overlay.samples.parsers.sesame.RDF2GoHandler;

/**
 * Loads all N-Triples from the specified file and add them into the
 * SemanticSpaces infrastructure.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: LoadNTriplesFileCommand.java 4062 2010-04-16 09:11:34Z laurentp
 *          $
 */
public class LoadNTriplesFileCommand extends Command {

    private Long startLoad;

    private long endLoad;

    private BigInteger nbStatementsLoaded = BigInteger.ZERO;

    private NTriplesParser parser = new NTriplesParser();

    /**
     * Constructor.
     * 
     * @param manager
     *            the manager that will execute this command.
     */
    public LoadNTriplesFileCommand(Manager manager) {
        super(manager, "Loads triples from N-Triples file and add them into the infrastructure",
                new String[] { "load" }, Arrays.asList(new Argument("spaceURI"), new Argument(
                        "fileURI")));
    }

    /**
     * {@inheritDoc}
     */
    public void execute(final String... args) {
        final OperationsConnector connector = ((SemanticSpaceOperationsManager) super.getManager())
                .getConnector();

        if (args.length == 2) {
            this.parser.setRDFHandler(new RDF2GoHandler() {

                public void startRDF() throws RDFHandlerException {
                    LoadNTriplesFileCommand.this.nbStatementsLoaded = BigInteger.ZERO;
                    LoadNTriplesFileCommand.this.startLoad = System.currentTimeMillis();
                }

                public void handleNamespace(String prefix, String uri) throws RDFHandlerException {

                }

                public void handleComment(String comment) throws RDFHandlerException {

                }

                public void handleStatement(org.ontoware.rdf2go.model.Statement stmt)
                        throws RDFHandlerException {
                    try {
                        connector.addStatement(new URIImpl(args[0]), stmt);
                    } catch (SemanticSpaceException e) {
                        LoadNTriplesFileCommand.super.error(e.getMessage());
                    }

                    LoadNTriplesFileCommand.this.nbStatementsLoaded = LoadNTriplesFileCommand.this.nbStatementsLoaded
                            .add(BigInteger.ONE);
                    LoadNTriplesFileCommand.this.info("Triple '" + stmt + "' added");
                }

                public void endRDF() throws RDFHandlerException {
                    LoadNTriplesFileCommand.this.endLoad = System.currentTimeMillis();
                    LoadNTriplesFileCommand.this
                            .info(LoadNTriplesFileCommand.this.nbStatementsLoaded
                                    + " triple(s) loaded from '"
                                    + args[1]
                                    + "' in "
                                    + (int) ((LoadNTriplesFileCommand.this.endLoad - LoadNTriplesFileCommand.this.startLoad) / 1000)
                                    + " s");
                }
            });

            InputStream inputStream = null;

            if (args[1].startsWith("http:")) {
                try {
                    inputStream = new URL(args[1]).openStream();
                } catch (MalformedURLException e) {
                    LoadNTriplesFileCommand.super.error("URL '" + args[1] + "' is malformed");
                    return;
                } catch (IOException e) {
                    LoadNTriplesFileCommand.super.error("Cannot access to '" + args[1] + "'");
                    return;
                }
            } else {
                try {
                    inputStream = new FileInputStream(args[1]);
                } catch (FileNotFoundException e) {
                    LoadNTriplesFileCommand.super.error("File '" + args[1] + "' not found");
                    return;
                }
            }

            try {
                this.parser.parse(inputStream, args[1]);
            } catch (RDFParseException e) {
                super.error("Error while parsing '" + args[1] + "'");
            } catch (RDFHandlerException e) {
                super.error(e.getMessage());
            } catch (FileNotFoundException e) {
                super.error(e.getMessage());
            } catch (IOException e) {
                super.error(e.getMessage());
            }
        } else {
            super.error("Operation must have two arguments which are spaceURI and the fileURI.");
        }
    }
}
