package fr.inria.gforge.dspace.overlay.samples.commands.operations;

import java.io.IOException;
import java.util.Arrays;

import org.ontoware.rdf2go.model.Statement;
import org.ontoware.rdf2go.model.node.impl.URIImpl;
import org.openrdf.rio.RDFHandlerException;
import org.openrdf.rio.RDFParseException;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.api.SemanticSpaceOperations;
import fr.inria.gforge.dspace.overlay.samples.commands.Argument;
import fr.inria.gforge.dspace.overlay.samples.commands.Command;
import fr.inria.gforge.dspace.overlay.samples.managers.Manager;
import fr.inria.gforge.dspace.overlay.samples.managers.SemanticSpaceOperationsManager;
import fr.inria.gforge.dspace.overlay.samples.parsers.TriplePatternStringParser;

/**
 * Performs a removeStatements operation from an active object which implements
 * {@link SemanticSpaceOperations}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: RemoveStatementsCommand.java 3819 2010-04-02 14:59:26Z laurentp
 *          $
 */
public class RemoveStatementsCommand extends Command {

    private static final TriplePatternStringParser parser = new TriplePatternStringParser();

    /**
     * Constructor.
     * 
     * @param manager
     *            the manager that will execute this command.
     */
    public RemoveStatementsCommand(Manager manager) {
        super(manager, "Removes the statements which matche the specified Triple Pattern.",
                new String[] { "remove-statements" }, Arrays.asList(new Argument("spaceURI"),
                        new Argument("triplePattern")));
    }

    /**
     * {@inheritDoc}
     */
    public void execute(String... args) {
        if (args.length >= super.getNbRequiredArguments()) {

            String triplePattern = super.getArgumentsLine().substring(args[0].length(),
                    super.getArgumentsLine().length());

            Statement parsedStatement = null;
            try {
                parsedStatement = parser.parse(args[0], triplePattern);
            } catch (RDFParseException e) {
                super.error("The triple to add doesn't use N-Triples syntax.");
            } catch (RDFHandlerException e) {
                super.error(e.getMessage());
            } catch (IOException e) {
                super.error(e.getMessage());
            }

            try {
                ((SemanticSpaceOperationsManager) super.getManager()).getConnector()
                        .removeStatements(new URIImpl(args[0]), parsedStatement);
                super.info("Operation has been performed.");
            } catch (SemanticSpaceException e) {
                super.error(e.getMessage());
            } catch (Exception e) {
                super.error(e.getMessage());
            }
        } else {
            super.error("Operation must have " + super.getNbRequiredArguments()
                    + " arguments which are spaceURI and the triple to use for removing.");
        }
    }

}
