package fr.inria.gforge.dspace.overlay.samples.commands.operations;

import java.util.Arrays;

import org.ontoware.aifbcommons.collection.ClosableIterable;
import org.ontoware.rdf2go.model.QueryResultTable;
import org.ontoware.rdf2go.model.Statement;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import at.sti2.semanticspaces.api.exceptions.SemanticSpaceQueryException;
import fr.inria.gforge.dspace.overlay.api.SemanticSpaceOperations;
import fr.inria.gforge.dspace.overlay.connector.OperationsConnector;
import fr.inria.gforge.dspace.overlay.samples.commands.Argument;
import fr.inria.gforge.dspace.overlay.samples.commands.Command;
import fr.inria.gforge.dspace.overlay.samples.managers.Manager;
import fr.inria.gforge.dspace.overlay.samples.managers.SemanticSpaceOperationsManager;
import fr.inria.gforge.dspace.overlay.util.RDF2GoBuilder;
import fr.inria.gforge.dspace.overlay.util.SemanticHelper;

/**
 * Performs a query command from an active object which implements
 * {@link SemanticSpaceOperations}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: SPARQLQueryCommand.java 5041 2010-08-19 13:20:59Z plaurent $
 */
public class SPARQLQueryCommand extends Command {

    /**
     * Constructor.
     * 
     * @param manager
     *            the manager that will execute this command.
     */
    public SPARQLQueryCommand(Manager manager) {
        super(manager, "Query a specified space by using ASK, CONSTRUCT or SELECT Sparql queries",
                new String[] { "sparql-query", "query" }, Arrays.asList(new Argument("spaceURI"),
                        new Argument("query")));
    }

    /**
     * {@inheritDoc}
     */
    public void execute(String... args) {
        if (args.length > 2) {
            String[] arguments = super.getArgumentsLine().split("\\s+");
            String query = super.getArgumentsLine().substring(
                    super.getArgumentsLine().indexOf(arguments[1]));
            OperationsConnector connector = ((SemanticSpaceOperationsManager) super.getManager())
                    .getConnector();

            if (query.contains("ASK")) {
                try {
                    super.info("" + connector.sparqlAsk(RDF2GoBuilder.createURI(arguments[0]), query));
                } catch (SemanticSpaceQueryException e) {
                    super.error("Unknown query type, only ASK, CONSTRUCT AND SELECT Sparql queries are allowed.");
                } catch (SemanticSpaceException e) {
                    super.error(e.getMessage());
                }
            } else if (query.contains("CONSTRUCT")) {
                ClosableIterable<Statement> statements = null;
                try {
                    statements = connector.sparqlConstruct(RDF2GoBuilder.createURI(arguments[0]), query);

                    String statementsReadyToString = SemanticHelper.beautifyStatements(statements);

                    if (statementsReadyToString == null) {
                        super.info("No data retrieved.");
                    } else {
                        super.info("Statements retrieved :\n" + statementsReadyToString);
                    }
                } catch (SemanticSpaceQueryException e) {
                    super.error("Unknown query type, only ASK, CONSTRUCT AND SELECT Sparql queries are allowed.");
                } catch (SemanticSpaceException e) {
                    super.error(e.getMessage());
                }
            } else if (query.contains("SELECT")) {
                QueryResultTable res = null;
                try {
                    res = connector.sparqlSelect(RDF2GoBuilder.createURI(arguments[0]), query);
                    String queryResultTableToString = SemanticHelper.beautifyStatements(res);
                    if (queryResultTableToString == null) {
                        super.info("No data retrieved.");
                    } else {
                        super.info("Data retrieved :\n" + queryResultTableToString);
                    }
                } catch (SemanticSpaceQueryException e) {
                    super.error("Unknown query type : only ASK, CONSTRUCT AND SELECT Sparql queries are allowed.");
                } catch (SemanticSpaceException e) {
                    super.error(e.getMessage());
                }
            } else {
                super.error("Unknow query type : only ASK, CONSTRUCT AND SELECT Sparql queries are allowed.");
            }
        } else {
            super.error("Operation must have two arguments (have only " + args.length
                    + " arguments) which are spaceURI and the Sparql queries as String.");
        }
    }

}
