package fr.inria.gforge.dspace.overlay.samples.commands.operations.shortcuts;

import java.util.Arrays;

import org.ontoware.rdf2go.model.impl.StatementImpl;
import org.ontoware.rdf2go.model.node.impl.URIImpl;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.samples.commands.Argument;
import fr.inria.gforge.dspace.overlay.samples.commands.Command;
import fr.inria.gforge.dspace.overlay.samples.managers.Manager;
import fr.inria.gforge.dspace.overlay.samples.managers.SemanticSpaceOperationsManager;

/**
 * Add a fixed set of statement from a specified spaceURI.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: AddStatementsCommand.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class AddStatementsCommand extends Command {

    private String[][] statements = {
            { "http://SOA4ALL", "http://is", "http://LargeScaleIntegratingProject" },
            { "http://SOA4ALL", "http://isFoundedBy", "http://EuropeanSeventhFrameworkProgramme" },
            { "http://INRIA", "http://isMemberOf", "http://SOA4ALL" },
            { "http://INRIA", "http://is", "http://FrenchNationalResearchInstitution" } };

    /**
     * Constructor.
     * 
     * @param manager
     *            the manager that will execute this command.
     */
    public AddStatementsCommand(Manager manager) {
        super(manager, "Add a fixed set of statement from a specified spaceURI",
                new String[] { "add-statements" }, Arrays.asList(new Argument("spaceURI")));
    }

    /**
     * {@inheritDoc}
     */
    public void execute(String... args) {
        if (args.length == 1) {
            try {
                for (int i = 0; i < statements.length; i++) {
                    ((SemanticSpaceOperationsManager) super.getManager()).getConnector()
                            .addStatement(
                                    new URIImpl(args[0]),
                                    new StatementImpl(new URIImpl(args[0]), new URIImpl(
                                            this.statements[i][0]), new URIImpl(
                                            this.statements[i][1]), new URIImpl(
                                            this.statements[i][2])));

                }
                super.info(this.statements.length + " have been added on space '" + args[0] + "'.");
            } catch (SemanticSpaceException e) {
                super.error(e.getMessage());
            }
        } else {
            super.error("Operation must have one argument which is the spaceURI.");
        }
    }

}
