package fr.inria.gforge.dspace.overlay.samples.commands.operations.shortcuts;

import java.util.Arrays;

import org.ontoware.aifbcommons.collection.ClosableIterable;
import org.ontoware.rdf2go.model.Statement;
import org.ontoware.rdf2go.model.node.impl.URIImpl;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.samples.commands.Argument;
import fr.inria.gforge.dspace.overlay.samples.commands.Command;
import fr.inria.gforge.dspace.overlay.samples.managers.Manager;
import fr.inria.gforge.dspace.overlay.samples.managers.SemanticSpaceOperationsManager;
import fr.inria.gforge.dspace.overlay.util.SemanticHelper;

/**
 * Retrieves all statements from a specified spaceURI.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: QueryAllCommand.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class QueryAllCommand extends Command {

    /**
     * Constructor.
     * 
     * @param manager
     *            the manager that will execute this command.
     */
    public QueryAllCommand(Manager manager) {
        super(manager, "Retrieves all data from a specified spaceURI", new String[] { "query-all",
                "query*" }, Arrays.asList(new Argument("spaceURI")));
    }

    /**
     * {@inheritDoc}
     */
    public void execute(String... args) {
        if (args.length == 1) {
            String query = "CONSTRUCT { ?s ?p ?o } WHERE { GRAPH<" + args[0] + "> { ?s ?p ?o } . }";

            ClosableIterable<Statement> statements = null;
            try {
                statements = ((SemanticSpaceOperationsManager) super.getManager()).getConnector()
                        .sparqlConstruct(new URIImpl(args[0]), query);

                String statementsToString = SemanticHelper.beautifyStatements(statements);

                if (statementsToString == null) {
                    super.info("No data retrieved.");
                } else {
                    super.info("Statements retrieved :\n" + statementsToString);
                }
            } catch (SemanticSpaceException e) {
                super.error(e.getMessage());
            }
        } else {
            super.error("Operation must have one argument which is the spaceURI to use for query.");
        }
    }

}
