package fr.inria.gforge.dspace.overlay.samples.launchers;

import java.io.IOException;
import java.io.Serializable;

import org.objectweb.proactive.ActiveObjectCreationException;
import org.objectweb.proactive.api.PAActiveObject;

import fr.inria.gforge.dspace.overlay.samples.managers.Manager;
import fr.inria.gforge.dspace.overlay.samples.managers.SemanticSpaceManagementManager;
import fr.inria.gforge.dspace.overlay.samples.managers.SemanticSpaceOperationsManager;
import fr.inria.gforge.dspace.overlay.tracker.SemanticChordTracker;

/**
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: SemanticSpaceAccessLauncher.java 3127 2010-01-06 15:59:42Z
 *          laurentp $
 */
public class ConnectorLauncher implements Serializable {

    private static final long serialVersionUID = 1L;

    public enum Type {
        OPERATIONS, MANAGEMENT
    };

    private String bindToURI;

    private Type type = Type.OPERATIONS;

    public ConnectorLauncher(String bindToURI) {
        this.bindToURI = bindToURI;
    }

    public void init() {
        Manager manager;
        SemanticChordTracker tracker = (SemanticChordTracker) this.lookup(
                                        SemanticChordTracker.class, this.bindToURI);
        if (this.type == Type.MANAGEMENT) {
            manager = new SemanticSpaceManagementManager(tracker);
        } else {
            manager = new SemanticSpaceOperationsManager(tracker);
        }
        manager.startExecution();
    }

    private Object lookup(Class<?> clazz, String bindingURI) {
        try {
            return PAActiveObject.lookupActive(clazz, bindingURI);
        } catch (ActiveObjectCreationException e) {
            e.printStackTrace();
            System.exit(1);
        } catch (IOException e) {
            e.printStackTrace();
            System.exit(1);
        }
        return null;
    }

    public static void main(String[] args) {
        if (args.length != 2) {
            ConnectorLauncher.printManual();
        }

        ConnectorLauncher connectorLauncher = new ConnectorLauncher(args[1]);

        if (args[0].equals("management")) {
            connectorLauncher.setType(Type.MANAGEMENT);
        } else if (args[0].equals("operations")) {
            connectorLauncher.setType(Type.OPERATIONS);
        } else {
            ConnectorLauncher.printManual();
        }
        connectorLauncher.init();
    }

    public static void printManual() {
        System.out
                .println("Usage:\n\tjava "
                        + ConnectorLauncher.class.getCanonicalName()
                        + " TYPE URI\n\nTYPE must be one of 'operations' or 'management'.\nURI must be a tracker URI.\n");
        System.exit(1);
    }

    public void setType(Type type) {
        this.type = type;
    }

}
