package fr.inria.gforge.dspace.overlay.samples.launchers;

import java.io.Serializable;
import java.util.UUID;

import org.objectweb.proactive.ActiveObjectCreationException;
import org.objectweb.proactive.api.PAActiveObject;
import org.objectweb.proactive.core.ProActiveException;
import org.objectweb.proactive.core.node.NodeException;
import org.objectweb.proactive.extensions.structuredp2p.tracker.Tracker;

import fr.inria.gforge.dspace.overlay.api.SemanticFactory;
import fr.inria.gforge.dspace.overlay.kernel.SemanticSpaceOverlayKernel;

/**
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: SemanticSpaceOverlayKernelLauncher.java 3141 2010-01-12
 *          11:36:06Z laurentp $
 */
public class KernelLauncher implements Serializable {

    private static final long serialVersionUID = 1L;

    private String trackerURI;

    private String uniqueID = UUID.randomUUID().toString();

    private Tracker remoteTracker;

    private SemanticSpaceOverlayKernel semanticSpaceOverlayKernel;

    public KernelLauncher(String trackerURI) {
        this.trackerURI = trackerURI;
    }

    public KernelLauncher(String trackerURI, String uniqueID) {
        this.trackerURI = trackerURI;
        this.uniqueID = uniqueID;
    }

    private void lookupTracker() {
        try {
            this.remoteTracker = (Tracker) PAActiveObject.lookupActive(Tracker.class
                    .getCanonicalName(), this.trackerURI);
            System.out.println("Tracker found from " + this.trackerURI);
        } catch (Exception e) {
            System.out.println("Error while looking up tracker");
            e.printStackTrace();
            System.exit(1);
        }
    }

    private void createAndRegisterOverlayKernel() {
        try {
            this.semanticSpaceOverlayKernel = SemanticFactory.newActiveSemanticSpaceOverlayKernel(
                    new Tracker[] { this.remoteTracker }, false);
        } catch (ActiveObjectCreationException e1) {
            e1.printStackTrace();
        } catch (NodeException e1) {
            e1.printStackTrace();
        }

        String bindName = null;
        try {
            bindName = PAActiveObject.registerByName(
            				this.semanticSpaceOverlayKernel, this.uniqueID + "/kernel");
        } catch (ProActiveException e) {
            System.err.println("Error while binding SemanticSpaceOverlayKernel");
            e.printStackTrace();
            System.exit(1);
        }

        System.out.println("SemanticSpaceOverlayKernel launched at:");
        System.out.println(bindName);
    }

    private void init() {
        this.lookupTracker();
        this.createAndRegisterOverlayKernel();
    }

    public static void main(String[] args) {
        KernelLauncher kernelLauncher = null;

        switch (args.length) {
            case 1:
                kernelLauncher = new KernelLauncher(args[0]);
                break;
            case 2:
                kernelLauncher = new KernelLauncher(args[0], args[1]);
                break;
            default:
                System.err.println("Usage : " + KernelLauncher.class.getCanonicalName()
                        + " trackerURI [uniqueID]");
                System.exit(1);
        }

        kernelLauncher.init();
    }

}
