package fr.inria.gforge.dspace.overlay.samples.launchers;

import java.io.File;

import org.objectweb.proactive.ActiveObjectCreationException;
import org.objectweb.proactive.api.PAActiveObject;
import org.objectweb.proactive.core.node.NodeException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import fr.inria.gforge.dspace.overlay.deployment.NodeProvider;
import fr.inria.gforge.dspace.overlay.deployment.NodeProviderKey;

/**
 * Provides a deployment by using GCMA and GCMD descriptors.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: NodeProviderLauncher.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class NodeProviderLauncher {

    private final static Logger logger = LoggerFactory.getLogger(NodeProviderLauncher.class);

    private File pathToGCMADescriptor;

    private String nodeProviderBindingName;

    private NodeProvider nodeProvider;

    private boolean isAlreadyDone = false;

    public NodeProviderLauncher(File pathToGCMADescriptor) {
        this.pathToGCMADescriptor = pathToGCMADescriptor;
    }

    private String startNodeProvider() {
        this.nodeProvider = new NodeProvider(this.pathToGCMADescriptor);

        try {
            this.nodeProvider = PAActiveObject.turnActive(nodeProvider, nodeProvider.getNextNode(NodeProviderKey.NODE_PROVIDER));
        } catch (ActiveObjectCreationException e) {
            e.printStackTrace();
        } catch (NodeException e) {
            e.printStackTrace();
        }
        this.nodeProviderBindingName = this.nodeProvider.register();
        
        if (logger.isInfoEnabled()) {
            logger.info("NodeProvider is intialized and bind to {}",
                    this.nodeProviderBindingName);
        }
        
        return this.nodeProviderBindingName;
    }

    public void startDeployment() {
        if (!this.isAlreadyDone) {
            this.startNodeProvider();

            System.out.println("Node provider launched at :");
            System.out.println(this.nodeProviderBindingName);

            this.isAlreadyDone = true;
        }
    }

    public String getNodeProviderBindingName() {
        return this.nodeProviderBindingName;
    }
    
    public static void main(String[] args) {
        if (args.length != 1) {
            System.err.println("Usage : java " + 
                    NodeProviderLauncher.class.getCanonicalName()
                    + " pathToGCMADescriptor");
        }

        NodeProviderLauncher semanticSpacesDeployment = 
            new NodeProviderLauncher(new File(args[0]));
        semanticSpacesDeployment.startDeployment();
    }

}
