package fr.inria.gforge.dspace.overlay.samples.launchers;

import java.io.File;
import java.net.UnknownHostException;

import org.objectweb.proactive.ActiveObjectCreationException;
import org.objectweb.proactive.core.node.NodeException;
import org.objectweb.proactive.extensions.structuredp2p.tracker.Tracker;

import fr.inria.gforge.dspace.overlay.api.SemanticFactory;
import fr.inria.gforge.dspace.overlay.tracker.SemanticChordTracker;
import fr.inria.gforge.dspace.overlay.util.DSpaceProperties;

/**
 * Creates a new active {@link SemanticChordTracker} and binds it to the RMI
 * registry with the specified network and tracker name.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: TrackerLauncher.java 5041 2010-08-19 13:20:59Z plaurent $
 */
public class TrackerLauncher {

    public static void main(String[] args) throws NodeException, UnknownHostException {
        if (args.length < 1 || args.length > 2) {
            System.err.println("Usage : " + TrackerLauncher.class.getCanonicalName()
                    + " networkName [trackerName]");
            System.exit(1);
        } else {
            SemanticChordTracker tracker = null;
            String trackerName = null;
            if (args.length == 2) {
                trackerName = args[1];
            }

            try {
                tracker = SemanticFactory.newActiveSemanticChordTracker(args[0], trackerName);
            } catch (ActiveObjectCreationException e) {
                e.printStackTrace();
            } catch (NodeException e) {
                e.printStackTrace();
            }

            String bindingName = Tracker.registerAndStoreBindingName(
            								tracker, new File(
            											DSpaceProperties.getDefaultPathForConfigurationFiles(), 
            											"trackers.properties").getAbsolutePath());

            StringBuffer buf = new StringBuffer();
            buf.append("Tracker launched at:\n");
            buf.append(bindingName);
            System.out.println(buf.toString());
        }
    }

}
