package fr.inria.gforge.dspace.overlay.samples.managers;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.LinkedList;
import java.util.List;

import jline.ArgumentCompletor;
import jline.Completor;
import jline.ConsoleReader;
import jline.SimpleCompletor;
import fr.inria.gforge.dspace.overlay.samples.commands.Argument;
import fr.inria.gforge.dspace.overlay.samples.commands.Command;

/**
 * @author Laurent Pellegrino
 * 
 * @version $Id: InteractiveManager.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public abstract class InteractiveManager extends Manager {

    private String actionsMenu;

    public InteractiveManager() {
        super();
    }

    private void createActionsMenu() {
        StringBuffer buf = new StringBuffer();
        buf.append("Select an action to perform from ");
        try {
            buf.append(InetAddress.getLocalHost().getHostName());
        } catch (UnknownHostException e) {
            buf.append("unKnown");
        }
        buf.append(":\n");

        for (Command cmd : super.getCommands()) {
            buf.append("   (*) ");
            buf.append(cmd.getCommandShortcuts().get(0));

            List<Argument> commandArguments = cmd.getCommandArguments();
            if (commandArguments.size() > 0) {
                buf.append(" ");
            }

            for (int i = 0; i < commandArguments.size(); i++) {
                if (commandArguments.get(i).isOptional()) {
                    buf.append("[");
                } else {
                    buf.append("<");
                }
                buf.append(commandArguments.get(i));
                if (commandArguments.get(i).isOptional()) {
                    buf.append("]");
                } else {
                    buf.append(">");
                }
                if (i != commandArguments.size() - 1) {
                    buf.append(" ");
                }
            }

            buf.append(" : ");
            buf.append(cmd.getDescription());
            buf.append("\n");
        }
        this.actionsMenu = buf.toString();
    }

    public void printActionsMenu() {
        System.out.println(this.actionsMenu);
    }

    public void run() {
        String inputLine;
        ConsoleReader reader = null;
        try {
            reader = new ConsoleReader();
        } catch (IOException e) {
            e.printStackTrace();
        }

        List<Completor> completors = new LinkedList<Completor>();
        String[] commands = new String[super.getCommands().size()];

        int index = 0;
        for (Command action : super.getCommands()) {
            commands[index] = action.getCommandShortcuts().get(0);
            index++;
        }

        completors.add(new SimpleCompletor(commands));
        reader.addCompletor(new ArgumentCompletor(completors));
        this.createActionsMenu();

        try {
            reader.clearScreen();
            this.printActionsMenu();

            while ((inputLine = reader.readLine("> ")) != null) {
                String[] splittedInputLine = inputLine.split("\\s+");
                String command = splittedInputLine[0];

                String[] args = new String[splittedInputLine.length - 1];
                for (int i = 1; i < splittedInputLine.length; i++) {
                    args[i - 1] = splittedInputLine[i];
                }

                for (Command cmd : super.getCommands()) {
                    if (cmd.getCommandShortcuts().contains(command)) {
                        if (args.length > 0) {
                            cmd.setArgumentsLine(inputLine.substring(inputLine.indexOf(args[0])));
                        }
                        cmd.execute(args);
                        break;
                    }
                }
                reader.printNewline();
                this.printActionsMenu();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
