package fr.inria.gforge.dspace.overlay.samples.managers;

import java.util.ArrayList;
import java.util.List;

import fr.inria.gforge.dspace.overlay.samples.commands.Command;

/**
 * Manager is an abstract class that must be extends by all managers. A manager
 * is used in order to launch some various kind of tests on a structured
 * peer-to-peer network.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: Manager.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public abstract class Manager extends Thread {

    private List<Command> commands = new ArrayList<Command>();

    private boolean isRunning = false;

    public Manager() {

    }

    public void addCommand(Command cmd) {
        this.commands.add(cmd);
    }

    public List<Command> getCommands() {
        return this.commands;
    }
    
    public boolean isRunning() {
        return this.isRunning;
    }

    public void removeAction(Command action) {
        this.commands.remove(action.getName());
    }

    public abstract void run();

    public void startExecution() {
        this.isRunning = true;
        super.start();
    }

    public void stopExecution() {
        this.isRunning = false;
    }
    
}
