package fr.inria.gforge.dspace.overlay.samples.managers;

import fr.inria.gforge.dspace.overlay.connector.ManagementConnector;
import fr.inria.gforge.dspace.overlay.samples.commands.management.CreateSpaceCommand;
import fr.inria.gforge.dspace.overlay.samples.commands.management.NewPeerOnCANCommand;
import fr.inria.gforge.dspace.overlay.samples.commands.management.NewPeerOnChordCommand;
import fr.inria.gforge.dspace.overlay.samples.commands.management.NumberOfReferencesCommand;
import fr.inria.gforge.dspace.overlay.samples.commands.management.RemovePeerOnCANCommand;
import fr.inria.gforge.dspace.overlay.samples.commands.management.RemovePeerOnChordCommand;
import fr.inria.gforge.dspace.overlay.samples.commands.management.shortcuts.AutoInitializationCommand;
import fr.inria.gforge.dspace.overlay.samples.commands.operations.QuitCommand;
import fr.inria.gforge.dspace.overlay.tracker.SemanticChordTracker;

/**
 * Manager used by administrators to manage Semantic Spaces.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: SemanticSpaceManagementManager.java 3127 2010-01-06 15:59:42Z
 *          laurentp $
 */
public class SemanticSpaceManagementManager extends InteractiveManager {

    private SemanticChordTracker tracker;

    private ManagementConnector managementConnector;
    
    public SemanticSpaceManagementManager(SemanticChordTracker remoteTracker) {
        super();
        this.tracker = remoteTracker;
        this.managementConnector = new ManagementConnector(this.tracker);
        
        super.addCommand(new AutoInitializationCommand(this));
        super.addCommand(new CreateSpaceCommand(this));
        super.addCommand(new NewPeerOnChordCommand(this));
        super.addCommand(new NewPeerOnCANCommand(this));
        super.addCommand(new RemovePeerOnChordCommand(this));
        super.addCommand(new RemovePeerOnCANCommand(this));
        super.addCommand(new NumberOfReferencesCommand(this));

        super.addCommand(new QuitCommand(this) {
            public void execute(String... args) {
                System.exit(0);
            }
        });
    }

    public SemanticChordTracker getTracker() {
        return this.tracker;
    }
    
    public ManagementConnector getManagementConnector() {
        return this.managementConnector;
    }

}
