package fr.inria.gforge.dspace.overlay.samples.managers;

import fr.inria.gforge.dspace.overlay.connector.OperationsConnector;
import fr.inria.gforge.dspace.overlay.samples.commands.operations.AddStatementCommand;
import fr.inria.gforge.dspace.overlay.samples.commands.operations.ListSpacesCommand;
import fr.inria.gforge.dspace.overlay.samples.commands.operations.LoadNTriplesFileCommand;
import fr.inria.gforge.dspace.overlay.samples.commands.operations.QuitCommand;
import fr.inria.gforge.dspace.overlay.samples.commands.operations.RemoveStatementCommand;
import fr.inria.gforge.dspace.overlay.samples.commands.operations.RemoveStatementsCommand;
import fr.inria.gforge.dspace.overlay.samples.commands.operations.SPARQLQueryCommand;
import fr.inria.gforge.dspace.overlay.samples.commands.operations.shortcuts.AddStatementsCommand;
import fr.inria.gforge.dspace.overlay.samples.commands.operations.shortcuts.QueryAllCommand;
import fr.inria.gforge.dspace.overlay.tracker.SemanticChordTracker;

/**
 * Manager used by all users in order to perform operations on Semantic Spaces.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: SemanticSpaceOperationsManager.java 3127 2010-01-06 15:59:42Z
 *          laurentp $
 */
public class SemanticSpaceOperationsManager extends InteractiveManager {

    private OperationsConnector connector;

    public SemanticSpaceOperationsManager() {
        super();

        // Tests operations
        super.addCommand(new AddStatementsCommand(this));
        super.addCommand(new QueryAllCommand(this));
        super.addCommand(new LoadNTriplesFileCommand(this));

        // Basic operations
        super.addCommand(new AddStatementCommand(this));
        super.addCommand(new RemoveStatementCommand(this));
        super.addCommand(new RemoveStatementsCommand(this));

        super.addCommand(new ListSpacesCommand(this));
        super.addCommand(new SPARQLQueryCommand(this));
        super.addCommand(new QuitCommand(this));
    }

    public SemanticSpaceOperationsManager(SemanticChordTracker chordTracker) {
        this();
        this.connector = new OperationsConnector(chordTracker);
    }

    public OperationsConnector getConnector() {
        return this.connector;
    }

}
