package fr.inria.gforge.dspace.overlay.samples.parsers;

import java.io.IOException;
import java.io.StringReader;

import org.ontoware.rdf2go.model.Statement;
import org.ontoware.rdf2go.model.impl.StatementImpl;
import org.openrdf.rio.RDFHandlerException;
import org.openrdf.rio.RDFParseException;
import org.openrdf.rio.helpers.RDFParserBase;

import fr.inria.gforge.dspace.overlay.samples.parsers.sesame.RDF2GoHandler;

/**
 * Generic parser in order to parse RDF string.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: RDFStringParser.java 5038 2010-08-19 09:09:50Z plaurent $
 * 
 * @param <T>
 *            Parser to use in order to parse.
 */
public abstract class RDFStringParser<T extends RDFParserBase> {

    private T parser;

    private Statement parsedStatement;

    public RDFStringParser(T parser) {
        this.parser = parser;
        this.parser.setRDFHandler(new RDF2GoHandler() {

            public void startRDF() throws RDFHandlerException {
            }

            public void handleNamespace(String prefix, String uri) throws RDFHandlerException {
            }

            public void handleComment(String comment) throws RDFHandlerException {
            }

            public void handleStatement(Statement stmt) throws RDFHandlerException {
                parsedStatement = new StatementImpl(stmt.getContext(), stmt.getSubject(), stmt
                        .getPredicate(), stmt.getObject());
            }

            public void endRDF() throws RDFHandlerException {
            }
        });
    }

    public Statement parse(String context, String statement) throws RDFParseException,
            RDFHandlerException, IOException {
        this.parser.parse(new StringReader(statement), context);
        return this.parsedStatement;
    }

    public Statement getParsedStatement() {
        return this.parsedStatement;
    }

    public T getParser() {
        return this.parser;
    }

}
