package fr.inria.gforge.dspace.overlay.samples.parsers.sesame;

import org.ontoware.rdf2go.model.Statement;
import org.ontoware.rdf2go.model.impl.StatementImpl;
import org.ontoware.rdf2go.model.node.Resource;
import org.ontoware.rdf2go.model.node.impl.URIImpl;
import org.openrdf.rdf2go.ConversionUtil;
import org.openrdf.rio.RDFHandler;
import org.openrdf.rio.RDFHandlerException;

/**
 * An interface defining methods related to RDF2go. It is used to transform
 * Sesame types from {@link RDFHandler} to RDF2Go.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: RDF2GoHandler.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public abstract class RDF2GoHandler implements RDFHandler {

    /**
     * Signals the start of the RDF data. This method is called before any data
     * is reported.
     * 
     * @throws RDFHandlerException
     *             If the RDF handler has encountered an unrecoverable error.
     */
    public abstract void startRDF() throws RDFHandlerException;

    /**
     * Signals the end of the RDF data. This method is called when all data has
     * been reported.
     * 
     * @throws RDFHandlerException
     *             If the RDF handler has encountered an unrecoverable error.
     */
    public abstract void endRDF() throws RDFHandlerException;

    /**
     * Handles a namespace declaration/definition. A namespace declaration
     * associates a (short) prefix string with the namespace's URI. The prefix
     * for default namespaces, which do not have an associated prefix, are
     * represented as empty strings.
     * 
     * @param prefix
     *            The prefix for the namespace, or an empty string in case of a
     *            default namespace.
     * @param uri
     *            The URI that the prefix maps to.
     * @throws RDFHandlerException
     *             If the RDF handler has encountered an unrecoverable error.
     */
    public abstract void handleNamespace(String prefix, String uri) throws RDFHandlerException;

    /**
     * Handles a statement.
     * 
     * @param stmt
     *            The statement.
     * @throws RDFHandlerException
     *             If the RDF handler has encountered an unrecoverable error.
     */
    public void handleStatement(org.openrdf.model.Statement stmt) throws RDFHandlerException {
        this.handleStatement(new StatementImpl(stmt.getContext() == null ? null : new URIImpl(stmt
                .getContext().toString()), (Resource) ConversionUtil.toRdf2go(stmt.getSubject()),
                ConversionUtil.toRdf2go(stmt.getPredicate()), ConversionUtil.toRdf2go(stmt
                        .getObject())));
    }

    public abstract void handleStatement(Statement stmt) throws RDFHandlerException;

    /**
     * Handles a comment.
     * 
     * @param comment
     *            The comment.
     * @throws RDFHandlerException
     *             If the RDF handler has encountered an unrecoverable error.
     */
    public abstract void handleComment(String comment) throws RDFHandlerException;

}
