package fr.inria.gforge.dspace.rdf2go;

import java.io.IOException;
import java.util.Properties;

import org.objectweb.proactive.ActiveObjectCreationException;
import org.objectweb.proactive.api.PAActiveObject;
import org.ontoware.rdf2go.exception.ModelRuntimeException;
import org.ontoware.rdf2go.impl.AbstractModelFactory;
import org.ontoware.rdf2go.model.Model;
import org.ontoware.rdf2go.model.ModelSet;
import org.ontoware.rdf2go.model.node.URI;

import at.sti2.semanticspaces.api.exceptions.OperationNotSupportedException;
import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.tracker.SemanticChordTracker;
import fr.inria.gforge.dspace.overlay.util.RDF2GoBuilder;

/**
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: OverlayModelFactory.java 5038 2010-08-19 09:09:50Z plaurent $
 */
public class OverlayModelFactory extends AbstractModelFactory {

    /**
     * Creates a new {@link Model} by using the specified
     * <code>properties</code>. Two properties are compulsory in order to create
     * a new model. The first one is <code>context.uri</code> which specify the
     * {@link URI} associated to the model (i.e. the space on which the
     * operations are performed). The second parameter is
     * <code>tracker.uri</code> which indicates the tracker to use in order to
     * perform the operations on the model.
     * 
     * @param properties
     *            the properties to use in order to create the new model.
     * 
     * @return a new {@link Model} by using the specified
     *         <code>properties</code>.
     */
    public Model createModel(Properties properties)
	    throws ModelRuntimeException {
	String contextURI = properties.getProperty("context.uri", null);
	String trackerURI = properties.getProperty("tracker.uri", null);
	
	if (contextURI == null || trackerURI == null) {
	    throw new ModelRuntimeException("'context.uri' and 'tracker.uri' properties must be defined!");
	}
	
	try {
	    return new OverlayModel(
	    	RDF2GoBuilder.createURI(contextURI), 
	    	PAActiveObject.lookupActive(SemanticChordTracker.class, trackerURI));
	} catch (ActiveObjectCreationException e) {
	    e.printStackTrace();
	} catch (SemanticSpaceException e) {
	    e.printStackTrace();
	} catch (IOException e) {
	    e.printStackTrace();
	}
	return null;
    }

    /**
     * Creates a new {@link Model} with the specified <code>contextURI</code>
     * (i.e. the spaceURI associated to the model created). By default a new
     * tracker is initialized so this method will create a new Model on a new
     * network. By default the network contains no peer.
     * 
     * @param contextURI
     *            the contextURI to use for the model creation.
     * 
     * @return a new {@link Model}.
     */
    public Model createModel(URI contextURI) throws ModelRuntimeException {
	SemanticChordTracker tracker = new SemanticChordTracker();
	tracker.register();
	
	try {
	    return new OverlayModel(contextURI, tracker);
	} catch (SemanticSpaceException e) {
	    e.printStackTrace();
	}
	return null;
    }

    public ModelSet createModelSet(Properties properties)
	    throws ModelRuntimeException {
	throw new OperationNotSupportedException();
    }

}