package fr.inria.gforge.dspace.rdf2go;

import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.ontoware.rdf2go.exception.ModelRuntimeException;
import org.ontoware.rdf2go.model.QueryResultTable;
import org.ontoware.rdf2go.model.node.URI;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import fr.inria.gforge.dspace.overlay.connector.ManagementConnector;
import fr.inria.gforge.dspace.overlay.initializers.GlobalNetworkInitializer;
import fr.inria.gforge.dspace.overlay.util.RDF2GoBuilder;

/**
 * Provides simple tests for {@link OverlayModel}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: TestOverlayModel.java 5043 2010-08-19 14:24:30Z plaurent $
 */
public class TestOverlayModel {

	private static GlobalNetworkInitializer initializer = new GlobalNetworkInitializer();

	private static URI defaultSpace = RDF2GoBuilder
			.createURI("http://www.inria.fr");

	private static OverlayModel model;

	@BeforeClass
	public static void setUp() {
		initializer.setUpNetworkOnLocalMachine();

		ManagementConnector connector = initializer.getManagementConnector();
		connector.setAutoRemoveDatastoreAtShutdown(true);
		connector.newPeersOnChordInParallel(5);

		try {
			// creates a space with one peer of type CAN by default
			connector.createSpace(defaultSpace);
			connector.newPeersOnCANInParallel(defaultSpace, 5);
		} catch (SemanticSpaceException e) {
			e.printStackTrace();
		}

		try {
			model = new OverlayModel(defaultSpace,
					initializer.getRandomTracker());
		} catch (SemanticSpaceException e) {
			e.printStackTrace();
		}
	}

	@Test
	public void test() {
		Assert.assertFalse(model.sparqlAsk("ASK { ?s ?p ?o }"));

		model.addStatement(defaultSpace,
				RDF2GoBuilder.createURI("http://www.inria.fr/onto#memberOf"),
				RDF2GoBuilder.createURI("http://www.soa4all.eu"));

		Assert.assertTrue(model.sparqlAsk("ASK { ?s ?p ?o }"));

		QueryResultTable selectResult = model
				.sparqlSelect("SELECT ?s WHERE { ?s ?p ?o }");
		Assert.assertEquals(defaultSpace, selectResult.iterator().next()
				.getValue("s"));

		// TODO support DESCRIBE query: currently not supported due to reasoner
		// rewriting (see decomposition part)
		// ClosableIterable<Statement> describeResult =
		// model.sparqlDescribe("DESCRIBE ?s WHERE { ?s ?p ?o }");

		// Assert.assertTrue(describeResult.iterator().hasNext());
	}

	@Test
	public void testQueryWithIncorrectSyntax() {
		try {
			model.sparqlAsk("ASKK { ?s ?p ?o }");
		} catch (ModelRuntimeException e) {
			// we encounter an exception because query syntax is incorrect
			e.printStackTrace();
		}

		// however, the infrastructure isn't broken and we can
		// perform other operations
		Assert.assertTrue(model.sparqlAsk("ASK { ?s ?p ?o }"));
	}

	@AfterClass
	public static void tearDown() {
		initializer.tearDownNetwork();
	}

}
