package org.objectweb.proactive.extensions.structuredp2p.api;

import org.objectweb.proactive.ActiveObjectCreationException;
import org.objectweb.proactive.api.PAActiveObject;
import org.objectweb.proactive.core.node.Node;
import org.objectweb.proactive.core.node.NodeException;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.requests.StructuredMetaObjectFactory;

/**
 * Provides some static methods in order to create easily
 * a new active peer.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: PeerFactory.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class PeerFactory {

    /**
     * Creates a new Peer ActiveObject by using the specified overlay
     * abstraction.
     * 
     * @param overlay
     *            the overlay to set.
     * @return the new Peer object created.
     * 
     * @throws ActiveObjectCreationException
     *             if a problem occurs during the creation of the active object.
     * @throws NodeException
     *             if a problem occurs during the deployment.
     */
    public static Peer newActivePeer(StructuredOverlay overlay)
            throws ActiveObjectCreationException, NodeException {
        return PeerFactory.newActivePeer(overlay, null);
    }

    /**
     * Creates a new Peer ActiveObject by using the specified overlay
     * abstraction and the given node which indicates where to deploy the
     * object.
     * 
     * @param overlay
     *            the overlay to set.
     * @param node
     *            the node used by the peer.
     * @return the new Peer object created.
     * 
     * @throws ActiveObjectCreationException
     *             if a problem occurs during the creation of the active object.
     * @throws NodeException
     *             if a problem occurs during the deployment.
     */
    public static Peer newActivePeer(StructuredOverlay overlay, Node node)
            throws ActiveObjectCreationException, NodeException {
        Peer newPeer = new Peer(overlay);

        return (Peer) PAActiveObject.turnActive(newPeer, node, null,
                new StructuredMetaObjectFactory());
    }

}
