package org.objectweb.proactive.extensions.structuredp2p.messages;

import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.router.Router;

/**
 * Interface implemented by all messages in order to be routable.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: Routable.java 5022 2010-08-14 16:14:45Z plaurent $
 * 
 * @param <K>
 *            the type of the key used to check if the constraints are validated
 *            (i.e. to make decision to route).
 */
public interface Routable<K> {

    /**
     * Returns the {@link Router} to use in order to route the message.
     * 
     * @return the {@link Router} to use in order to route the message.
     */
    public abstract Router<? extends SynchronousMessage<K>, K> getRouter();

    /**
     * Route the {@link SynchronousMessage} to the correct {@link Peer}. If the
     * current peer contains the key to reach, the query is handled and a
     * response is routed to the sender.
     * 
     * @param overlay
     *            the overlay used in order to route the request.
     */
    public abstract void route(StructuredOverlay overlay);

}
