package org.objectweb.proactive.extensions.structuredp2p.messages;

import java.io.Serializable;


/**
 * {@link SynchronousMessageEntry} contains informations about responses
 * received while dispatching a query.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: SynchronousMessageEntry.java 3127 2010-01-06 15:59:42Z laurentp
 *          $
 */
public class SynchronousMessageEntry implements Serializable {

	private static final long serialVersionUID = 1L;

	public enum Status {
		ALL_RESPONSES_RECEIVED, RECEIPT_IN_PROGRESS
	};

	private Status status = Status.RECEIPT_IN_PROGRESS;

	private SynchronousMessage<?> response;

	private final int nbStepsWhenSynchronizationPointCreated;
	
	private final int numberOfResponseWaiting;

	private int numberOfResponseReceived = 0;

	public SynchronousMessageEntry(int numberOfResponseWaiting, int nbStepsWhenSynchronizationPointCreated) {
	    this.numberOfResponseWaiting = numberOfResponseWaiting;
	    this.nbStepsWhenSynchronizationPointCreated = nbStepsWhenSynchronizationPointCreated;
	}

	/**
	 * Returns the numberOfResponseReceived
	 * 
	 * @return the numberOfResponseReceived
	 */
	public int getNumberOfResponseReceived() {
		return this.numberOfResponseReceived;
	}

	/**
	 * Returns the numberOfResponseWaiting
	 * 
	 * @return the numberOfResponseWaiting
	 */
	public int getNumberOfResponseWaiting() {
		return this.numberOfResponseWaiting;
	}

	public int getNbStepsWhenSynchronizationPointCreated() {
        return nbStepsWhenSynchronizationPointCreated;
    }

    /**
	 * Returns the status
	 * 
	 * @return the status
	 */
	public Status getStatus() {
		return this.status;
	}

	public SynchronousMessage<?> getSynchronousMessage() {
		return this.response;
	}

	public void incrementNbResponsesReceivedBy(int increment) {
		this.numberOfResponseReceived += increment;
		if (this.numberOfResponseReceived == this.numberOfResponseWaiting) {
			this.status = Status.ALL_RESPONSES_RECEIVED;
		}

		assert this.numberOfResponseReceived <= this.numberOfResponseWaiting;
	}

	public void setSynchronousMessage(SynchronousMessage<?> response) {
		this.response = response;
	}

}
