package org.objectweb.proactive.extensions.structuredp2p.messages.query.can;

import org.objectweb.proactive.extensions.structuredp2p.messages.query.AbstractQueryMessage;
import org.objectweb.proactive.extensions.structuredp2p.messages.response.AbstractResponseMessage;
import org.objectweb.proactive.extensions.structuredp2p.messages.response.can.ForwardResponseMessage;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.AbstractCANOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.coordinates.CoordinateValue;
import org.objectweb.proactive.extensions.structuredp2p.router.Router;
import org.objectweb.proactive.extensions.structuredp2p.router.can.UnicastQueryRouter;
import org.objectweb.proactive.extensions.structuredp2p.validator.can.UnicastConstraintsValidator;

/**
 * A <code>ForwardQueryMessage</code> is a query message which can be used in
 * order to find a peer which manages a specified coordinate on a CAN structured
 * peer-to-peer network.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: ForwardQueryMessage.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class ForwardQueryMessage extends AbstractQueryMessage<CoordinateValue[]> {

    private static final long serialVersionUID = 1L;

    /**
     * The zone which is managed by the sender. It is used in order to send the
     * response when the keyToReach has been reached.
     */
    private CoordinateValue[] senderCoordinates;

    public ForwardQueryMessage(CoordinateValue[] coordinatesToReach) {
        super(coordinatesToReach);
    }

    /**
     * Returns the key which is managed by the sender in order to send the
     * response when the keyToReach has been reached.
     * 
     * @return the key which is managed by the sender in order to send the
     *         response when the keyToReach has been reached.
     */
    public CoordinateValue[] getSenderCoordinates() {
        return this.senderCoordinates;
    }

    /**
     * {@inheritDoc}
     */
    public Router<ForwardQueryMessage, CoordinateValue[]> getRouter() {
        return new UnicastQueryRouter<ForwardQueryMessage>(
                    new UnicastConstraintsValidator());
    }

    /**
     * {@inheritDoc}
     */
    public void route(StructuredOverlay overlay) {
        if (this.senderCoordinates == null) {
            this.senderCoordinates = ((AbstractCANOverlay) overlay).getZone().getCoordinateMin();
        }
        super.route(overlay);
    }

    /**
     * {@inheritDoc}
     */
    public AbstractResponseMessage<CoordinateValue[]> createResponseMessage() {
        return new ForwardResponseMessage(this);
    }

}
