package org.objectweb.proactive.extensions.structuredp2p.messages.query.can;

import org.objectweb.proactive.extensions.structuredp2p.messages.response.AbstractResponseMessage;
import org.objectweb.proactive.extensions.structuredp2p.messages.response.can.LookupResponseMessage;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.coordinates.CoordinateValue;
import org.objectweb.proactive.extensions.structuredp2p.router.Router;
import org.objectweb.proactive.extensions.structuredp2p.router.can.UnicastQueryRouter;
import org.objectweb.proactive.extensions.structuredp2p.validator.can.UnicastConstraintsValidator;

/**
 * A <code>LookupQueryMessage</code> is a query message which can be used in
 * order to find a peer which manages a specified coordinate on a CAN structured
 * peer-to-peer network.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: LookupQueryMessage.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class LookupQueryMessage extends ForwardQueryMessage {

    private static final long serialVersionUID = 1L;

    protected Peer remotePeerReached;

    public LookupQueryMessage(CoordinateValue[] coordinatesToReach) {
        super(coordinatesToReach);
    }

    public Peer getRemotePeerReached() {
        return this.remotePeerReached;
    }

    public void setRemotePeerReached(Peer remotePeerReached) {
        this.remotePeerReached = remotePeerReached;
    }

    /**
     * {@inheritDoc}
     */
    public Router<ForwardQueryMessage, CoordinateValue[]> getRouter() {
        return new UnicastQueryRouter<ForwardQueryMessage>(new UnicastConstraintsValidator()) {
            protected void onDestinationReached(StructuredOverlay overlay, ForwardQueryMessage msg) {
                ((LookupQueryMessage) msg).setRemotePeerReached(overlay.getRemotePeer());
            };
        };
    }

    /**
     * {@inheritDoc}
     */
    public AbstractResponseMessage<CoordinateValue[]> createResponseMessage() {
        return new LookupResponseMessage(this, this.remotePeerReached);
    }

}
