package org.objectweb.proactive.extensions.structuredp2p.messages.query.chord;

import org.objectweb.proactive.api.PAFuture;
import org.objectweb.proactive.extensions.structuredp2p.messages.query.AbstractQueryMessage;
import org.objectweb.proactive.extensions.structuredp2p.messages.response.chord.FingerResponseMessage;
import org.objectweb.proactive.extensions.structuredp2p.operations.chord.GetKeyOperation;
import org.objectweb.proactive.extensions.structuredp2p.operations.chord.GetKeyResponseOperation;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.chord.AbstractChordOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.chord.Finger;
import org.objectweb.proactive.extensions.structuredp2p.overlay.chord.Key;
import org.objectweb.proactive.extensions.structuredp2p.router.Router;
import org.objectweb.proactive.extensions.structuredp2p.router.chord.FingerQueryRouter;
import org.objectweb.proactive.extensions.structuredp2p.validator.ConstraintsValidator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Message used to perform a lookup for a {@link Key} by using {@link Finger}s.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: FingerQueryMessage.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public abstract class FingerQueryMessage extends AbstractQueryMessage<Key> {

    private static final long serialVersionUID = 1L;

    private final static transient Logger logger = 
        LoggerFactory.getLogger(FingerQueryMessage.class);

    protected Key senderKey;

    public FingerQueryMessage(final Key keyToReach) {
        super(keyToReach);
    }

    public FingerResponseMessage createResponseMessage() {
        return new FingerResponseMessage(this, this.senderKey);
    }

    public abstract void onDestinationReached(StructuredOverlay overlay);
    
    public Router<FingerQueryMessage, Key> getRouter() {
        ConstraintsValidator<Key> validator =  new ConstraintsValidator<Key>() {
            public boolean validatesKeyConstraints(StructuredOverlay overlay, Key key) {
                AbstractChordOverlay<?> chordOverlay = ((AbstractChordOverlay<?>) overlay);

                Key predecessorKey = ((GetKeyResponseOperation) PAFuture.getFutureValue(chordOverlay
                        .getPredecessor().send(new GetKeyOperation()))).getKey();

                if (getKeyToReach().compareTo(chordOverlay.getKey()) == 0
                        || getKeyToReach().isBetween(predecessorKey, chordOverlay.getKey())) {
                    return true;
                }

                return false;
            };
        };
        
        return new FingerQueryRouter<FingerQueryMessage>(validator) {
            protected void onDestinationReached(StructuredOverlay overlay, FingerQueryMessage msg) {
                logger.debug("FingerQueryMessage is onDestinationReach");
                msg.onDestinationReached(overlay);
            }
        };
    }

    public void route(StructuredOverlay overlay) {
        if (this.senderKey == null) {
            this.senderKey = ((AbstractChordOverlay<?>) overlay).getKey();
        }
        super.route(overlay);
    }

}
