package org.objectweb.proactive.extensions.structuredp2p.messages.query.chord;

import java.util.ArrayList;

import org.objectweb.proactive.extensions.structuredp2p.messages.query.AbstractQueryMessage;
import org.objectweb.proactive.extensions.structuredp2p.messages.response.AbstractResponseMessage;
import org.objectweb.proactive.extensions.structuredp2p.messages.response.chord.TurnAroundResponseMessage;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.chord.AbstractChordOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.chord.Key;
import org.objectweb.proactive.extensions.structuredp2p.router.Router;
import org.objectweb.proactive.extensions.structuredp2p.router.chord.TurnAroundRouter;
import org.objectweb.proactive.extensions.structuredp2p.validator.ConstraintsValidator;

/**
 * Message used to iterate on each {@link Peer} over the ring.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: TurnAroundQueryMessage.java 3576 2010-03-04 15:20:20Z laurentp
 *          $
 */
public abstract class TurnAroundQueryMessage<V> extends AbstractQueryMessage<Key> {

    private static final long serialVersionUID = 1L;

    private Key senderKey;

    private boolean onSender = true;

    protected ArrayList<V> values = new ArrayList<V>();

    public TurnAroundQueryMessage() {
        super(null);
    }

    public AbstractResponseMessage<Key> createResponseMessage() {
        return new TurnAroundResponseMessage<V>(this, senderKey);
    }

    public Router<TurnAroundQueryMessage<V>, Key> getRouter() {
        ConstraintsValidator<Key> validator = new ConstraintsValidator<Key>() {
            public boolean validatesKeyConstraints(StructuredOverlay overlay, Key key) {
                return key.equals(((AbstractChordOverlay<?>) overlay).getKey());
            }
        };
        
        return new TurnAroundRouter<TurnAroundQueryMessage<V>>(validator);
    }

    public void route(StructuredOverlay overlay) {
        if (this.senderKey == null) {
            this.senderKey = ((AbstractChordOverlay<?>) overlay).getKey();
            super.setKeyToReach(this.senderKey);
        }
        super.route(overlay);
    }
    
    public ArrayList<V> getValues() {
        return this.values;
    }
    
    public boolean isOnSender() {
        return this.onSender;
    }

    public void setOnSender(boolean onSender) {
        this.onSender = onSender;
    }
    
    public abstract void updateValues(StructuredOverlay overlay);

}
