package org.objectweb.proactive.extensions.structuredp2p.messages.response.can;

import org.objectweb.proactive.extensions.structuredp2p.api.messages.Response;
import org.objectweb.proactive.extensions.structuredp2p.messages.SynchronousMessageEntry;
import org.objectweb.proactive.extensions.structuredp2p.messages.query.can.ForwardQueryMessage;
import org.objectweb.proactive.extensions.structuredp2p.messages.response.AbstractResponseMessage;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.coordinates.CoordinateValue;
import org.objectweb.proactive.extensions.structuredp2p.router.can.UnicastResponseRouter;
import org.objectweb.proactive.extensions.structuredp2p.validator.can.UnicastConstraintsValidator;

/**
 * Response associated to {@link ForwardQueryMessage}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: ForwardResponseMessage.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class ForwardResponseMessage extends
        AbstractResponseMessage<CoordinateValue[]> {

    private static final long serialVersionUID = 1L;
    
    public ForwardResponseMessage(ForwardQueryMessage query) {
        super(query, query.getSenderCoordinates());
    }

    /**
     * Handles the last step for the current response using the specified
     * {@link StructuredOverlay}.
     * 
     * @param overlay
     *            the overlay to use in order to handle the response.
     */
    public void handle(StructuredOverlay overlay) {
        SynchronousMessageEntry entry = overlay.getSynchronousMessagesReceived().get(
                super.getUUID());
        entry.incrementNbResponsesReceivedBy(1);
        entry.setSynchronousMessage(this);

        synchronized (overlay.getSynchronousMessagesReceived()) {
            overlay.getSynchronousMessagesReceived().notifyAll();
        }
    }

    /**
     * {@inheritDoc}
     */
    public UnicastResponseRouter<ForwardResponseMessage> getRouter() {
        return new UnicastResponseRouter<ForwardResponseMessage>(new UnicastConstraintsValidator());
    }

    /**
     * Nothing to do because LookupResponseMessage is not accessible from the
     * public API.  
     */
    public Response createResponse() {
        return null;
    }


}
