package org.objectweb.proactive.extensions.structuredp2p.operations.can;

import org.objectweb.proactive.extensions.structuredp2p.operations.EmptyResponseOperation;
import org.objectweb.proactive.extensions.structuredp2p.operations.Operation;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.AbstractCANOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.NeighborEntry;

/**
 * A {@code AddNeighbor} is used in order to add a neighbor to {@link Peer} on
 * the network.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: AddAndUpdateNeighborOperation.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class AddAndUpdateNeighborOperation implements Operation {

    private static final long serialVersionUID = 1L;

    private NeighborEntry neighborToAdd;

    private NeighborEntry neighborToUpdate;

    /**
     * The dimension on which we add the peer as neighbor.
     */
    private final int dimension;

    /**
     * The direction on which we add the peer as neighbor.
     */
    private final int direction;

    /**
     * Constructor.
     * 
     * @param entryToAdd
     *            the new neighbor to add.
     * @param entryToUpdate
     *            the zone from the existing neighbor to update.
     * @param dimension
     *            the dimension on which the peer must be add as neighbor.
     * @param direction
     *            the direction on which the peer must be add as neighbor.
     */
    public AddAndUpdateNeighborOperation(NeighborEntry entryToAdd, NeighborEntry entryToUpdate,
            int dimension,
            int direction) {
        this.neighborToAdd = entryToAdd;
        this.neighborToUpdate = entryToUpdate;
        this.dimension = dimension;
        this.direction = direction;
    }

    /**
     * Returns the dimension on which the peer must be add as neighbor.
     * 
     * @return the dimension on which the peer must be add as neighbor.
     */
    public int getDimension() {
        return this.dimension;
    }

    /**
     * Returns the direction on which the peer must be add as neighbor.
     * 
     * @return the direction on which the peer must be add as neighbor.
     */
    public int getDirection() {
        return this.direction;
    }

    public NeighborEntry getNeighborEntryToAdd() {
        return this.neighborToAdd;
    }

    public NeighborEntry getNeighborEntryToUpdate() {
        return this.neighborToUpdate;
    }

    /**
     * Handles a {@link AddAndUpdateNeighborOperation} by double dispatch.
     * 
     * @param overlay
     *            the overlay which handles the message.
     */
    public EmptyResponseOperation handle(StructuredOverlay overlay) {
        return ((AbstractCANOverlay) overlay).handleAddAndUpdateNeighborMessage(this);
    }

}
