package org.objectweb.proactive.extensions.structuredp2p.operations.can;

import org.objectweb.proactive.extensions.structuredp2p.operations.Operation;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.AbstractCANOverlay;

/**
 * A {@code CANJoinMessage} is used when a {@link Peer} want to join an another
 * {@link Peer} which is already on the network.
 * 
 * @author Alexandre Trovato
 * @author Fanny Kilanga
 * @author Laurent Pellegrino
 * 
 * @version $Id: JoinOperation.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class JoinOperation implements Operation {

    private static final long serialVersionUID = 1L;
    
    /**
     * The remote peer.
     */
    private final Peer remotePeer;

    /**
     * Constructor.
     * 
     * @param remotePeer
     *            the remote peer.
     */
    public JoinOperation(Peer remotePeer) {
        super();
        this.remotePeer = remotePeer;
    }

    /**
     * Returns the remote peer.
     * 
     * @return the remote peer
     */
    public Peer getRemotePeer() {
        return this.remotePeer;
    }

    /**
     * {@inheritDoc}
     */
    public JoinResponseOperation handle(StructuredOverlay overlay) {
        return ((AbstractCANOverlay) overlay).handleJoinMessage(this);
    }

}
