package org.objectweb.proactive.extensions.structuredp2p.operations.can;

import java.util.LinkedList;
import java.util.UUID;

import org.objectweb.proactive.extensions.structuredp2p.operations.ResponseOperation;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.NeighborTable;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.SplitEntry;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.Zone;

/**
 * Associated response for {@link JoinOperation}.
 * 
 * @author Kilanga Fanny
 * @author Laurent Pellegrino
 * @author Trovato Alexandre
 * 
 * @version $Id: JoinResponseOperation.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class JoinResponseOperation implements ResponseOperation {

    private static final long serialVersionUID = 1L;

    private final int dimension;

    private final int direction;

    private final UUID peerToUpdate;

    private final Zone zoneForUpdate;

    private final Object dataAffected;

    private final NeighborTable neighborsAffected;

    private final Zone zoneAffected;

    private final LinkedList<SplitEntry> splitHistory;

    /**
     * Constructor
     * 
     * @param dimension
     *            the dimension.
     * @param directionInv
     *            the direction.
     * @param peerToUpdate
     *            the identifier of the peer to update.
     * @param zoneForUpdate
     *            zone to update.
     * @param zoneAffected
     *            the zone of the new peer.
     * @param neighborsAffected
     *            the neighbors for the new peer.
     * @param dataAffected
     *            data affected to the new peer.
     * @param splitHistory
     *            the splitHistory of the remote peer.
     */
    public JoinResponseOperation(int dimension, int directionInv, UUID peerToUpdate,
            Zone zoneForUpdate, Zone zoneAffected, NeighborTable neighborsAffected,
            Object dataAffected, LinkedList<SplitEntry> splitHistory) {
        this.dimension = dimension;
        this.direction = directionInv;

        this.peerToUpdate = peerToUpdate;
        this.zoneForUpdate = zoneForUpdate;

        this.dataAffected = dataAffected;
        this.neighborsAffected = neighborsAffected;
        this.zoneAffected = zoneAffected;
        this.splitHistory = splitHistory;
    }

    public int getDimension() {
        return this.dimension;
    }

    public int getDirection() {
        return this.direction;
    }

    public UUID getPeerToUpdate() {
        return this.peerToUpdate;
    }

    public Zone getZoneForUpdate() {
        return this.zoneForUpdate;
    }

    public Object getDataAffected() {
        return this.dataAffected;
    }

    public NeighborTable getNeighborsAffected() {
        return this.neighborsAffected;
    }

    public Zone getZoneAffected() {
        return this.zoneAffected;
    }

    public LinkedList<SplitEntry> getSplitHistory() {
        return this.splitHistory;
    }

}
