package org.objectweb.proactive.extensions.structuredp2p.operations.can;

import java.util.UUID;

import org.objectweb.proactive.extensions.structuredp2p.operations.BooleanResponseOperation;
import org.objectweb.proactive.extensions.structuredp2p.operations.Operation;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.AbstractCANOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.NeighborTable;

/**
 * A {@code RemoveNeighborOperation} is used to remove the specified {@link Peer}
 * (by using the given identifier) from the {@link NeighborTable} of the peer
 * which receives this message.
 * 
 * @author Alexandre Trovato
 * @author Fanny Kilanga
 * @author Laurent Pellegrino
 * 
 * @version $Id: RemoveNeighborOperation.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class RemoveNeighborOperation implements Operation {

    private static final long serialVersionUID = 1L;

    /**
     * The neighbor peer to remove.
     */
    private final UUID peerIdentifier;

    /**
     * The dimension of the neighbor to remove.
     */
    private int dimension;

    /**
     * The direction of the neighbor to remove.
     */
    private int direction = NeighborTable.ANY_DIRECTION;

    /**
     * Constructor.
     * 
     * @param peerIdentifier
     *            the peer identifier which identify the neighbor to remove.
     */
    public RemoveNeighborOperation(UUID peerIdentifier) {
        this.peerIdentifier = peerIdentifier;
    }

    /**
     * Constructor.
     * 
     * @param peerIdentifier
     *            the peer identifier which identify the neighbor to remove.
     * @param dimension
     *            the dimension of the neighbor to remove.
     * @param direction
     *            the direction of the neighbor to remove
     */
    public RemoveNeighborOperation(UUID peerIdentifier, int dimension, int direction) {
        assert dimension >= 0 && dimension < AbstractCANOverlay.NB_DIMENSIONS
                && direction >= 0 && direction < 2;

        this.peerIdentifier = peerIdentifier;
        this.dimension = dimension;
        this.direction = direction;
    }

    public UUID getPeerIdentifier() {
        return this.peerIdentifier;
    }

    /**
     * Returns the dimension of the neighbor to remove.
     * 
     * @return the dimension of the neighbor to remove.
     */
    public int getDimension() {
        return this.dimension;
    }

    /**
     * Returns the direction of the neighbor to remove.
     * 
     * @return the direction of the neighbor to remove.
     */
    public int getDirection() {
        return this.direction;
    }

    /**
     * {@inheritDoc}
     */
    public BooleanResponseOperation handle(StructuredOverlay overlay) {
        return ((AbstractCANOverlay) overlay).handleRemoveNeighborMessage(this);
    }

}
