package org.objectweb.proactive.extensions.structuredp2p.operations.can;

import java.util.UUID;

import org.objectweb.proactive.extensions.structuredp2p.operations.EmptyResponseOperation;
import org.objectweb.proactive.extensions.structuredp2p.operations.Operation;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.AbstractCANOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.NeighborEntry;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.NeighborTable;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.Zone;

/**
 * Message used in order to update a {@link Zone} of a neighbor cached in the
 * {@link NeighborTable} of the peer which receives the message.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: UpdateZoneCacheOperation.java 4853 2010-07-05 13:39:56Z
 *          laurentp $
 */
public class UpdateZoneCacheOperation implements Operation {

    private static final long serialVersionUID = 1L;

    private final UUID neighborID;

    private final Zone newZoneToSet;

    /**
     * Constructs a new UpdateZoneCacheOperation with the specified
     * <code>neighborID</code> and <code>newZoneToSet</code>.
     * 
     * @param neighborID
     *            the identifier of the neighbor to use for update.
     * @param newZoneToSet
     *            the new {@link Zone} to set.
     */
    public UpdateZoneCacheOperation(UUID neighborID, Zone newZoneToSet) {
        this.neighborID = neighborID;
        this.newZoneToSet = newZoneToSet;
    }

    /**
     * Handles a {@link UpdateZoneCacheOperation} by double dispatch.
     * 
     * @param overlay
     *            the overlay which handles the message.
     */
    public EmptyResponseOperation handle(StructuredOverlay overlay) {
        NeighborEntry entry = ((AbstractCANOverlay) overlay).getNeighborTable().getNeighborEntry(
                this.neighborID);
        if (entry != null) {
            entry.setZone(newZoneToSet);
        }

        return new EmptyResponseOperation();
    }

}
