package org.objectweb.proactive.extensions.structuredp2p.operations.chord;

import org.objectweb.proactive.extensions.structuredp2p.operations.ResponseOperation;
import org.objectweb.proactive.extensions.structuredp2p.operations.BooleanResponseOperation;
import org.objectweb.proactive.extensions.structuredp2p.operations.Operation;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.chord.AbstractChordOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.chord.Entry;

/**
 * Add a new entryToPut on the appropriate {@link Peer} (i.e. respecting Chord
 * property which says that entryToPut identified by key K will be managed by
 * the closest successor of K) by performing a lookup from the peer which
 * receives this message.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: PutEntryOperation.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class PutEntryOperation<T> implements Operation {

    private static final long serialVersionUID = 1L;
    
    private Entry<T> entryToPut;

    /**
     * Constructor.
     * 
     * @param entryToPut
     *            the entryToPut to put.
     */
    public PutEntryOperation(Entry<T> entryToPut) {
        this.entryToPut = entryToPut;
    }

    @SuppressWarnings("unchecked")
    public ResponseOperation handle(StructuredOverlay overlay) {
        return new BooleanResponseOperation(
                ((AbstractChordOverlay<T>) overlay).putEntry(
                        this.entryToPut));
    }

}
