package org.objectweb.proactive.extensions.structuredp2p.overlay.can;

import java.io.Serializable;
import java.util.UUID;

import org.objectweb.proactive.api.PAFuture;
import org.objectweb.proactive.extensions.structuredp2p.operations.can.GetIdAndZoneOperation;
import org.objectweb.proactive.extensions.structuredp2p.operations.can.GetIdAndZoneResponseOperation;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;

/**
 * NeighborEntry is an entry in a {@link NeighborTable}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: NeighborEntry.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class NeighborEntry implements Serializable {

    private static final long serialVersionUID = 1L;

    private final UUID neighborIdentifier;

    private final Peer neighborStub;

    private Zone neighborZone;

    public NeighborEntry(Peer peerStub) {
        this.neighborStub = peerStub;

        GetIdAndZoneResponseOperation response = (GetIdAndZoneResponseOperation) PAFuture
                .getFutureValue(this.neighborStub.send(new GetIdAndZoneOperation()));

        this.neighborIdentifier = response.getPeerIdentifier();
        this.neighborZone = response.getPeerZone();
    }

    public NeighborEntry(UUID peerIdentifier, Peer peerStub, Zone peerZone) {
        super();
        this.neighborIdentifier = peerIdentifier;
        this.neighborStub = peerStub;
        this.neighborZone = peerZone;
    }

    public UUID getIdentifier() {
        return this.neighborIdentifier;
    }

    public Peer getStub() {
        return this.neighborStub;
    }

    public Zone getZone() {
        return this.neighborZone;
    }

    public void setZone(Zone peerZone) {
        this.neighborZone = peerZone;
    }

}
