package org.objectweb.proactive.extensions.structuredp2p.overlay.chord;

import java.io.Serializable;

/**
 * An entry represents a couple ({@link Key}, T) stored on the overlay.
 * 
 * @param <T>
 *            the type of the resource associated to the Entry.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: Entry.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class Entry<T> implements Serializable {

    private static final long serialVersionUID = 1L;

    private Key key;

    private T value;

    /**
     * Constructor.
     * 
     * @param key
     *            the key to use.
     * @param value
     *            the value to use.
     */
    public Entry(Key key, T value) {
        this.key = key;
        this.value = value;
    }

    /**
     * Returns the key associated to this entry.
     * 
     * @return the key associated to this entry.
     */
    public Key getKey() {
        return this.key;
    }

    /**
     * Returns the value associated to this entry.
     * 
     * @return the value associated to this entry.
     */
    public T getValue() {
        return this.value;
    }

    /**
     * {@inheritDoc}
     */
    public String toString() {
        return this.value.toString();
    }

}
