package org.objectweb.proactive.extensions.structuredp2p.overlay.chord;

import java.io.Serializable;
import java.math.BigInteger;

import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;

/**
 * A finger is an abstraction of Chord. It can be see like a successor to the
 * power of 2.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: Finger.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class Finger implements Serializable {

    private static final long serialVersionUID = 1L;

    private BigInteger startIndex;

    private Peer node;

    public Finger(BigInteger startIndex, Peer node) {
        super();
        this.startIndex = startIndex;
        this.node = node;
    }

    /**
     * Returns the peer to which this finger is associated.
     * 
     * @return the peer to which this finger is associated.
     */
    public Peer getNode() {
        return this.node;
    }

    /**
     * Sets the peer to which this finger is associated.
     * 
     * @param node
     *            the new peer to use.
     */
    public void setNode(Peer node) {
        this.node = node;
    }

    /**
     * Returns the identifier to which this finger is associated.
     * 
     * @return the identifier to which this finger is associated.
     */
    public BigInteger getStartIndex() {
        return this.startIndex;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
        return "Finger[startIndex=" + this.startIndex + ", node=" + this.node + "]";
    }

}
