package org.objectweb.proactive.extensions.structuredp2p.overlay.chord;

import java.io.Serializable;
import java.math.BigInteger;

import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;

/**
 * Table that contains {@link Finger}. The number of fingers depends of the size
 * of the {@link Key} used to identify peers. The number of finger entries is
 * {@link Key#NB_BITS}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: FingerTable.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class FingerTable implements Serializable {

    private static final long serialVersionUID = 1L;
    
    private Finger[] fingers = new Finger[Key.NB_BITS];

    /**
     * Constructor.
     * 
     * @param overlay
     *            the {@link AbstractChordOverlay} of the creator of the finger
     *            table.
     * 
     * @param remotePeer
     *            the stud of the creator of the finger table.
     */
    public FingerTable(AbstractChordOverlay<?> overlay, Peer remotePeer) {
        for (int i = 0; i < Key.NB_BITS; i++) {
            this.fingers[i] = new Finger(overlay.getKey().getValue().add(
                    BigInteger.valueOf(2).pow(i)).mod(BigInteger.valueOf(2).pow(Key.NB_BITS)),
                    remotePeer);
        }
    }

    /**
     * Returns the finger at the specified position.
     * 
     * @param index
     *            the position.
     * @return the finger found at the specified position.
     */
    public Finger getFinger(int index) {
        return this.fingers[index];
    }

    /**
     * Returns all the fingers.
     * 
     * @return all the fingers.
     */
    public Finger[] getFingers() {
        return this.fingers;
    }

    public int size() {
        return this.fingers.length;
    }

}
