package org.objectweb.proactive.extensions.structuredp2p.router.chord;

import org.objectweb.proactive.extensions.structuredp2p.messages.query.chord.FingerQueryMessage;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.chord.AbstractChordOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.chord.Key;
import org.objectweb.proactive.extensions.structuredp2p.router.Router;
import org.objectweb.proactive.extensions.structuredp2p.validator.ConstraintsValidator;

/**
 * Route message to a specified keyToReach by using the finger table of nodes.
 * 
 * @author Laurent Pellegrino
 * @version $Id: FingerQueryRouter.java 5022 2010-08-14 16:14:45Z plaurent $
 * 
 * @param <T>
 *            the type of query message to route.
 */
public class FingerQueryRouter<T extends FingerQueryMessage> extends Router<T, Key> {

    public FingerQueryRouter(ConstraintsValidator<Key> validator) {
        super(validator);
    }

    protected void performHandle(StructuredOverlay overlay, T msg) {
	this.onDestinationReached(overlay, msg);
        msg.createResponseMessage().route(overlay);
    }

    protected void performRoute(StructuredOverlay overlay, T msg) {
        if (super.validatesKeyConstraints(overlay, msg.getKeyToReach())) {
            this.performHandle(overlay, msg);
        } else {
            msg.incrementNbStepsBy(1);
            ((AbstractChordOverlay<?>) overlay).findSuccessor(msg.getKeyToReach()).route(msg);
        }
    }

}
