package org.objectweb.proactive.extensions.structuredp2p.router.chord;

import org.objectweb.proactive.extensions.structuredp2p.messages.query.chord.TurnAroundQueryMessage;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.chord.AbstractChordOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.chord.Key;
import org.objectweb.proactive.extensions.structuredp2p.router.Router;
import org.objectweb.proactive.extensions.structuredp2p.validator.ConstraintsValidator;

/**
 * Route message in order to force it to do a complete turn of the chord ring by
 * using successors pointers.
 * 
 * @author Laurent Pellegrino
 * @version $Id: TurnAroundRouter.java 5022 2010-08-14 16:14:45Z plaurent $
 * 
 * @param <T>
 *            the type of the message to route.
 */
public class TurnAroundRouter<T extends TurnAroundQueryMessage<?>> extends Router<T, Key> {

    public TurnAroundRouter(ConstraintsValidator<Key> validator) {
        super(validator);
    }

    protected void performHandle(StructuredOverlay overlay, T msg) {
        overlay.getQueryManager().putFinalResponseAndNotifyInitialSender(
                msg.createResponseMessage());
    }

    protected void performRoute(StructuredOverlay overlay, T msg) {
        // First time on route : we are on initial sender we need to skip it.
        if (msg.isOnSender()) {
            msg.setOnSender(false);
            ((AbstractChordOverlay<?>) overlay).getSuccessor().route(msg);
        } else {
            msg.incrementNbStepsBy(1);
            msg.updateValues(overlay);
            // Query is back on the initial sender
            if (super.validatesKeyConstraints(overlay, msg.getKeyToReach())) {
                super.onDestinationReached(overlay, msg);
                this.performHandle(overlay, msg);
            } else { // Need to route query to reach initial sender
                ((AbstractChordOverlay<?>) overlay).getSuccessor().route(msg);
            }
        }
    }

}
