package org.objectweb.proactive.extensions.structuredp2p.util;

import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * Utility class used for hash functions.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: HashUtil.java 4853 2010-07-05 13:39:56Z laurentp $
 */
public class HashUtil {

    /**
     * Calculates the hash of a specified String.
     * 
     * @param key
     *            the key to hash.
     * @return an array of bytes which the hash of the specified key.
     */
    public static byte[] getSHA1(String key) {
        MessageDigest md = null;
        try {
            md = MessageDigest.getInstance("SHA-1");
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        return md.digest(key.getBytes());
    }

    /**
     * Calculates the hash of a specified key as String to a given modulo.
     * 
     * @param key
     *            the key to use.
     * @param modulo
     *            the modulo to use.
     * @return a {@link BigInteger} which the associated hash to a given key and
     *         modulo.
     */
    public static BigInteger getSHA1KeyRestrictedToIntervalByModulo(String key, int modulo) {
        BigInteger hashedKey = new BigInteger(HashUtil.getSHA1(key)).abs();
        BigInteger mod = new BigInteger(Integer.toString(modulo));
        return hashedKey.mod(mod);
    }

}
