package org.objectweb.proactive.extensions.structuredp2p.core.overlay.can;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.coordinates.CoordinateValue;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.coordinates.LexicographicCoordinateValue;

/**
 * Test cases for {@link CoordinateValue}.
 * 
 * @author Alexandre Trovato
 * @author Laurent Pellegrino
 * 
 * @version $Id: TestCoordinate.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class TestCoordinate {

    private CoordinateValue coordinateValueA;

    private CoordinateValue coordinateValueB;

    private CoordinateValue coordinateValueC;

    @Before
    public void setUp() throws Exception {
        this.coordinateValueB = new LexicographicCoordinateValue("e");
        this.coordinateValueC = new LexicographicCoordinateValue("l");
    }

    @Test
    public void testCompare() {
        Assert.assertTrue(this.coordinateValueB.compareTo(this.coordinateValueC) < 0);
        Assert.assertTrue(this.coordinateValueC.compareTo(this.coordinateValueB) > 0);

        this.coordinateValueA = new LexicographicCoordinateValue("e");
        Assert.assertTrue(this.coordinateValueB.compareTo(this.coordinateValueA) == 0);
        Assert.assertTrue(this.coordinateValueA.compareTo(this.coordinateValueB) == 0);

        this.coordinateValueA = new LexicographicCoordinateValue("a");
        Assert.assertTrue(this.coordinateValueC.compareTo(this.coordinateValueA) > 0);
        Assert.assertTrue(this.coordinateValueA.compareTo(this.coordinateValueC) < 0);

        this.coordinateValueA = new LexicographicCoordinateValue("l");
        Assert.assertTrue(this.coordinateValueC.compareTo(this.coordinateValueA) == 0);
        Assert.assertTrue(this.coordinateValueA.compareTo(this.coordinateValueC) == 0);
    }

    @Test
    public void testIsBetween() {
        this.coordinateValueA = new LexicographicCoordinateValue("a");
        Assert.assertFalse(CoordinateValue.isBetween(this.coordinateValueA, this.coordinateValueB,
                this.coordinateValueC));

        this.coordinateValueA = new LexicographicCoordinateValue("g");
        Assert.assertTrue(CoordinateValue.isBetween(this.coordinateValueA, this.coordinateValueB,
                this.coordinateValueC));

        this.coordinateValueA = new LexicographicCoordinateValue("eff");
        Assert.assertTrue(CoordinateValue.isBetween(this.coordinateValueA, this.coordinateValueB,
                this.coordinateValueC));

        this.coordinateValueA = new LexicographicCoordinateValue("lee");
        Assert.assertFalse(CoordinateValue.isBetween(this.coordinateValueA, this.coordinateValueB,
                this.coordinateValueC));
    }

    @Test
    public void testMiddle() {
        Assert.assertEquals(new LexicographicCoordinateValue("h\u0001"), this.coordinateValueB
                .getMiddleWith(this.coordinateValueC));

        Assert.assertNotSame(new LexicographicCoordinateValue("e"), this.coordinateValueB
                .getMiddleWith(this.coordinateValueC));
    }

    @Test
    public void testGetMiddle() {
        CoordinateValue middleCoordinate = this.coordinateValueB;
        for (int nbSplit = 0; nbSplit < 500; nbSplit++) {
            middleCoordinate = middleCoordinate.getMiddleWith(this.coordinateValueC);
            Assert.assertTrue(middleCoordinate.compareTo(this.coordinateValueC) < 0);
        }
    }

    @After
    public void tearDown() {
        this.coordinateValueA = null;
        this.coordinateValueC = null;
        this.coordinateValueB = null;
    }
}
