package org.objectweb.proactive.extensions.structuredp2p.core.overlay.can;

import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.AbstractCANOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.Zone;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.ZoneException;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.coordinates.CoordinateValue;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.coordinates.LexicographicCoordinateValue;

/**
 * Test cases for class {@link Zone}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: TestZone.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class TestZone {

    private static Zone zone;

    @BeforeClass
    public static void setUp() throws Exception {
        TestZone.zone = new Zone();
    }

    @Test
    public void testZone() {
        CoordinateValue[] coordinate = new CoordinateValue[AbstractCANOverlay.NB_DIMENSIONS];

        for (int i = 0; i < AbstractCANOverlay.NB_DIMENSIONS; i++) {
            coordinate[i] = new LexicographicCoordinateValue("" + Zone.LOWER_BOUND);
        }
        Assert.assertArrayEquals(TestZone.zone.getCoordinateMin(), coordinate);

        for (int i = 0; i < AbstractCANOverlay.NB_DIMENSIONS; i++) {
            coordinate[i] = new LexicographicCoordinateValue("" + Zone.UPPER_BOUND);
        }
        Assert.assertArrayEquals(TestZone.zone.getCoordinateMax(), coordinate);
    }

    @Test
    public void testSplitAndMerge() throws ZoneException {
        Zone[] newZones = TestZone.zone.split(0);
        CoordinateValue[] coordinate = new CoordinateValue[AbstractCANOverlay.NB_DIMENSIONS];

        for (int i = 0; i < AbstractCANOverlay.NB_DIMENSIONS; i++) {
            coordinate[i] = new LexicographicCoordinateValue(new String(
                    new char[] { Zone.LOWER_BOUND }));
        }
        Assert.assertArrayEquals(newZones[0].getCoordinateMin(), coordinate);

        coordinate[0] = (new LexicographicCoordinateValue(new String(
                new char[] { Zone.LOWER_BOUND })))
                .getMiddleWith(new LexicographicCoordinateValue(new String(
                        new char[] { Zone.UPPER_BOUND })));
        for (int i = 1; i < AbstractCANOverlay.NB_DIMENSIONS; i++) {
            coordinate[i] = new LexicographicCoordinateValue(new String(
                    new char[] { Zone.UPPER_BOUND }));
        }
        Assert.assertArrayEquals(newZones[0].getCoordinateMax(), coordinate);

        coordinate[0] = (new LexicographicCoordinateValue(new String(
                new char[] { Zone.LOWER_BOUND })))
                .getMiddleWith(new LexicographicCoordinateValue(new String(
                        new char[] { Zone.UPPER_BOUND })));
        for (int i = 1; i < AbstractCANOverlay.NB_DIMENSIONS; i++) {
            coordinate[i] = new LexicographicCoordinateValue(new String(
                    new char[] { Zone.LOWER_BOUND }));
        }
        Assert.assertArrayEquals(newZones[1].getCoordinateMin(), coordinate);

        for (int i = 0; i < AbstractCANOverlay.NB_DIMENSIONS; i++) {
            coordinate[i] = new LexicographicCoordinateValue(new String(
                    new char[] { Zone.UPPER_BOUND }));
        }
        Assert.assertArrayEquals(newZones[1].getCoordinateMax(), coordinate);

        Assert.assertEquals(newZones[0].getAbutsDimensionWith(newZones[1]), 0);
        Assert.assertTrue(newZones[0].isAbutsWith(newZones[1], 0));

        Zone mergedZone = newZones[0].merge(newZones[1]);
        Assert.assertEquals(mergedZone, TestZone.zone);
    }

    @AfterClass
    public static void tearDown() {
        TestZone.zone = null;
    }

}
