package org.objectweb.proactive.extensions.structuredp2p.core.overlay.chord;

import java.math.BigInteger;
import java.util.Random;

import junit.framework.Assert;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.objectweb.proactive.ActiveObjectCreationException;
import org.objectweb.proactive.api.PAFuture;
import org.objectweb.proactive.core.node.NodeException;
import org.objectweb.proactive.extensions.structuredp2p.api.PeerFactory;
import org.objectweb.proactive.extensions.structuredp2p.core.overlay.BasicChordOverlay;
import org.objectweb.proactive.extensions.structuredp2p.initializers.ChordNetworkInitializer;
import org.objectweb.proactive.extensions.structuredp2p.operations.BooleanResponseOperation;
import org.objectweb.proactive.extensions.structuredp2p.operations.chord.EntryResponseOperation;
import org.objectweb.proactive.extensions.structuredp2p.operations.chord.FindEntryOperation;
import org.objectweb.proactive.extensions.structuredp2p.operations.chord.GetSuccessorOperation;
import org.objectweb.proactive.extensions.structuredp2p.operations.chord.GetSuccessorResponseOperation;
import org.objectweb.proactive.extensions.structuredp2p.operations.chord.PutEntryOperation;
import org.objectweb.proactive.extensions.structuredp2p.overlay.OverlayType;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.overlay.chord.Entry;
import org.objectweb.proactive.extensions.structuredp2p.overlay.chord.Key;

/**
 * @author Laurent Pellegrino
 * 
 * @version $Id: TestBasicChordOverlay.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class TestBasicChordOverlay extends ChordNetworkInitializer {

    private static final int NB_NODES_TO_CREATE = 5;

    private static BigInteger counter = BigInteger.ZERO;

    @Before
    public void testCreation() {
        TestBasicChordOverlay.counter = BigInteger.ZERO;
        try {
            super.initializeNewNetwork(OverlayType.CHORD, NB_NODES_TO_CREATE);
        } catch (Exception e) {
            e.printStackTrace();
        }

        try {
            Thread.sleep(8000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    @Test
    public void testTurnAround() {
        Peer initialPeer = super.get(0);
        Peer currentPeer = super.get(1);
        int counter = 1;

        while (!initialPeer.equals(currentPeer)) {
            currentPeer = ((GetSuccessorResponseOperation) PAFuture.getFutureValue(currentPeer
                    .send(new GetSuccessorOperation()))).getSuccessor();
            counter++;
        }

        Assert.assertEquals(NB_NODES_TO_CREATE, counter);
    }

    @Test
    @SuppressWarnings("unchecked")
    public void testPut() {
        Key keyToUse = new Key(new Random().nextInt(NB_NODES_TO_CREATE));

        Assert.assertTrue(((BooleanResponseOperation) PAFuture.getFutureValue(super.getARandomPeer()
                .send(new PutEntryOperation<String>(new Entry<String>(keyToUse, "test")))))
                .getValue());

        Entry<String> entryFound = (Entry<String>) ((EntryResponseOperation<?>) PAFuture
                .getFutureValue(super.get(0).send(new FindEntryOperation(keyToUse)))).getEntryFound();

        Assert.assertTrue(entryFound != null);
        Assert.assertEquals("test", entryFound.getValue());
    }

    @Test
    public void testFindEntryFailure() {
        Assert.assertEquals(null, ((EntryResponseOperation<?>) PAFuture.getFutureValue(super
                .getARandomPeer().send(new FindEntryOperation(new Key(6))))).getEntryFound());
    }

    @After
    public void tearDown() {
        super.clearNetwork();
    }

    public Peer createPeer() throws ActiveObjectCreationException, NodeException {
        return PeerFactory.newActivePeer(
                new BasicChordOverlay(
                        TestBasicChordOverlay.generateIdentifier()));
    }

    private static BigInteger generateIdentifier() {
        if (TestBasicChordOverlay.counter.compareTo(Key.MAX_VALUE) > 0) {
            throw new IllegalStateException(
                    "Try to generate a key which is greater than 2^Key.NB_BITS");
        }

        TestBasicChordOverlay.counter = TestBasicChordOverlay.counter.add(BigInteger.valueOf(1));
        return TestBasicChordOverlay.counter;
    }

}
