package org.objectweb.proactive.extensions.structuredp2p.core.tracker;

import java.util.Arrays;

import junit.framework.Assert;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.objectweb.proactive.ActiveObjectCreationException;
import org.objectweb.proactive.api.PAActiveObject;
import org.objectweb.proactive.core.node.NodeException;
import org.objectweb.proactive.core.util.ProActiveRandom;
import org.objectweb.proactive.extensions.structuredp2p.api.PeerFactory;
import org.objectweb.proactive.extensions.structuredp2p.api.TrackerFactory;
import org.objectweb.proactive.extensions.structuredp2p.core.overlay.BasicCANOverlay;
import org.objectweb.proactive.extensions.structuredp2p.exceptions.StructuredP2PException;
import org.objectweb.proactive.extensions.structuredp2p.overlay.OverlayType;
import org.objectweb.proactive.extensions.structuredp2p.tracker.Tracker;

/**
 * Test that trackers can communicate and that each peer reference is replicated
 * to tracker references each tracker maintained.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: TestTracker.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class TestTracker {

    private static final int NB_TRACKERS = 5;

    private static Tracker[] trackers;

    @BeforeClass
    public static void setUp() {
        trackers = TrackerFactory.newActiveTrackerInParallel(OverlayType.CAN, NB_TRACKERS);

        for (Tracker tracker : Arrays.copyOfRange(trackers, 1, trackers.length)) {
            tracker.join(trackers[0]);
        }
    }

    @Test
    public void testLinksCreation() {
        for (int i = 0; i < trackers.length; i++) {
            Assert.assertEquals(NB_TRACKERS - 1, trackers[i].getTrackersGroup().size());
        }

    }

    @Test
    public void testPeerReferencesReplication() {
        try {
            trackers[ProActiveRandom.nextInt(trackers.length)].
                    addOnNetwork(PeerFactory.newActivePeer(new BasicCANOverlay()));
        } catch (ActiveObjectCreationException e) {
            e.printStackTrace();
        } catch (NodeException e) {
            e.printStackTrace();
        } catch (StructuredP2PException e) {
            e.printStackTrace();
        }

        for (int i = 0; i < trackers.length; i++) {
            Assert.assertEquals(1, trackers[i].getNbOfPeersStored());
        }
    }

    @AfterClass
    public static void tearDown() {
        for (Tracker tracker : trackers) {
            PAActiveObject.terminateActiveObject(tracker, true);
        }
        trackers = null;
    }

}
