package org.objectweb.proactive.extensions.structuredp2p.core.tracker;

import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.objectweb.proactive.ActiveObjectCreationException;
import org.objectweb.proactive.api.PAActiveObject;
import org.objectweb.proactive.api.PAFuture;
import org.objectweb.proactive.core.node.NodeException;
import org.objectweb.proactive.extensions.structuredp2p.api.PeerFactory;
import org.objectweb.proactive.extensions.structuredp2p.api.TrackerFactory;
import org.objectweb.proactive.extensions.structuredp2p.core.overlay.BasicCANOverlay;
import org.objectweb.proactive.extensions.structuredp2p.core.overlay.BasicChordOverlay;
import org.objectweb.proactive.extensions.structuredp2p.exceptions.StructuredP2PException;
import org.objectweb.proactive.extensions.structuredp2p.overlay.OverlayType;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.tracker.Tracker;
import org.objectweb.proactive.extensions.structuredp2p.tracker.TrackerCAN;

/**
 * Test cases for class {@link TrackerCAN}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: TestTrackerCAN.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class TestTrackerCAN {

    private static Tracker tracker;

    private static Peer peer;

    @BeforeClass
    public static void setUp() throws Exception {
        try {
            TestTrackerCAN.tracker = TrackerFactory.newActiveTracker(OverlayType.CAN);
            // Binds the tracker to a specific URL on the RMI registry
            PAActiveObject.registerByName(TestTrackerCAN.tracker, "TestCANTracker");
        } catch (ActiveObjectCreationException e) {
            e.printStackTrace();
        } catch (NodeException e) {
            e.printStackTrace();
        }
    }

    @Test(expected = StructuredP2PException.class)
    public void testAddOnNetworkWithWrongPeerType() throws ActiveObjectCreationException,
            NodeException, StructuredP2PException {
        TestTrackerCAN.tracker.addOnNetwork(PeerFactory.newActivePeer(new BasicChordOverlay()));
    }

    @Test
    public void testGetRandomPeerWithEmptyTracker() {
        Assert.assertEquals(null, PAFuture.getFutureValue(TestTrackerCAN.tracker.getRandomPeer()));
        Assert.assertEquals(0, TestTrackerCAN.tracker.getNbOfPeersStored());
    }

    @Test
    public void testAddOnNetworkWithCorrectPeerType() throws ActiveObjectCreationException,
            NodeException {
        TestTrackerCAN.peer = PeerFactory.newActivePeer(new BasicCANOverlay());
        try {
            TestTrackerCAN.tracker.addOnNetwork(peer);
        } catch (StructuredP2PException e) {
            e.printStackTrace();
        }
    }

    @Test
    public void testGetRandomPeer() {
        Assert.assertEquals(TestTrackerCAN.peer, PAFuture.getFutureValue(TestTrackerCAN.tracker
                .getRandomPeer()));
    }

    @AfterClass
    public static void tearDown() throws Exception {
        PAActiveObject.terminateActiveObject(TestTrackerCAN.peer, false);
        PAActiveObject.terminateActiveObject(TestTrackerCAN.tracker, false);
    }
    
}
