package org.objectweb.proactive.extensions.structuredp2p.core.tracker;

import java.util.List;

import junit.framework.Assert;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.objectweb.proactive.extensions.structuredp2p.tracker.TrackersProperties;

/**
 * @author Laurent Pellegrino
 * 
 * @version $Id: TestTrackerProperties.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class TestTrackerProperties {

    private TrackersProperties trackerProperties;

    @Before
    public void setUp() {
        this.trackerProperties = new TrackersProperties("trackers.properties");
    }

    @Test
    public void testPutAndGet() {
        this.trackerProperties.store("my-network",
                "rmi://localhost:1099/network2/6066763f-6ca6-49ad-949d-b4f87a45ab71",
                "CHORD");

        Assert.assertEquals(this.trackerProperties.getBindingNames("my-network").get(0),
                "rmi://localhost:1099/network2/6066763f-6ca6-49ad-949d-b4f87a45ab71");
    }

    @Test
    public void testDoubloon() {
        this.trackerProperties.store("my-network",
                "rmi://localhost:1099/network2/6066763f-6ca6-49ad-949d-b4f87a45ab71",
                "CHORD");
        this.trackerProperties.store("my-network",
                "rmi://localhost:1099/network2/6066763f-6ca6-49ad-949d-b4f87a45ab71",
                "CHORD");

        Assert.assertEquals(1, this.trackerProperties.getBindingNames("my-network").size());
    }

    @Test
    public void testRemove() {
        this.trackerProperties.store("my-network", "rmi://localhost:1099/network/tracker1",
                "CHORD");
        this.trackerProperties.store("my-network", "rmi://localhost:1099/network/tracker2",
                "CHORD");

        this.trackerProperties.remove("my-network", "rmi://localhost:1099/network/tracker1");

        List<String> bindingNames = this.trackerProperties.getBindingNames("my-network");
        Assert.assertEquals(1, bindingNames.size());
        Assert.assertEquals("rmi://localhost:1099/network/tracker2", this.trackerProperties
                .getBindingNames("my-network").get(0));
    }

    @Test(expected = NullPointerException.class)
    public void testGetRandomActiveAndValidTracker() {
        this.trackerProperties.store("my-network", "rmi://localhost:1099/network/tracker1",
                "CHORD");
        this.trackerProperties.store("my-network", "rmi://localhost:1099/network/tracker2",
                "CHORD");
        this.trackerProperties.getRandomActiveAndValidTracker("my-network");
        Assert.assertEquals(0, this.trackerProperties.getBindingNames("my-network"));
    }

    @After
    public void tearDown() {
        this.trackerProperties.getPropertiesFilePath().delete();
        this.trackerProperties = null;
    }

}
