package org.objectweb.proactive.extensions.structuredp2p.initializers;

import java.math.BigInteger;
import java.util.HashSet;
import java.util.Set;

import org.objectweb.proactive.ActiveObjectCreationException;
import org.objectweb.proactive.api.PAFuture;
import org.objectweb.proactive.core.node.NodeException;
import org.objectweb.proactive.extensions.structuredp2p.api.PeerFactory;
import org.objectweb.proactive.extensions.structuredp2p.core.overlay.BasicChordOverlay;
import org.objectweb.proactive.extensions.structuredp2p.operations.chord.GetKeyOperation;
import org.objectweb.proactive.extensions.structuredp2p.operations.chord.GetKeyResponseOperation;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.overlay.chord.Key;

/**
 * Specific network initializer for Chord network.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: ChordNetworkInitializer.java 3470 2010-02-24 10:09:53Z laurentp
 *          $
 */
public class ChordNetworkInitializer extends NetworkInitializer {

    private Set<Key> keysAlreadyAssigned = new HashSet<Key>();
    
    public Peer createPeer() throws ActiveObjectCreationException, NodeException {
        if (BigInteger.valueOf(
        	super.getAllPeers().size()).compareTo(Key.MAX_VALUE) == 0) {
            throw new AssertionError("The network contains the maximum number of peers allowed for a key size of " + Key.NB_BITS);
        }
        
        return this.createPeerWithoutNetworkSizeAssertion();
    }

    private Peer createPeerWithoutNetworkSizeAssertion() throws ActiveObjectCreationException, NodeException {
	Peer peer = PeerFactory.newActivePeer(new BasicChordOverlay());
	Key peerKey = ((GetKeyResponseOperation)PAFuture.getFutureValue(
			peer.send(new GetKeyOperation()))).getKey();

	/*
	 * Check if there is no other peer which is already on the network with
	 * the same identifier (the verification is local because this
	 * initializer is supposed to be used only for local tests).
	 * 
	 * It is not possible to perform the verification by sending messages
	 * because the network need to be stabilized to handle it.
	 */
        if (super.getAllPeers().size() > 0 && this.keysAlreadyAssigned.contains(peerKey)) {
            return this.createPeerWithoutNetworkSizeAssertion();
        } else {
            this.keysAlreadyAssigned.add(peerKey);
            return peer;
        }
    }
    
}
