package org.objectweb.proactive.extensions.structuredp2p.initializers;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.proactive.ActiveObjectCreationException;
import org.objectweb.proactive.api.PAFuture;
import org.objectweb.proactive.core.node.NodeException;
import org.objectweb.proactive.core.util.ProActiveRandom;
import org.objectweb.proactive.extensions.structuredp2p.api.TrackerFactory;
import org.objectweb.proactive.extensions.structuredp2p.overlay.OverlayType;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.tracker.Tracker;

/**
 * Abstract network initializer used in order to initialize a new structured p2p
 * network for tests.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: NetworkInitializer.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public abstract class NetworkInitializer {

    private List<Peer> peers = new ArrayList<Peer>();

    private Tracker tracker;

    private OverlayType type;

    protected abstract Peer createPeer() throws ActiveObjectCreationException, NodeException;

    public void initializeNewNetwork(OverlayType type, int nbPeersToCreate) throws Exception {
        this.tracker = TrackerFactory.newActiveTracker(type);
        this.type = type;
        Peer peerCreated;

        for (int i = 0; i < nbPeersToCreate; i++) {
            peerCreated = this.createPeer();
            PAFuture.waitFor(this.tracker.addOnNetwork(peerCreated));
            this.peers.add(peerCreated);
        }
    }

    public void clearNetwork() {
        // TODO uncomment when leave operation works!
        // for (Peer peer : this.peers) {
        // try {
        // PAFuture.waitFor(peer.leave());
        // } catch (StructuredP2PException e) {
        // e.printStackTrace();
        // }
        // }

        this.peers.clear();
    }

    public List<Peer> getAllPeers() {
        return this.peers;
    }

    public Peer getARandomPeer() {
        return this.peers.get(ProActiveRandom.nextInt(peers.size()));
    }

    public Peer get(int index) {
        if (index < 0 || index >= this.peers.size()) {
            throw new IllegalArgumentException();
        }
        
        return this.peers.get(index);
    }

    public OverlayType getType() {
        return this.type;
    }

}
