package org.objectweb.proactive.extensions.structuredp2p.messages.can;

import junit.framework.Assert;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.objectweb.proactive.api.PAFuture;
import org.objectweb.proactive.extensions.structuredp2p.core.overlay.BasicCANOverlay;
import org.objectweb.proactive.extensions.structuredp2p.initializers.CANNetworkInitializer;
import org.objectweb.proactive.extensions.structuredp2p.messages.query.can.LookupQueryMessage;
import org.objectweb.proactive.extensions.structuredp2p.messages.response.can.LookupResponseMessage;
import org.objectweb.proactive.extensions.structuredp2p.overlay.OverlayType;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;

/**
 * Test case for {@link LookupQueryMessage}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: TestLookupQueryMessage.java 3127 2010-01-06 15:59:42Z laurentp
 *          $
 */
public class TestLookupQueryMessage extends CANNetworkInitializer {

    private LookupQueryMessage query;

    private Peer remotePeerToReach;
    
    @Before
    public void setUp() throws Exception {
        super.initializeNewNetwork(OverlayType.CAN, 5);
        this.remotePeerToReach = super.getARandomPeer();
        this.query = new LookupQueryMessage(((BasicCANOverlay) PAFuture.getFutureValue(
                this.remotePeerToReach.getStructuredOverlay())).getZone().getCoordinateMin());
    }

    @Test
    public void testLookupQuery() {
        LookupResponseMessage response = null;
        try {
            response = (LookupResponseMessage) PAFuture.getFutureValue(super.get(0).send(this.query));
        } catch (Exception e) {
            e.printStackTrace();
        }

        Assert.assertTrue(response.getLatency() > 1);
        // the peer to reach can be the initiator of the request
        Assert.assertTrue(response.getHopCount() >= 0);
        Assert.assertTrue(response.getReturnHopCount() >= 0);
        Assert.assertTrue(response.getOutboundHopCount() >= 0);
         
        Assert.assertEquals(this.remotePeerToReach, response.getPeerFound());
    }

    @After
    public void tearDown() {
        // TODO uncomment when CAN leave works
        // super.clearNetwork();
    }
}
