package org.objectweb.proactive.extensions.structuredp2p.messages.chord;

import java.io.Serializable;

import junit.framework.Assert;

import org.junit.Before;
import org.junit.Test;
import org.objectweb.proactive.api.PAFuture;
import org.objectweb.proactive.extensions.structuredp2p.initializers.ChordNetworkInitializer;
import org.objectweb.proactive.extensions.structuredp2p.messages.query.chord.TurnAroundQueryMessage;
import org.objectweb.proactive.extensions.structuredp2p.messages.response.chord.TurnAroundResponseMessage;
import org.objectweb.proactive.extensions.structuredp2p.overlay.OverlayType;
import org.objectweb.proactive.extensions.structuredp2p.overlay.StructuredOverlay;

/**
 * @author Laurent Pellegrino
 * 
 * @version $Id: TestTurnAroundQueryMessage.java 3576 2010-03-04 15:20:20Z
 *          laurentp $
 */
public class TestTurnAroundQueryMessage extends ChordNetworkInitializer implements Serializable {

    private static final long serialVersionUID = 1L;
    
    private static final int NB_NODES_TO_CREATE = 5;

    @Before
    public void testCreation() {
        try {
            super.initializeNewNetwork(OverlayType.CHORD, NB_NODES_TO_CREATE);
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            Thread.sleep(5000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    @Test(timeout = 10000)
    @SuppressWarnings("unchecked")
    public void testTurnAroundQueryMessage() {
        TurnAroundResponseMessage<String> response = 
            ((TurnAroundResponseMessage<String>) PAFuture.getFutureValue(
        	    super.getARandomPeer().send(
        		    new MyTurnAroundQueryMessage())));
        Assert.assertEquals(NB_NODES_TO_CREATE, response.getOutboundHopCount());

        for (String s : response.getValues()) {
            Assert.assertEquals("Hello World!", s);
        }

        Assert.assertEquals(NB_NODES_TO_CREATE, response.getValues().size());
    }

    private class MyTurnAroundQueryMessage extends TurnAroundQueryMessage<String> {
        private static final long serialVersionUID = 1L;

        public void updateValues(StructuredOverlay overlay) {
            super.getValues().add("Hello World!");
        }
    }

}
