package org.objectweb.proactive.extensions.structuredp2p.operations.can;

import junit.framework.Assert;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.objectweb.proactive.api.PAFuture;
import org.objectweb.proactive.extensions.structuredp2p.core.overlay.BasicCANOverlay;
import org.objectweb.proactive.extensions.structuredp2p.initializers.CANNetworkInitializer;
import org.objectweb.proactive.extensions.structuredp2p.operations.can.AddAndUpdateNeighborOperation;
import org.objectweb.proactive.extensions.structuredp2p.operations.can.RemoveNeighborOperation;
import org.objectweb.proactive.extensions.structuredp2p.overlay.OverlayType;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.NeighborEntry;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.NeighborTable;

/**
 * Test cases for {@link AddAndUpdateNeighborOperation}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: TestAddNeighborMessage.java 3127 2010-01-06 15:59:42Z laurentp$
 */
public class TestAddAndUpdateNeighborOperation extends CANNetworkInitializer {

    @Before
    public void setUp() throws Exception {
        super.initializeNewNetwork(OverlayType.CAN, 2);
    }

    @Test
    public void testHandle() {
        Peer firstPeer = super.getARandomPeer();
        Peer secondPeer = super.getARandomPeer();

        NeighborEntry secondPeerEntry = new NeighborEntry(secondPeer);

        PAFuture.waitFor(firstPeer.send(
                new RemoveNeighborOperation(secondPeerEntry.getIdentifier())));

        boolean contains = ((BasicCANOverlay) PAFuture.getFutureValue(
                firstPeer.getStructuredOverlay())).hasNeighbor(secondPeerEntry.getIdentifier());
        Assert.assertTrue(!contains);

        PAFuture.waitFor(firstPeer.send(
                new AddAndUpdateNeighborOperation(
                        secondPeerEntry, secondPeerEntry, 0, NeighborTable.INFERIOR_DIRECTION)));
        Assert.assertTrue(((BasicCANOverlay) PAFuture.getFutureValue(
                firstPeer.getStructuredOverlay())).hasNeighbor(secondPeerEntry.getIdentifier()));
    }

    @After
    public void tearDown() {
         super.clearNetwork();
    }

}
