package org.objectweb.proactive.extensions.structuredp2p.operations.can;

import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.objectweb.proactive.api.PAActiveObject;
import org.objectweb.proactive.api.PAFuture;
import org.objectweb.proactive.extensions.structuredp2p.api.PeerFactory;
import org.objectweb.proactive.extensions.structuredp2p.core.overlay.BasicCANOverlay;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;
import org.objectweb.proactive.extensions.structuredp2p.overlay.can.AbstractCANOverlay;

/**
 * Test cases for the Join operation in {@link AbstractCANOverlay}.
 * 
 * @author Alexandre Trovato
 * @author Fanny Kilanga
 * @author Laurent Pellegrino
 * 
 * @version $Id: TestJoinOperation.java 5022 2010-08-14 16:14:45Z plaurent $
 */
public class TestJoinOperation {

    private static Peer firstPeer;

    private static Peer secondPeer;

    private static Peer thirdPeer;

    private static Peer fourthPeer;

    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
        TestJoinOperation.firstPeer = PeerFactory.newActivePeer(new BasicCANOverlay());
        TestJoinOperation.secondPeer = PeerFactory.newActivePeer(new BasicCANOverlay());
        TestJoinOperation.thirdPeer = PeerFactory.newActivePeer(new BasicCANOverlay());
        TestJoinOperation.fourthPeer = PeerFactory.newActivePeer(new BasicCANOverlay());
    }

    @Test
    public void testSecondPeer() {
        try {
            Assert.assertTrue(TestJoinOperation.secondPeer.join(TestJoinOperation.firstPeer));
        } catch (Exception e) {
            e.printStackTrace();
        }

        // Check that the peers are neighbors
        Assert.assertTrue(TestJoinOperation.getOverlay(TestJoinOperation.firstPeer).hasNeighbor(
                TestJoinOperation.secondPeer.getIdentifier()));
        Assert.assertTrue(TestJoinOperation.getOverlay(TestJoinOperation.secondPeer).hasNeighbor(
                TestJoinOperation.firstPeer.getIdentifier()));

        // Check if a peer has itself in it neighbors list
        Assert.assertFalse(TestJoinOperation.getOverlay(TestJoinOperation.firstPeer).hasNeighbor(
                TestJoinOperation.firstPeer.getIdentifier()));
        Assert.assertFalse(TestJoinOperation.getOverlay(TestJoinOperation.secondPeer).hasNeighbor(
                TestJoinOperation.secondPeer.getIdentifier()));

        // Check the zone of the peers are bordered
        Assert.assertNotSame(TestJoinOperation.getOverlay(TestJoinOperation.firstPeer).getZone()
                .getAbutsDimensionWith(
                        TestJoinOperation.getOverlay(TestJoinOperation.secondPeer).getZone()), -1);
    }

    @Test
    public void testThirdPeer() {
        try {
            Assert.assertTrue(TestJoinOperation.thirdPeer.join(TestJoinOperation.secondPeer));
        } catch (Exception e) {
            e.printStackTrace();
        }

        // Check that the peers are neighbors
        Assert.assertTrue(TestJoinOperation.getOverlay(TestJoinOperation.firstPeer).hasNeighbor(
                TestJoinOperation.thirdPeer.getIdentifier()));
        Assert.assertTrue(TestJoinOperation.getOverlay(TestJoinOperation.secondPeer).hasNeighbor(
                TestJoinOperation.thirdPeer.getIdentifier()));
        Assert.assertTrue(TestJoinOperation.getOverlay(TestJoinOperation.thirdPeer).hasNeighbor(
                TestJoinOperation.firstPeer.getIdentifier()));
        Assert.assertTrue(TestJoinOperation.getOverlay(TestJoinOperation.thirdPeer).hasNeighbor(
                TestJoinOperation.secondPeer.getIdentifier()));

        // Check if a peer has itself in it neighbors list
        Assert.assertFalse(TestJoinOperation.getOverlay(TestJoinOperation.thirdPeer).hasNeighbor(
                TestJoinOperation.thirdPeer.getIdentifier()));

        // Check the zone of the peers are bordered
        Assert.assertNotSame(TestJoinOperation.getOverlay(TestJoinOperation.firstPeer).getZone()
                .getAbutsDimensionWith(
                        TestJoinOperation.getOverlay(TestJoinOperation.thirdPeer).getZone()), -1);
        Assert.assertNotSame(TestJoinOperation.getOverlay(TestJoinOperation.secondPeer).getZone()
                .getAbutsDimensionWith(
                        TestJoinOperation.getOverlay(TestJoinOperation.thirdPeer).getZone()), -1);
    }

    @Test
    public void testFourthPeer() {
        try {
            Assert.assertTrue(TestJoinOperation.fourthPeer.join(TestJoinOperation.thirdPeer));
        } catch (Exception e) {
            e.printStackTrace();
        }

        // Check that the peers are neighbors
        Assert.assertTrue(TestJoinOperation.getOverlay(TestJoinOperation.secondPeer).hasNeighbor(
                TestJoinOperation.fourthPeer.getIdentifier()));
        Assert.assertTrue(TestJoinOperation.getOverlay(TestJoinOperation.thirdPeer).hasNeighbor(
                TestJoinOperation.fourthPeer.getIdentifier()));
        Assert.assertTrue(TestJoinOperation.getOverlay(TestJoinOperation.fourthPeer).hasNeighbor(
                TestJoinOperation.secondPeer.getIdentifier()));
        Assert.assertTrue(TestJoinOperation.getOverlay(TestJoinOperation.fourthPeer).hasNeighbor(
                TestJoinOperation.thirdPeer.getIdentifier()));

        // Check if a peer has itself in it neighbors list
        Assert.assertFalse(TestJoinOperation.getOverlay(TestJoinOperation.fourthPeer).hasNeighbor(
                TestJoinOperation.fourthPeer.getIdentifier()));

        // Check the zone of the peers are bordered
        Assert.assertNotSame(TestJoinOperation.getOverlay(TestJoinOperation.secondPeer).getZone()
                .getAbutsDimensionWith(
                        TestJoinOperation.getOverlay(TestJoinOperation.thirdPeer).getZone()), -1);

        if (TestJoinOperation.getOverlay(TestJoinOperation.firstPeer).getZone().getAbutsDimensionWith(
                TestJoinOperation.getOverlay(TestJoinOperation.thirdPeer).getZone()) != -1) {
            Assert.assertTrue(TestJoinOperation.getOverlay(TestJoinOperation.firstPeer).hasNeighbor(
                    TestJoinOperation.thirdPeer.getIdentifier()));
            Assert.assertTrue(TestJoinOperation.getOverlay(TestJoinOperation.thirdPeer).hasNeighbor(
                    TestJoinOperation.firstPeer.getIdentifier()));
        } else {
            Assert.assertFalse(TestJoinOperation.getOverlay(TestJoinOperation.firstPeer).hasNeighbor(
                    TestJoinOperation.thirdPeer.getIdentifier()));
            Assert.assertFalse(TestJoinOperation.getOverlay(TestJoinOperation.thirdPeer).hasNeighbor(
                    TestJoinOperation.firstPeer.getIdentifier()));
        }

        if (TestJoinOperation.getOverlay(TestJoinOperation.firstPeer).getZone().getAbutsDimensionWith(
                TestJoinOperation.getOverlay(TestJoinOperation.fourthPeer).getZone()) != -1) {
            Assert.assertTrue(TestJoinOperation.getOverlay(TestJoinOperation.firstPeer).hasNeighbor(
                    TestJoinOperation.fourthPeer.getIdentifier()));
            Assert.assertTrue(TestJoinOperation.getOverlay(TestJoinOperation.fourthPeer).hasNeighbor(
                    TestJoinOperation.firstPeer.getIdentifier()));
        } else {
            Assert.assertFalse(TestJoinOperation.getOverlay(TestJoinOperation.firstPeer).hasNeighbor(
                    TestJoinOperation.fourthPeer.getIdentifier()));
            Assert.assertFalse(TestJoinOperation.getOverlay(TestJoinOperation.fourthPeer).hasNeighbor(
                    TestJoinOperation.firstPeer.getIdentifier()));
        }

    }

    @AfterClass
    public static void tearDownAfterClass() throws Exception {
        PAActiveObject.terminateActiveObject(firstPeer, false);
        PAActiveObject.terminateActiveObject(secondPeer, false);
        PAActiveObject.terminateActiveObject(thirdPeer, false);
        PAActiveObject.terminateActiveObject(fourthPeer, false);
    }

    public static BasicCANOverlay getOverlay(Peer peer) {
        return ((BasicCANOverlay) PAFuture.getFutureValue(peer.getStructuredOverlay()));
    }
}
