package org.objectweb.proactive.extensions.structuredp2p.operations.can;

import java.util.UUID;

import junit.framework.Assert;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.objectweb.proactive.api.PAFuture;
import org.objectweb.proactive.extensions.structuredp2p.initializers.CANNetworkInitializer;
import org.objectweb.proactive.extensions.structuredp2p.operations.BooleanResponseOperation;
import org.objectweb.proactive.extensions.structuredp2p.operations.can.HasNeighborOperation;
import org.objectweb.proactive.extensions.structuredp2p.operations.can.RemoveNeighborOperation;
import org.objectweb.proactive.extensions.structuredp2p.overlay.OverlayType;
import org.objectweb.proactive.extensions.structuredp2p.overlay.Peer;

/**
 * Test cases for {@link RemoveNeighborOperation}.
 * 
 * @author Laurent Pellegrino
 * 
 * @version $Id: TestRemoveNeighborOperation.java 3127 2010-01-06 15:59:42Z
 *          laurentp $
 */
public class TestRemoveNeighborOperation extends CANNetworkInitializer {

    @Before
    public void setUp() throws Exception {
        super.initializeNewNetwork(OverlayType.CAN, 2);
    }

    @Test
    public void testMessage() {
        Peer firstPeer = super.get(0);
        Peer secondPeer = super.get(1);

        UUID firstPeerID = firstPeer.getIdentifier();
        UUID secondPeerID = secondPeer.getIdentifier();

        Assert.assertTrue(((BooleanResponseOperation) PAFuture.getFutureValue(firstPeer
                .send(new HasNeighborOperation(secondPeerID)))).getValue());
        Assert.assertTrue(((BooleanResponseOperation) PAFuture.getFutureValue(secondPeer
                .send(new HasNeighborOperation(firstPeerID)))).getValue());

        PAFuture.waitFor(firstPeer.send(new RemoveNeighborOperation(secondPeerID)));

        Assert.assertFalse(((BooleanResponseOperation) PAFuture.getFutureValue(firstPeer
                .send(new HasNeighborOperation(secondPeerID)))).getValue());
        Assert.assertTrue(((BooleanResponseOperation) PAFuture.getFutureValue(secondPeer
                .send(new HasNeighborOperation(firstPeerID)))).getValue());
    }

    @After
    public void tearDown() {
        super.clearNetwork();
    }

}
