package com.ontotext.trree.rdf2go;

import java.util.HashMap;
import java.util.UUID;

import org.ontoware.rdf2go.model.Model;
import org.ontoware.rdf2go.model.Statement;
import org.ontoware.rdf2go.model.impl.StatementImpl;
import org.ontoware.rdf2go.model.node.Resource;
import org.ontoware.rdf2go.model.node.URI;
import org.openrdf.rdf2go.ConversionUtil;
import org.openrdf.rdf2go.RepositoryModel;
import org.openrdf.repository.Repository;

import com.ontotext.trree.AbstractRepository;
import com.ontotext.trree.EntityPool;
import com.ontotext.trree.OwlimSchemaRepository;
import com.ontotext.trree.SPARQLQueryListener;
import com.ontotext.trree.rdf2go.api.SubscribableModel;
import com.ontotext.trree.rdf2go.api.Subscriber;


public class OwlimModel extends RepositoryModel implements SubscribableModel {

	private static final long serialVersionUID = 1L;
	
	private HashMap<URI,SPARQLQueryListener> subscribers = 
		new HashMap<URI,SPARQLQueryListener>();
	

	public OwlimModel(URI contextURI, Repository repository) {
		super(contextURI, repository);
	}

	public Model open() {
		getOwlimRepository().open();
		return super.open();
	}
	
	public void close() {
		super.close();
		getOwlimRepository().close();
	}
	
	public URI subscribe(String query, final Subscriber subscriber) {
		OwlimSchemaRepository owlimSail = (OwlimSchemaRepository) getOwlimRepository().getSail();
		
		AbstractRepository rep = owlimSail.getRepository();
		final EntityPool pool = owlimSail.getEntities();
		
		int ctx = 0;
		if (getContextURI() != null) {
			org.openrdf.model.URI openRDFURI = 
				ConversionUtil.toOpenRDF(getContextURI(), repository.getValueFactory());
			if (openRDFURI != null)
				ctx = pool.getId(openRDFURI);
		}
		
		final URI identifierURI = 
			createURI("http://www.ontotext.com/subscribe/" + UUID.randomUUID());
		
		SPARQLQueryListener listener = new SPARQLQueryListener(query, rep, pool, ctx) {
			public void notifyMatch(int subj, int pred, int obj, int context) {
				Statement matchingStatement = new StatementImpl(
					(URI) 		ConversionUtil.toRdf2go(pool.toObject(context)),
					(Resource) 	ConversionUtil.toRdf2go(pool.toObject(subj)),
					(URI) 		ConversionUtil.toRdf2go(pool.toObject(pred)),
								ConversionUtil.toRdf2go(pool.toObject(obj))
				);
				subscriber.matchingStatement(identifierURI, matchingStatement);
			}
		};
		rep.addListener(listener);
		
		subscribers.put(identifierURI, listener);
		
		return identifierURI;
	}
	
	public void unsubscribe(URI subscriberURI) {
		SPARQLQueryListener subscriber = subscribers.get(subscriberURI);
		if (subscriber == null) return;
		
		OwlimSchemaRepository owlimSail = (OwlimSchemaRepository) getOwlimRepository().getSail();
		owlimSail.getRepository().removeListener(subscriber);
		
		subscribers.remove(subscriberURI);
	}
	
	private OwlimRepository getOwlimRepository() {
		return (OwlimRepository) getUnderlyingModelImplementation();
	}
}
