package com.ontotext.trree.rdf2go;

import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.ontoware.aifbcommons.collection.ClosableIterator;
import org.ontoware.rdf2go.RDF2Go;
import org.ontoware.rdf2go.model.Model;
import org.ontoware.rdf2go.model.ModelSet;
import org.ontoware.rdf2go.model.node.URI;
import org.openrdf.rdf2go.RepositoryModelSet;
import org.openrdf.repository.Repository;

public class OwlimModelSet extends RepositoryModelSet {

	private static final long serialVersionUID = 1L;
	
	private Map<URI,WeakReference<Model>> openModels = 
		new HashMap<URI,WeakReference<Model>>();
	
	private boolean autoCommit = true;
	
	public OwlimModelSet(Repository repository) {
		super(repository);
	}

	public Model getDefaultModel() {
		return openModel(null);
	}
	
	public Model getModel(URI contextURI) {
		return openModel(contextURI);
	}
	
	public ClosableIterator<Model> getModels() {
		final ClosableIterator<URI> contexts = getModelURIs();
		return new ClosableIterator<Model>() {
			public void close() {
				contexts.close();
			}
			public boolean hasNext() {
				return contexts.hasNext();
			}
			public Model next() {
				URI nextURI = contexts.next();
				return openModel(nextURI);
			}
			public void remove() {
				throw new UnsupportedOperationException();
			}
		};
	}

	public void setAutocommit(boolean autoCommit) {
		this.autoCommit = autoCommit;
		super.setAutocommit(autoCommit);
		for (WeakReference<Model> modelRef : openModels.values()) {
			Model model = modelRef.get();
			if (model != null)
				model.setAutocommit(autoCommit);
		}
	}
	
	private Model openModel(URI contextURI) {
		if (openModels.containsKey(contextURI)) {
			WeakReference<Model> modelRef = openModels.get(contextURI);
			if (modelRef != null && modelRef.get() != null)
				return modelRef.get();
		}
		OwlimRepository repository = getOwlimRepository().clone();
		Model model = new OwlimModel(contextURI, repository);
		model.open();
		model.setAutocommit(autoCommit);
		openModels.put(contextURI, new WeakReference<Model>(model));
		return model;
	}
	
	private OwlimRepository getOwlimRepository() {
		return (OwlimRepository) getUnderlyingModelImplementation();
	}
	
	public void open() {
		getOwlimRepository().open();
		super.open();
	}
	
	public void close() {
		super.close();
		for (WeakReference<Model> modelRef : openModels.values()) {
			Model model = modelRef.get();
			if (model != null)
				model.close();
		}
		openModels.clear();
		getOwlimRepository().close();
	}
	
	public static void main(String[] args) {
		Properties props = new Properties();
		props.setProperty(OwlimModelFactory.REASONING, "OWL");

		RDF2Go.register("com.ontotext.trree.rdf2go.OwlimModelFactory");

		ModelSet mRoot = RDF2Go.getModelFactory().createModelSet(props);
		mRoot.open();

		Model m = mRoot.getModel(new org.ontoware.rdf2go.model.node.impl.URIImpl(
				"http://tutu"));
		m.open();
		m.addStatement(m.createURI("http://example.com/a"), 
				m.createURI("http://example.com/b"), 
				m.createURI("http://example.com/c"));

		mRoot.setAutocommit(true);
		
		m.addStatement(m.createURI("http://example.com/a1"), 
				m.createURI("http://example.com/b1"), 
				m.createURI("http://example.com/c1"));

		m.close();
		
		ClosableIterator<Model> models = mRoot.getModels();
		while (models.hasNext()) {
			OwlimModel cm = (OwlimModel) models.next();
			System.out.println(cm.getContextURI());
		}
		
		mRoot.close();
	}
}
