package com.ontotext.trree.rdf2go;

import java.io.File;

import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.sail.SailRepository;
import org.openrdf.sail.Sail;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.ontotext.trree.OwlimSchemaRepository;

public class OwlimRepository extends SailRepository {
	private static final transient Logger logger = LoggerFactory
			.getLogger(OwlimRepository.class);
	
	private boolean isOwner, isOpen, isAutoRemove;
	
	public OwlimRepository(Sail sail) {
		this(sail, true);
	}
	
	public OwlimRepository(Sail sail, boolean owner) {
		super(sail);
		isOwner = owner;
	}
	
	public void open() {
		if (isOwner && !isOpen) {
			try {
				initialize();
				isOpen = true;
			} catch (RepositoryException e) {
				e.printStackTrace();
			}
		}
	}
	
	public void close() {
		if (isOwner && isOpen) {
			try {
				isOpen = false;
				shutDown();
				if (isAutoRemove) {
					OwlimSchemaRepository owlimSail = (OwlimSchemaRepository)getSail();
					File storageDir = new File(owlimSail.getStorageFolder());
					
					// Wait repository has finished to shutdown
					// before to remove it
					while (owlimSail.isShuttingDown()) {
						Thread.yield();
					}
					
					removeDir(storageDir);
					if (logger.isInfoEnabled()) {
						logger.info("Repository '{}' has been shutdown and removed", storageDir);
					}
				} else {
					if (logger.isDebugEnabled()) {
						logger.info("Repository '{}' has been shutdown", 
								((OwlimSchemaRepository)getSail()).getStorageFolder());
					}
				}
			} catch (RepositoryException e) {
				e.printStackTrace();
			}
		}
	}
	
	private void removeDir(File dir) {
		for (File entry : dir.listFiles()) {
			if (entry.isDirectory()) {
				removeDir(entry);
			} else {
				entry.delete();
			}
		}
		dir.delete();
	}
	
	public void setAutoRemove(boolean value) {
		isAutoRemove = value;
	}
	
	public OwlimRepository clone() {
		return new OwlimRepository(getSail(), false);
	}
}
