package com.ontotext.trree.rdf2go;

import java.io.File;
import java.util.Properties;
import java.util.Random;

import junit.framework.Assert;

import org.junit.Test;
import org.ontoware.aifbcommons.collection.ClosableIterable;
import org.ontoware.aifbcommons.collection.ClosableIterator;
import org.ontoware.rdf2go.ModelFactory;
import org.ontoware.rdf2go.RDF2Go;
import org.ontoware.rdf2go.model.ModelSet;
import org.ontoware.rdf2go.model.Statement;
import org.ontoware.rdf2go.model.impl.StatementImpl;
import org.ontoware.rdf2go.model.node.URI;
import org.ontoware.rdf2go.model.node.impl.URIImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class TestOwlimAdapter {

	private static final transient Logger logger = LoggerFactory
			.getLogger(TestOwlimAdapter.class);
	
	private static final String DATASTORE_PATH = "/tmp/owlim";

	private static final int NUMBER_OF_STATEMENTS_TO_ADD = 1000;

	private static final URI DEFAULT_SPACE_URI = new URIImpl(
			"http://www.ontotext.com");

	@Test
	public void testAddStatements() {
		final OwlimAdapterDS datastore = new OwlimAdapterDS(new File(
				DATASTORE_PATH), false, true);

		datastore.clear(DEFAULT_SPACE_URI);
		
		Statement[] stmts = new Statement[NUMBER_OF_STATEMENTS_TO_ADD];
		for (int i = 0; i < NUMBER_OF_STATEMENTS_TO_ADD; i++) {
			stmts[i] = generateRandomStatement(DEFAULT_SPACE_URI);
		}

		long start = System.currentTimeMillis();
		for (int i = 0; i < NUMBER_OF_STATEMENTS_TO_ADD; i++) {
			datastore.addStatement(DEFAULT_SPACE_URI, stmts[i]);
		}
		datastore.getModelSet().getModel(DEFAULT_SPACE_URI).commit();
		
		if (logger.isInfoEnabled()) {
			logger.info((System.currentTimeMillis() - start) + " ms to add "
					+ NUMBER_OF_STATEMENTS_TO_ADD + " statements");
		}

		ClosableIterable<Statement> res = datastore.getModelSet().getModel(
				DEFAULT_SPACE_URI).sparqlConstruct(
				"CONSTRUCT { ?s ?p ?o } WHERE { ?s ?p ?o }");

		ClosableIterator<Statement> it = res.iterator();
		int i = 0;
		while (it.hasNext()) {
			it.next();
			i++;
		}
		Assert.assertEquals(NUMBER_OF_STATEMENTS_TO_ADD, i);
		datastore.getModelSet().close();
	}

	public static Statement generateRandomStatement(URI spaceURI) {
		return new StatementImpl(spaceURI, new URIImpl(generateRandomURL()),
				new URIImpl(generateRandomURL()), new URIImpl(
						generateRandomURL()));
	}

	public static String generateRandomURL() {
		StringBuffer buf = new StringBuffer("http://www.");
		Random rand = new Random();
		for (int i = 0; i < (3 + rand.nextInt(10)); i++) {
			buf.append((char) ((int) 'a' + rand.nextInt(25)));
		}
		buf.append(".com");
		return buf.toString();
	}
}

class OwlimAdapterDS {

	private ModelSet rootModel;

	public OwlimAdapterDS(File repositoryPath, boolean autoCommit,
			boolean autoRemove) {
		Properties props = new Properties();
		props.setProperty("auto-remove", autoRemove ? "true" : "false");
		props.setProperty(ModelFactory.STORAGE, repositoryPath
				.getAbsolutePath());

		RDF2Go.register("com.ontotext.trree.rdf2go.OwlimModelFactory");

		this.rootModel = RDF2Go.getModelFactory().createModelSet(props);
		this.rootModel.open();
		this.rootModel.setAutocommit(autoCommit);
	}

	public void addStatement(URI space, Statement statement) {
		this.rootModel.getModel(space).addStatement(statement);
	}

	public void clear(URI space) {
		this.rootModel.getModel(space).removeAll();
		this.rootModel.getModel(space).commit();
	}
	
	public ModelSet getModelSet() {
		return this.rootModel;
	}

}
