/*
 * Copyright (C) 2010

 * STI Innsbruck
 * ICT - Technologie Park Innsbruck, 2nd Floor
 * Technikerstrasse 21a
 * A-6020 Innsbruck, Austria 
 * 
 * Contact:
 * reto.krummenacher@sti2.at
 * gerald.schrempf@sti2.at (development)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 */

package at.sti2.semanticspaces.api;

import java.util.Set;

import org.ontoware.rdf2go.model.node.URI;

import at.sti2.semanticspaces.api.enums.SpaceModelType;
import at.sti2.semanticspaces.api.enums.SpaceRelation;
import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;

/**
 * 
* API for using the Semantic Spaces. {@link SemanticSpaceImpl}
* 
* @author Gerald Schrempf
* 
*/

public interface ISemanticSpace {
    // ***** SPACE CREATION *****//
	public ISpaceModel createSpace(URI space, SpaceModelType type) throws SemanticSpaceException;
    public ISpaceModel createSpace(URI space, SpaceModelType type, URI parent) throws SemanticSpaceException;
    
    //***** GET SPACEMODEL *****//
    public ISpaceModel getSpace(URI space) throws SemanticSpaceException;
    public Set<ISpaceModel> getSpaces(Set<URI> spaces) throws SemanticSpaceException;
    public Set<ISpaceModel> getSubspaces(URI space) throws SemanticSpaceException;
    
    //***** LIST SPACES *****//
    public Set<URI> listSpaces();
    public Set<URI> listFederations();
    
    //***** SPACE RELATIONS *****//
    public URI createFederation(Set<ISpaceModel> spaces) throws SemanticSpaceException;
    public void deleteFederation(URI fed) throws SemanticSpaceException;
    public IFederationModel getFederation(URI fedURI);
    
    public void createSpaceRelation(ISpaceModel sSpace, SpaceRelation relation, ISpaceModel oSpace) throws SemanticSpaceException;
    public void removeSpaceRelation(ISpaceModel sSpace, SpaceRelation relation, ISpaceModel oSpace) throws SemanticSpaceException;
    
    //***** SHUTDOWN, PERSISTANCY *****//
    public void shutdown() throws SemanticSpaceException;
}
