/*
 * Copyright (C) 2010
 * 
 * STI Innsbruck ICT - Technologie Park Innsbruck, 2nd Floor Technikerstrasse
 * 21a A-6020 Innsbruck, Austria
 * 
 * Contact: reto.krummenacher@sti2.at gerald.schrempf@sti2.at (development)
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
package at.sti2.semanticspaces.api.enums;

import org.ontoware.rdf2go.model.node.URI;
import org.ontoware.rdf2go.model.node.impl.URIImpl;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import at.sti2.semanticspaces.api.spaceOntology.SpaceOntology;

/**
 * defines the spaceModelTypes. its possible to call getURI() for each enum.
 * 
 * @author Gerald Schrempf
 *
 */
public enum SpaceModelType {
    LocalRepository, 
    DistributedSpace, 
    LOD;
    
    private URI uri;
    
    /**
     * creates an URI for each SpaceModelType.
     * @param name
     */
    SpaceModelType(){
        uri = new URIImpl(SpaceOntology.spaceModelTypes.toString() + this.toString());
    }
    
    public URI getURI(){
        return uri;
    }
    
    /**
     * converts from typeURI to type. (from metadataspace)
     * @param typeURI
     * @throws SemanticSpaceException 
     */
    public static SpaceModelType getSpaceType(URI typeURI) throws SemanticSpaceException {
        String typeStr = typeURI.toString().substring(SpaceOntology.spaceModelTypes.toString().length());
        SpaceModelType type;
        try {
            type = SpaceModelType.valueOf(typeStr);
        } catch (Exception e) {
            throw new SemanticSpaceException("SpaceModelType '" + typeStr + "' was found in URI '" + typeURI + "', but is not a valid enum.");
        }
        return type;
    }
}
