/*
 * Copyright (C) 2010
 * 
 * STI Innsbruck ICT - Technologie Park Innsbruck, 2nd Floor Technikerstrasse
 * 21a A-6020 Innsbruck, Austria
 * 
 * Contact: reto.krummenacher@sti2.at gerald.schrempf@sti2.at (development)
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

package at.sti2.semanticspaces.impl;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;

/**
 * reads a properties-file for configuration of the semantic space kernel (@see
 * SemanticSpaceImpl). Searches the file on the classpath and afterwards on the
 * absolute path.
 * 
 * @author Gerald Schrempf
 * 
 */
public class PropertyReader {
	/**
	 * reads properties from the configFile.
	 * 
	 * @param configFile
	 * @return
	 * @throws SemanticSpaceException
	 */
	public static Properties getProperties(String configFile)
			throws SemanticSpaceException {
		Properties props = new Properties();

		BufferedInputStream stream;
		try {
			InputStream inputStream = PropertyReader.class.getClassLoader()
					.getResourceAsStream(configFile);

			if (inputStream != null) { // search on classpath
				stream = new BufferedInputStream(inputStream);
				props.load(stream);
				stream.close();
			} else {
				props.load(new FileInputStream(configFile)); // absolute path
			}
		} catch (IOException e) {
			throw new SemanticSpaceException(
					"properties-file cannot be read/found: " + configFile, e);
		}
		return props;
	}
}
