/*
 * Copyright (C) 2010
 * 
 * STI Innsbruck ICT - Technologie Park Innsbruck, 2nd Floor Technikerstrasse
 * 21a A-6020 Innsbruck, Austria
 * 
 * Contact: reto.krummenacher@sti2.at gerald.schrempf@sti2.at (development)
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
package at.sti2.semanticspaces.impl.model;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.io.Writer;
import java.util.Map;
import java.util.Set;

import org.ontoware.aifbcommons.collection.ClosableIterable;
import org.ontoware.aifbcommons.collection.ClosableIterator;
import org.ontoware.rdf2go.exception.LockException;
import org.ontoware.rdf2go.exception.MalformedQueryException;
import org.ontoware.rdf2go.exception.ModelRuntimeException;
import org.ontoware.rdf2go.model.Model;
import org.ontoware.rdf2go.model.QueryResultTable;
import org.ontoware.rdf2go.model.Statement;
import org.ontoware.rdf2go.model.Syntax;
import org.ontoware.rdf2go.model.impl.AbstractModel;
import org.ontoware.rdf2go.model.node.BlankNode;
import org.ontoware.rdf2go.model.node.Node;
import org.ontoware.rdf2go.model.node.NodeOrVariable;
import org.ontoware.rdf2go.model.node.Resource;
import org.ontoware.rdf2go.model.node.ResourceOrVariable;
import org.ontoware.rdf2go.model.node.URI;
import org.ontoware.rdf2go.model.node.UriOrVariable;

import at.sti2.semanticspaces.api.IFederationModel;
import at.sti2.semanticspaces.api.ISpaceModel;

/**
 * A Model for representing a federation. Only possible operations are: dump,
 * getContextURI, findStatements, sparqlConstruct, sparqlDescribe, sparqlSelect.
 * 
 * Others throw a ModelRuntimeException.
 * 
 * @author Gerald Schrempf
 * 
 */

public class FederationModel extends AbstractModel implements IFederationModel {
    private static final long serialVersionUID = 1L;

    /**
     * stores the URIs of the spaces of this federation.
     */
    final Set<ISpaceModel> federationSpaces;

    /**
     * stores the URI of this Federation.
     */
    final URI federationURI;

    public FederationModel(URI federationURI, Set<ISpaceModel> spaces) {
        this.federationURI = federationURI;
        federationSpaces = spaces;
    }

    @Override
    public void addStatement(Resource arg0, URI arg1, Node arg2) throws ModelRuntimeException {
    	ModelHelper.throwOperationNotSupportedException();
    }

    @Override
    public void removeStatement(Resource arg0, URI arg1, Node arg2) throws ModelRuntimeException {
    	ModelHelper.throwOperationNotSupportedException();
    }

    public void dump() {
        for (ISpaceModel spaceModel : federationSpaces) {
            spaceModel.dump();
        }
    }

    public URI getContextURI() {
        return federationURI;
    }

    public boolean isIsomorphicWith(Model arg0) {
    	ModelHelper.throwOperationNotSupportedException();
        return false;
    }

    public BlankNode createBlankNode() {
    	ModelHelper.throwOperationNotSupportedException();
        return null;
    }

    public BlankNode createBlankNode(String arg0) {
    	ModelHelper.throwOperationNotSupportedException();
        return null;
    }

    public boolean isValidURI(String arg0) {
    	ModelHelper.throwOperationNotSupportedException();
        return false;
    }

    public ClosableIterator<Statement> iterator() {
    	ModelHelper.throwOperationNotSupportedException();
        return null;
    }

    public boolean isLocked() {
    	ModelHelper.throwOperationNotSupportedException();
        return false;
    }

    public void lock() throws LockException {
    	ModelHelper.throwOperationNotSupportedException();
    }

    public void unlock() {
    	ModelHelper.throwOperationNotSupportedException();
    }

    /**
     * merge the results of all subspaces.
     */
    public ClosableIterator<Statement> findStatements(ResourceOrVariable arg0, UriOrVariable arg1,
            NodeOrVariable arg2) throws ModelRuntimeException {
        return ModelHelper.findStatements(federationSpaces, arg0, arg1, arg2, null);
    }

    /**
     * merge the results of all subspaces.
     */
    public ClosableIterable<Statement> sparqlConstruct(String query) throws ModelRuntimeException,
            MalformedQueryException {
        return ModelHelper.sparqlConstruct(federationSpaces, query, null);
    }

    /**
     * merge the results of all subspaces.
     */
    public ClosableIterable<Statement> sparqlDescribe(String query) throws ModelRuntimeException {
        return ModelHelper.sparqlDescribe(federationSpaces, query, null);
    }

    /**
     * merge the results of all subspaces.
     */
    public QueryResultTable sparqlSelect(String query) throws MalformedQueryException,
            ModelRuntimeException {
        return ModelHelper.sparqlSelect(federationSpaces, query, null);
    }
    
    @Override
    public boolean sparqlAsk(String query) throws ModelRuntimeException, MalformedQueryException {
        return ModelHelper.sparqlAsk(federationSpaces, query);
    }

    public void readFrom(Reader arg0) throws IOException, ModelRuntimeException {
    	ModelHelper.throwOperationNotSupportedException();
    }

    public void readFrom(InputStream arg0) throws IOException, ModelRuntimeException {
    	ModelHelper.throwOperationNotSupportedException();
    }

    public void readFrom(Reader arg0, Syntax arg1) throws IOException, ModelRuntimeException {
    	ModelHelper.throwOperationNotSupportedException();
    }

    public void writeTo(Writer arg0) throws IOException, ModelRuntimeException {
    	ModelHelper.throwOperationNotSupportedException();
    }

    public void writeTo(OutputStream arg0) throws IOException, ModelRuntimeException {
    	ModelHelper.throwOperationNotSupportedException();
    }

    public void writeTo(Writer arg0, Syntax arg1) throws IOException, ModelRuntimeException {
    	ModelHelper.throwOperationNotSupportedException();
    }

    public String getNamespace(String arg0) {
        ModelHelper.throwOperationNotSupportedException();
        return null;
    }

    public Map<String, String> getNamespaces() {
    	ModelHelper.throwOperationNotSupportedException();
        return null;
    }

    public void removeNamespace(String arg0) {
    	ModelHelper.throwOperationNotSupportedException();
    }

    public void setNamespace(String arg0, String arg1) throws IllegalArgumentException {
    	ModelHelper.throwOperationNotSupportedException();
    }
}
