package at.sti2.semanticspaces.impl.model;

import java.util.Iterator;

import org.ontoware.aifbcommons.collection.ClosableIterator;

/**
 * flattens many ClosableIterators to one ClosableIterator.
 * 
 * @author Gerald Schrempf
 * 
 * @param <T>
 */
public class FlattenClosableIterator<T> implements ClosableIterator<T> {
	private Iterator<ClosableIterator<T>> iterator;
	private ClosableIterator<T> currentIterator;

	public FlattenClosableIterator(Iterator<ClosableIterator<T>> iterators) {
		this.iterator = iterators;
		currentIterator = null;
	}

	/**
	 * moves to next iterator, if actual one is empty.
	 */
	public boolean hasNext() {
		// currentIterator.hasNext()
		if (currentIterator != null && currentIterator.hasNext()) {
			return true;
		}
		
		// iterator.hasNext() and currentIterator.hasNext()
		while(iterator != null && iterator.hasNext()){
			closeCurrentIterator();
			currentIterator = iterator.next();
			
			if(currentIterator != null && currentIterator.hasNext()){
				return true;
			}
		}
		closeCurrentIterator(); // close last ClosableIterator
		return false;
	}

	public T next() {
		return currentIterator.next();
	}

	public void remove() {
		currentIterator.remove();
	}

	@Override
	public void close() {
		closeCurrentIterator();
		while (iterator != null && iterator.hasNext()) {
			currentIterator = iterator.next();
			closeCurrentIterator();
		}
	}

	private void closeCurrentIterator() {
		if (currentIterator != null) {
			currentIterator.close();
		}
	}
}