/*
 * Copyright (C) 2010
 * 
 * STI Innsbruck ICT - Technologie Park Innsbruck, 2nd Floor Technikerstrasse
 * 21a A-6020 Innsbruck, Austria
 * 
 * Contact: reto.krummenacher@sti2.at gerald.schrempf@sti2.at (development)
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

package at.sti2.semanticspaces.test;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;

import org.junit.AfterClass;
import org.junit.Test;
import org.ontoware.aifbcommons.collection.ClosableIterator;
import org.ontoware.rdf2go.RDF2Go;
import org.ontoware.rdf2go.exception.ModelRuntimeException;
import org.ontoware.rdf2go.model.Model;
import org.ontoware.rdf2go.model.QueryRow;
import org.ontoware.rdf2go.model.Statement;
import org.ontoware.rdf2go.model.impl.TriplePatternImpl;
import org.ontoware.rdf2go.model.node.BlankNode;
import org.ontoware.rdf2go.model.node.LanguageTagLiteral;
import org.ontoware.rdf2go.model.node.PlainLiteral;
import org.ontoware.rdf2go.model.node.URI;
import org.ontoware.rdf2go.model.node.impl.URIImpl;
import org.ontoware.rdf2go.vocabulary.XSD;

import at.sti2.semanticspaces.api.ISemanticSpace;
import at.sti2.semanticspaces.api.ISemanticSpaceListener;
import at.sti2.semanticspaces.api.ISpaceModel;
import at.sti2.semanticspaces.api.enums.SpaceModelType;
import at.sti2.semanticspaces.api.enums.SpaceRelation;
import at.sti2.semanticspaces.api.exceptions.OperationNotSupportedException;
import at.sti2.semanticspaces.api.exceptions.SemanticSpaceException;
import at.sti2.semanticspaces.api.exceptions.SpaceAlreadyExistsException;
import at.sti2.semanticspaces.impl.SemanticSpace;

/**
 * provides tests for the {@link SemanticSpace}.
 * use the different test-classes for OWLIM, Sesame, Overlay separately!
 * 
 * @author Gerald Schrempf
 * 
 */
public abstract class TestSemanticSpace {
	private static final String SPARQL_SELECT = "Select * where {?s ?p ?o}";

	private static final URI STI2_HAS_NAME = new URIImpl("http://sti2.at#has_name");

	private static final URI STI2_AT = new URIImpl("http://sti2.at");

	protected static SpaceModelType spaceModelType = null;
	
	protected static ISemanticSpace semanticSpace = null;

    protected static URI testSpace = null;

    protected static URI testSpace2 = null;

    protected static URI testSpace3 = null;

    protected static URI testSpaceChild = null;

    protected static URI testSpaceRest = null;

    protected static String foafURI = null;

    protected static URI max = null;

    protected static URI name = null;

    protected static URI typeInteger = null;

    protected static BlankNode konrad = null;

    protected static PlainLiteral maxNameAsPlainLiteral = null;

    protected static LanguageTagLiteral konradNameEnglish = null;

    protected static LanguageTagLiteral konradNameGerman = null;

    protected static Statement maxNameStatement = null;

    protected static Statement konradNameEnglishStatement = null;

    protected static Statement konradNameGermanStatement = null;
    
    protected static Statement winklerName = null;

    protected static ArrayList<Statement> konradNameStatements = null;
    

    // Deletes all files and subdirectories under dir.
    // Returns true if all deletions were successful.
    // If a deletion fails, the method stops attempting to delete and returns
    // false.
    public static boolean deleteDir(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            for (int i = 0; i < children.length; i++) {
                boolean success = deleteDir(new File(dir, children[i]));
                if (!success) {
                    return false;
                }
            }
        }

        // The directory is now empty, delete it
        return dir.delete();
    }

	protected static void preInitialize() throws InterruptedException {
		Thread.sleep(300);

        // delete locks
//        new File("repositories/spaceslogic/spaceslogic/lock").delete();
//        new File("repositories/spaces/spaces/lock").delete();
//        new File("store").renameTo(new File("store" + UUID.randomUUID()));
        
        // delete test-repository of SemanticSpaceImpl
        deleteDir(new File("/tmp/space-repository"));
	}
	
	protected static void postInitialize() throws SemanticSpaceException {
        testSpace = new URIImpl("http://example.com/testspace");
        testSpace2 = new URIImpl("http://example.com/testspace2");
        testSpace3 = new URIImpl("http://example.com/testspace3");
        testSpaceChild = new URIImpl("http://example.com/testspaceChild");
        testSpaceRest = new URIImpl("http://example.com/testspaceRest");

        Model model = RDF2Go.getModelFactory().createModel();
        
        foafURI = "http://xmlns.com/foaf/0.1/";
        max = new URIImpl("http://xam.de/foaf.rdf.xml#i");
        name = new URIImpl(foafURI + "#term_name");
        
        typeInteger = XSD._integer;
        maxNameAsPlainLiteral = model.createPlainLiteral("Max Vlkel");
        konradNameEnglish = model.createLanguageTagLiteral("Konrad Woelkel", "en");
        konradNameGerman = model.createLanguageTagLiteral("Konrad Wlkel", "de");
        maxNameStatement = model.createStatement(max, name, maxNameAsPlainLiteral);
        konrad = model.createBlankNode();
        konradNameEnglishStatement = model.createStatement(konrad, name, konradNameEnglish);
        konradNameGermanStatement = model.createStatement(konrad, name, konradNameGerman);
        konradNameStatements = new ArrayList<Statement>();
        konradNameStatements.add(konradNameGermanStatement);
        konradNameStatements.add(konradNameEnglishStatement);
	}
	
	/**
	 * cleanup resources.
	 * @throws SemanticSpaceException 
	 */
	@AfterClass
	public static void cleanup() throws SemanticSpaceException{
		// gets also deleted when tests are initialized.
		deleteDir(new File("/tmp/space-repository"));
		
		semanticSpace.getSpace(testSpace).close();
		semanticSpace.getSpace(testSpace2).close();
		semanticSpace.getSpace(testSpace3).close();
		semanticSpace.getSpace(testSpaceChild).close();
		semanticSpace.getSpace(testSpaceRest).close();
		semanticSpace.shutdown();
	}
    
    /**
     * creates the needed testSpaces for the following test cases. checks if the
     * spaces are really created.
     * 
     * @throws SemanticSpaceException
     */
    @Test
    public void test00_createSpaces() throws SemanticSpaceException {
    	int size = semanticSpace.listSpaces().size();
    	
        // create spaces
        semanticSpace.createSpace(testSpace, spaceModelType);
        semanticSpace.createSpace(testSpace2, spaceModelType);
        semanticSpace.createSpace(testSpace3, spaceModelType);
        semanticSpace.createSpace(testSpaceChild, spaceModelType, testSpace);
        semanticSpace.createSpace(testSpaceRest, spaceModelType);

        boolean defused = false;
        try {
            semanticSpace.createSpace(testSpace3, spaceModelType);
        } catch (SpaceAlreadyExistsException e) {
            defused = true;
        }
        assertTrue(defused);
        assertTrue(semanticSpace.listSpaces().size() == size + 5);
    }

    /**
     * tests the subscribe/unsubscribe-operations and functionality.
     * 
     * @throws SemanticSpaceException
     */
    @Test
    public void test02_subscribe_unsubscribe() throws SemanticSpaceException {
        final ArrayList<Statement> statements = new ArrayList<Statement>();

        // listener to get notification if statements are added to space
        ISemanticSpaceListener listener = new ISemanticSpaceListener() {
            public void notify(URI subscription, Statement statement) {
                statements.add(statement);
            }
        };

        // subscribe
        assertTrue(statements.size() == 0);
        ISpaceModel space3 = semanticSpace.getSpace(testSpace3);

        try{
	        URI subscription = space3.subscribe(SPARQL_SELECT, listener);
	        space3.addStatement(maxNameStatement);
	        assertTrue(statements.size() == 1);
	        space3.addStatement(foafURI, name, "Fried", "en");
	        assertTrue(statements.size() == 2);
	
	        // unsubscribe: no notification should be thrown
	        space3.unsubscribe(subscription);
	        space3.addStatement(konrad, name, "Konrad");
	        assertTrue(statements.size() == 2);
        }catch(OperationNotSupportedException e){
        	// subscription not possible for this model. no error.
        }
    }

    /**
     * adds statements to the spaces.
     * 
     * @throws SemanticSpaceException
     * @throws URISyntaxException
     */
    @Test
    public void test03_addValu() throws SemanticSpaceException, URISyntaxException {
        semanticSpace.getSpace(testSpace2).addAll(konradNameStatements.iterator());
        semanticSpace.getSpace(testSpace3).addStatement(maxNameStatement);
        semanticSpace.getSpace(testSpace3).addStatement(konrad, name, "Konrad");
        semanticSpace.getSpace(testSpace3).addStatement(foafURI, name, "Fried", "en");
        semanticSpace.getSpace(testSpaceChild).addStatement(foafURI, name, "Fried", typeInteger);
        semanticSpace.getSpace(testSpaceRest).addStatement(STI2_AT,
                STI2_HAS_NAME, "Winkler");
        semanticSpace.getSpace(testSpaceRest).addStatement(STI2_AT,
                STI2_HAS_NAME, "Winkler", "en");
        semanticSpace.getSpace(testSpaceRest).addStatement(STI2_AT,
                STI2_HAS_NAME, "Winkler", "^^Integer");

        // testSpace includes tuple of testSpaceChild!
        assertCountSpaceModel(testSpace, SPARQL_SELECT, 1);
        assertCountSpaceModel(testSpaceChild, SPARQL_SELECT, 1);
        assertCountSpaceModel(testSpace2, SPARQL_SELECT, 2);
        assertCountSpaceModel(testSpace3, SPARQL_SELECT, 3);
        assertCountSpaceModel(testSpaceRest, SPARQL_SELECT, 3);
    }

    /**
     * checks if the added Statements are contained in the spaces.
     * 
     * @throws SemanticSpaceException
     */
    @Test
    public void test04_contains() throws SemanticSpaceException {
        assertTrue(semanticSpace.getSpace(testSpace2).contains(konradNameGermanStatement));
        assertTrue(semanticSpace.getSpace(testSpace2).contains(konradNameEnglishStatement));
        assertTrue(semanticSpace.getSpace(testSpace3).contains(konrad, name, "Konrad"));
        assertTrue(semanticSpace.getSpace(testSpaceRest).contains(STI2_AT, STI2_HAS_NAME, "Winkler"));
        assertTrue(semanticSpace.getSpace(testSpaceRest).findStatements(
                new TriplePatternImpl(STI2_AT, STI2_HAS_NAME, "Winkler")).hasNext());
        assertFalse(semanticSpace.getSpace(testSpace).contains(maxNameStatement));
    }

    /**
     * tests the createFederation, getFederation and deleteFederation operations
     * and also the thrown exception if forbidden operations are called like
     * addStatement.
     * 
     * @throws SemanticSpaceException
     */
    @Test
    public void test05_federations() throws SemanticSpaceException {
        HashSet<URI> spaces = new HashSet<URI>();
        boolean exceptionThrown = false;

        assertCountSpaceModel(testSpace, SPARQL_SELECT, 1);
        assertCountSpaceModel(testSpace3, SPARQL_SELECT, 3);
        
        spaces.add(testSpace); // includes testSpaceChild, count = 1
        spaces.add(testSpace3);
        
        assertTrue(semanticSpace.getSpaces(spaces).size() == 2);
        URI fedURI = semanticSpace.createFederation(semanticSpace.getSpaces(spaces));
        assertTrue(semanticSpace.listFederations().size() == 1);

        assertCountFederation(fedURI, SPARQL_SELECT, 4);
        
        try {
            semanticSpace.getFederation(fedURI).addStatement(foafURI, name, "Friedlasd");
        } catch (Exception e) {
            exceptionThrown = true;
        }
        assertTrue(exceptionThrown);
        
        semanticSpace.getSpace(testSpace3).addStatement(STI2_AT, STI2_HAS_NAME, "Winkler2");
        assertCountFederation(fedURI, SPARQL_SELECT, 5);
        
        semanticSpace.getSpace(testSpace3).removeStatement(STI2_AT, STI2_HAS_NAME, "Winkler2");
        assertCountFederation(fedURI, SPARQL_SELECT, 4);
        
        semanticSpace.deleteFederation(fedURI);
        assertTrue(semanticSpace.getFederation(fedURI) == null);
        assertTrue(semanticSpace.listFederations().size() == 0);
    }

    /**
     * checks if the SpaceRelations in the metadataSpace are correctly created.
     * 
     * @throws SemanticSpaceException
     */
    @Test
    public void test06_spaceRelations() throws SemanticSpaceException {
        ISpaceModel testSpaceModel2 = semanticSpace.getSpace(testSpace2);
        ISpaceModel testSpaceModel3 = semanticSpace.getSpace(testSpace3);

        assertTrue(semanticSpace.getSubspaces(testSpace2).size() == 0);
        semanticSpace.createSpaceRelation(testSpaceModel2, SpaceRelation.tsonto_hasSubspace,
                testSpaceModel3);
        semanticSpace.createSpaceRelation(testSpaceModel2, SpaceRelation.tsonto_isRelatedTo,
                testSpaceModel3);
        semanticSpace.createSpaceRelation(testSpaceModel2, SpaceRelation.tsonto_isSimilarTo,
                testSpaceModel3);
        semanticSpace.createSpaceRelation(testSpaceModel2, SpaceRelation.tsonto_isSubspaceof,
                testSpaceModel3);
        semanticSpace.createSpaceRelation(testSpaceModel2, SpaceRelation.tsonto_seeAlsoSpace,
                testSpaceModel3);

        // testSpace2 hasSubspace testSpace3.
        assertTrue(semanticSpace.getSubspaces(testSpace2).size() == 1);
        assertCountSpaceModel(testSpace2, SPARQL_SELECT, 5);

        // testSpace2 alone.
        semanticSpace.removeSpaceRelation(testSpaceModel2, SpaceRelation.tsonto_hasSubspace,
                testSpaceModel3);
        assertCountSpaceModel(testSpace2, SPARQL_SELECT, 2);
        assertTrue(semanticSpace.getSubspaces(testSpace2).size() == 0);
        
        semanticSpace.removeSpaceRelation(testSpaceModel2, SpaceRelation.tsonto_isRelatedTo,
                testSpaceModel3);
        semanticSpace.removeSpaceRelation(testSpaceModel2, SpaceRelation.tsonto_isSimilarTo,
                testSpaceModel3);
        semanticSpace.removeSpaceRelation(testSpaceModel2, SpaceRelation.tsonto_isSubspaceof,
                testSpaceModel3);
        semanticSpace.removeSpaceRelation(testSpaceModel2, SpaceRelation.tsonto_seeAlsoSpace,
                testSpaceModel3);
    }

    /**
     * tests the sparqlConstruct operation.
     * 
     * @throws SemanticSpaceException
     */
    @Test
    public void test07_sparqlConstruct() throws SemanticSpaceException {
        String query = "Construct {?s ?p ?o} where { " + konrad + " <" + name + "> \"Konrad\" }";
        Integer i = countResults(semanticSpace.getSpace(testSpace).sparqlConstruct(query).iterator());
        assertTrue("count " + i + " should be " + 0, i == 0);

        i = countResults(semanticSpace.getSpace(testSpace3).sparqlConstruct(
                "Construct {?s ?p \"Konrad\"} where { ?s ?p \"Konrad\" }").iterator());
        assertTrue("count " + i + " should be " + 1, i == 1);

        i = countResults(semanticSpace.getSpace(testSpace).sparqlConstruct(
                "Construct {?s ?p ?o} where { ?s ?p ?o }").iterator());
        assertTrue("count " + i + " should be " + 1, i == 1);
    }

    /**
     * tests the sparqlDescribe operation.
     * 
     * @throws ModelRuntimeException
     * @throws SemanticSpaceException
     */
    @Test
    public void test08_sparqlDescribe() throws ModelRuntimeException, SemanticSpaceException {
        assertTrue(semanticSpace.getSpace(testSpace)
                .sparqlDescribe("Describe ?s where { ?s ?p ?o }").iterator().hasNext());
    }

    /**
     * tests the sparqlSelect operation.
     * 
     * @throws SemanticSpaceException
     */
    @Test
    public void test09_sparqlSelect() throws SemanticSpaceException {
    	String selectKonrad = "Select * where { " + konrad + " <" + name + "> \"Konrad\" }";
    	
    	assertCountSpaceModel(testSpace, selectKonrad, 0);
    	assertCountSpaceModel(testSpace3, selectKonrad, 1);
        assertCountSpaceModel(testSpace3, SPARQL_SELECT, 3);
        assertCountSpaceModel(testSpace, SPARQL_SELECT, 1);
    }

    /**
     * tests the sparqlAsk operation.
     * 
     * @throws SemanticSpaceException
     */
    @Test
    public void test10_sparqlAsk() throws SemanticSpaceException {
        assertFalse(semanticSpace.getSpace(testSpace).sparqlAsk(
                "ASK { " + konrad + " <" + name + "> \"Konrad\" }"));
        assertTrue(semanticSpace.getSpace(testSpace3).sparqlAsk(
                "ASK { " + konrad + " <" + name + "> \"Konrad\" }"));
        assertTrue(semanticSpace.getSpace(testSpace).sparqlAsk("ASK { ?s ?p ?o }"));
    }

    /**
     * tests the removeStatement and removeAll operations.
     * 
     * @throws SemanticSpaceException
     */
    @Test
    public void test11_removeValu() throws SemanticSpaceException {
        semanticSpace.getSpace(testSpace2).removeAll(konradNameStatements.iterator());
        semanticSpace.getSpace(testSpace3).removeStatement(maxNameStatement);
        semanticSpace.getSpace(testSpace3).removeStatement(konrad, name, "Konrad");
        semanticSpace.getSpace(testSpace3).removeStatement(foafURI, name, "Fried", "en");
        semanticSpace.getSpace(testSpaceChild).removeStatement(foafURI, name, "Fried", typeInteger);
        semanticSpace.getSpace(testSpaceRest).removeStatement(STI2_AT,
                STI2_HAS_NAME, "Winkler");
        semanticSpace.getSpace(testSpaceRest).removeStatement(STI2_AT,
                STI2_HAS_NAME, "Winkler", "en");
        semanticSpace.getSpace(testSpaceRest).removeStatement(STI2_AT,
                STI2_HAS_NAME, "Winkler", "^^Integer");

        assertCountSpaceModel(testSpace, SPARQL_SELECT, 0);
        assertCountSpaceModel(testSpace3, SPARQL_SELECT, 0);
        assertCountSpaceModel(testSpaceChild, SPARQL_SELECT, 0);
        assertCountSpaceModel(testSpaceRest, SPARQL_SELECT, 0);

        semanticSpace.getSpace(testSpace3).addStatement(maxNameStatement);
        assertCountSpaceModel(testSpace3, SPARQL_SELECT, 1);
        
        semanticSpace.getSpace(testSpace3).removeAll();
        assertCountSpaceModel(testSpace3, SPARQL_SELECT, 0);
    }

    /**
     * test if SpaceModelType is correctly converted between URI and
     * SpaceModelType. test method for SpaceModelType.valueOf(typeStr).
     * 
     * @throws SemanticSpaceException
     */
    @Test
    public void test12_SpaceModelType() throws SemanticSpaceException {
        assertTrue(SpaceModelType.getSpaceType(SpaceModelType.LocalRepository.getURI()) == SpaceModelType.LocalRepository);
        assertTrue(SpaceModelType.getSpaceType(SpaceModelType.DistributedSpace.getURI()) == SpaceModelType.DistributedSpace);
        assertTrue(SpaceModelType.getSpaceType(SpaceModelType.LOD.getURI()) == SpaceModelType.LOD);

        boolean exceptionThrown = false;
        try {
            URI errorURI = new URIImpl(SpaceModelType.LocalRepository.getURI().toString() + "abc");
            SpaceModelType.getSpaceType(errorURI);
            assertTrue(false);
        } catch (Exception e) {
            exceptionThrown = true;
        }
        assertTrue(exceptionThrown);
    }

    /**
     * test if SpaceRelation is correctly converted between URI and
     * SpaceRelation.
     * 
     * @throws SemanticSpaceException
     */
    @Test
    public void test13_SpaceRelation() throws SemanticSpaceException {
        assertTrue(SpaceRelation.getSpaceRelation(SpaceRelation.tsonto_hasSubspace.getURI()) == SpaceRelation.tsonto_hasSubspace);
    }

    /**
     * tests the convertion between URI and SpaceModelType.
     * 
     * @throws SemanticSpaceException
     */
    @Test
    public void test14_SpaceModelTypes() throws SemanticSpaceException {
        assertTrue(SpaceModelType.getSpaceType(SpaceModelType.DistributedSpace.getURI()) == SpaceModelType.DistributedSpace);
    }

    /**
     * counts the entries of the iterator.
     * 
     * @param iterator
     * @return
     */
    protected static int countResults(Iterator<?> iterator) {
        int i = 0;
        while (iterator.hasNext()) {
            iterator.next();
            i++;
        }
        return i;
    }
    
	/**
	 * prints query results.
	 * 
	 * @param iterator
	 */
	protected static void printResult(String query, ClosableIterator<QueryRow> iterator) {
		System.err.println(query);
		while (iterator.hasNext()) {
			System.err.println(printQueryRow(iterator.next()));
		}
		System.err.println("---------------------------------------------\n");
	}

	private static String printQueryRow(QueryRow row) {
		String text = row.getValue("s") + " ... "
			+ row.getValue("p") + " ... "
			+ row.getValue("o");
		return text;
	}


    /**
     * checks number of statements of a space using a SPARQL select query. 
     * assertTrue is called internally.
     * 
     * @param spaceURI space for querying
     * @param count count of expected statements
     * @throws SemanticSpaceException
     */
	protected void assertCountSpaceModel(URI spaceURI, String query, int count) throws SemanticSpaceException {
		assertResultCount(semanticSpace.getSpace(spaceURI), query, count);
	}

	
	protected void assertCountFederation(URI fedURI, String query, int count){
		assertResultCount(semanticSpace.getFederation(fedURI), query, count);
	}
	
	protected void assertResultCount(Model model, String query, int count) {
		int i = countResults(model.sparqlSelect(query).iterator());
        assertTrue("count " + i + " should be " + count, i == count);
	}
}
