/*
 * Copyright Aduna (http://www.aduna-software.com/) (c) 1997-2007.
 *
 * Licensed under the Aduna BSD-style license.
 * 
 * original source code from:
 * http://semweb4j.googlecode.com/svn/trunk/org.semweb4j.rdf2go.impl.sesame23/src/main/java/org/openrdf/rdf2go/RepositoryModelFactory.java
 * 
 */

package at.sti2.semanticspaces.sesame;

import java.io.File;
import java.util.Properties;

import org.ontoware.rdf2go.Reasoning;
import org.ontoware.rdf2go.exception.ModelRuntimeException;
import org.ontoware.rdf2go.impl.AbstractModelFactory;
import org.ontoware.rdf2go.model.Model;
import org.ontoware.rdf2go.model.ModelSet;
import org.ontoware.rdf2go.model.node.URI;
import org.openrdf.rdf2go.RepositoryModel;
import org.openrdf.rdf2go.RepositoryModelSet;
import org.openrdf.repository.Repository;
import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.sail.SailRepository;
import org.openrdf.sail.Sail;
import org.openrdf.sail.inferencer.fc.ForwardChainingRDFSInferencer;
import org.openrdf.sail.memory.MemoryStore;
import org.openrdf.sail.nativerdf.NativeStore;

/**
 * 
 * Added the Storage-Functionality for choosing between a NativeStore and a
 * MemoryStore like defined in the RDF2Go.ModelFactory interface.
 * 
 * If the Storage-property is empty, a MemoryStore is created. Otherwise, the
 * data is stored persistently with a NativeStore to the specified Storage-File.
 * 
 * @author Gerald Schrempf
 * 
 */

public class SesameModelFactory extends AbstractModelFactory {

	public Model createModel(Properties properties)
			throws ModelRuntimeException {
		return new RepositoryModel(createRepository(properties));
	}

	public Model createModel(URI contextURI) throws ModelRuntimeException {
		return new RepositoryModel(contextURI, createRepository(null));
	}

	public ModelSet createModelSet(Properties properties)
			throws ModelRuntimeException {
		return new RepositoryModelSet(createRepository(properties));
	}

	private Repository createRepository(Properties properties)
			throws ModelRuntimeException {
		// find out if we need reasoning
		String reasoningProperty = properties == null ? null : properties
				.getProperty(REASONING);
		boolean reasoning = Reasoning.rdfs.toString().equalsIgnoreCase(
				reasoningProperty);

		// read Storage property. create a Sail stack
		Sail sail;
		String storageProperty = properties.getProperty(STORAGE);

		if (storageProperty == null) {
			sail = new MemoryStore();
			if (reasoning) {
				sail = new ForwardChainingRDFSInferencer((MemoryStore) sail);
			}
		} else {
			sail = new NativeStore(new File(storageProperty));
		}

		// create a Repository
		Repository repository = new SailRepository(sail);
		try {
			repository.initialize();
		} catch (RepositoryException e) {
			throw new ModelRuntimeException(e);
		}

		return repository;
	}
}
